/*
 * The information contained in this file is confidential and proprietary to
 * Broadcom Corporation.  No part of this file may be reproduced or
 * distributed, in any form or by any means for any purpose, without the
 * express written permission of Broadcom Corporation.
 *
 * (c) COPYRIGHT 2010-2012 Broadcom Corporation, ALL RIGHTS RESERVED.
 */

#pragma ident "(C) 2010-2012 Broadcom Corporation"
#pragma ident " $File$$Revision$$Change$$DateTime$"
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/stat.h>
#include <sys/pci.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>
#include <sys/cmn_err.h>
#include <sys/file.h>
#include <sys/cred.h>
#include <sys/byteorder.h>
#include <sys/atomic.h>
#include <sys/scsi/scsi.h>
#include <sys/modhash.h>
#include <sys/sunldi.h>
#include <sys/kstat.h>
#include <sys/kmem.h>
#include <sys/ethernet.h>
#include <sys/stream.h>
#include <sys/sunldi.h>
/*
 * leadville header files
 */
#include <sys/fibre-channel/fc_types.h>
#include <sys/fibre-channel/impl/fc_fcaif.h>
#include <bnxe_binding.h>
#include <bnxef_tq_param.h>
#include <bnxef_fcf_info.h>
#include <bnxef_hsi.h>
#include <bnxef_fcoe_task.h>
#include <bnxef_fcoe_session.h>
#include <bnxef_state.h>
#include <bnxef_fip.h>
#include <bnxef_fcoe.h>
#include <bnxef_debug.h>

/*
 * The following handles the FCF related utility routines
 */

/*
 * FCF Cache constructor
 */
/*ARGSUSED*/
int
bnxef_fcf_cache_constructor(void *bufp, void *args, int flags)
{
	char  name[KSTAT_STRLEN];
	bnxef_fcf_info_t *fcfp = (bnxef_fcf_info_t *)bufp;
	bnxef_driver_state_t *dsp = (bnxef_driver_state_t *)args;


	bzero((uint8_t *)bufp, sizeof (bnxef_fcf_info_t));
	(void) snprintf(name, KSTAT_STRLEN, "fcf%d_mutex", dsp->fcf_ref_cnt++);
	mutex_init(&fcfp->fcf_mutex, name, MUTEX_DRIVER, NULL);

	return (0);
}

/*
 * The FCF chache destructor
 */

void
bnxef_fcf_cache_destructor(void *buf, void *args)
{
	bnxef_driver_state_t  *dsp = (bnxef_driver_state_t *)args;
	bnxef_fcf_info_t *fcfp = (bnxef_fcf_info_t *)buf;

	ASSERT(dsp != NULL);
	ASSERT(fcfp != NULL);

	mutex_destroy(&fcfp->fcf_mutex);

}


/*
 * THis is only called from Attach, therefore no locks are held
 */
int
bnxef_fcf_cache_create(bnxef_driver_state_t *dsp)
{
	char  buf[KSTAT_STRLEN];

	ASSERT(dsp != NULL);

	(void) snprintf(buf, KSTAT_STRLEN, "fcf%d_cache", dsp->instance);
	dsp->fcf_cache = kmem_cache_create(buf, sizeof (bnxef_fcf_info_t),
		8, bnxef_fcf_cache_constructor, bnxef_fcf_cache_destructor,
		NULL, dsp, 0, KM_SLEEP);

	if (dsp->fcf_cache == NULL) {
		cmn_err(CE_PANIC, "%s%d Unable to allocate fcf cache\n",
		    BNXEF_DRIVER_NAME, dsp->instance);
		return (DDI_FAILURE);
	}

	return (DDI_SUCCESS);
}

int
bnxef_fcf_cache_destroy(bnxef_driver_state_t *dsp)
{
	ASSERT(dsp != NULL);

	kmem_cache_destroy(dsp->fcf_cache);

	return (DDI_SUCCESS);
}

/*
 * THis initializes the FCF list for subsequent use.
 */
int
bnxef_fcf_list_init(bnxef_driver_state_t *dsp)
{
	bnxef_fcf_list_t *fcflstp;
	char  name[KSTAT_STRLEN];

	ASSERT(dsp != NULL);

	fcflstp = &dsp->fcf_list;

	bzero((uint8_t *)fcflstp, sizeof (bnxef_fcf_list_t));
	snprintf(name, KSTAT_STRLEN, "fcflist%dmutex", dsp->instance);
	mutex_init(&fcflstp->fcflist_mutex, name, MUTEX_DRIVER, NULL);

	return (DDI_SUCCESS);
}

/*
 * This will get the first item from the list. Explict remove must be called
 * following this, if it need to be removed from the list.
 */
bnxef_fcf_info_t *
bnxef_fcf_get_first(bnxef_driver_state_t *dsp)
{
	bnxef_fcf_info_t *fcfp;

	mutex_enter(&dsp->fcf_list.fcflist_mutex);
	fcfp = dsp->fcf_list.head;
	mutex_exit(&dsp->fcf_list.fcflist_mutex);

	return (fcfp);
}

/*
 * THis routine will destroy the fcf_list mutex and return success.
 */
int
bnxef_fcf_list_fini(bnxef_driver_state_t *dsp)
{
	bnxef_fcf_list_t	 *fcflstp;
	bnxef_fcf_info_t	*fcfp;
	uint32_t num_items;
	int i, ret;

	ASSERT(dsp != NULL);

	fcflstp = &dsp->fcf_list;
	mutex_enter(&fcflstp->fcflist_mutex);
	num_items = fcflstp->num_items;
	mutex_exit(&fcflstp->fcflist_mutex);
	for (i = 0; i < num_items; i++) {
		fcfp = bnxef_fcf_get_first(dsp);
		ASSERT(fcfp != NULL);
		ret = bnxef_fcf_list_remove(dsp, fcfp);
		ASSERT(ret == BNXEF_SUCCESS);
		kmem_cache_free(dsp->fcf_cache, fcfp);
	}

	mutex_destroy(&fcflstp->fcflist_mutex);

	return (DDI_SUCCESS);
}

void
bnxef_fcf_list_purge(bnxef_driver_state_t *dsp)
{
	bnxef_fcf_list_t	*fcflstp;
	bnxef_fcf_info_t	*fcfp;
	int ret, i;
	uint32_t num_items;

	ASSERT(dsp != NULL);

	fcflstp = &dsp->fcf_list;

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Purging FCF List Num Items 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, fcflstp->num_items);
	num_items = fcflstp->num_items;
	for (i = 0; i < num_items; i++) {
		fcfp = bnxef_fcf_get_first(dsp);
		ASSERT(fcfp != NULL);
		ret = bnxef_fcf_list_remove(dsp, fcfp);
		ASSERT(ret == BNXEF_SUCCESS);
		kmem_cache_free(dsp->fcf_cache, fcfp);
	}
}

/*
 * The following finds an FCF MAC with the given MAC Address
 */
bnxef_fcf_info_t *
bnxef_fcf_list_find_mac_addr(bnxef_driver_state_t *dsp, uint8_t *mac_addr)
{
	bnxef_fcf_info_t *fcfp;
	bnxef_fcf_list_t *fcflstp;
	int found = 0, i, ret;

	ASSERT(dsp != NULL);
	ASSERT(mac_addr != NULL);

	fcflstp = &dsp->fcf_list;
	mutex_enter(&fcflstp->fcflist_mutex);

	fcfp = fcflstp->head;
	if (fcfp == NULL) {
		mutex_exit(&fcflstp->fcflist_mutex);
		return (NULL);
	}
	ASSERT(fcfp != NULL);
	for (i = 0; i < fcflstp->num_items; i++) {
		ASSERT(fcfp != NULL);
		ret = bcmp(fcfp->mac_addr, mac_addr, 6);
		if (ret == 0) {
			found++;
			break;
		}
		fcfp = fcfp->next;
	}

	mutex_exit(&fcflstp->fcflist_mutex);
	return (found? fcfp: 0);
}

/*
 * Get the item which has the least priority Number.
 */
bnxef_fcf_info_t *
bnxef_identify_highest_priority_fcf(bnxef_driver_state_t *dsp)
{
	bnxef_fcf_info_t *fcfp, *retfcfp;
	bnxef_fcf_list_t *fcflstp;
	int i;
	uint8_t priority = 0xff;

	ASSERT(dsp != NULL);

	fcflstp = &dsp->fcf_list;
	mutex_enter(&fcflstp->fcflist_mutex);

	fcfp = fcflstp->head;
	if (fcfp == NULL) {
		mutex_exit(&fcflstp->fcflist_mutex);
		return (NULL);
	}
	ASSERT(fcfp != NULL);
	retfcfp = fcflstp->head;
	for (i = 0; i < fcflstp->num_items; i++) {
		ASSERT(fcfp != NULL);
		if ((fcfp->priority < priority) &&
			(fcfp->fcf_state &
				FCF_INFO_VLAN_NOTIF_RECEIVED)) {
			priority = fcfp->priority;
			retfcfp = fcfp;
		}
		fcfp = fcfp->next;
	}

	mutex_exit(&fcflstp->fcflist_mutex);
	return (retfcfp);
}



/*
 * This will add the given fcf pointer to the fcf list at the end.
 */
int
bnxef_fcf_list_add(bnxef_driver_state_t *dsp, bnxef_fcf_info_t *fcfp)
{

	bnxef_fcf_list_t *fcflstp;

	ASSERT(dsp != NULL);
	ASSERT(fcfp != NULL);

	fcflstp = &dsp->fcf_list;
	BNXEF_DEBUG(dsp, FIP, -1, "%s%d: Trying to add FCF in the list",
		BNXEF_DRIVER_NAME, dsp->instance);

	mutex_enter(&fcflstp->fcflist_mutex);

	if ((fcflstp->head == NULL)&&(fcflstp->tail == NULL)) {
		/* The list is empty, insert this element as first elememnt */
		fcflstp->head = fcfp;
		fcflstp->tail = fcfp;
	} else {
		ASSERT(fcflstp->tail != NULL);
		ASSERT(fcflstp->tail->next == NULL);
		fcflstp->tail->next = fcfp;
		fcflstp->tail = fcfp;
	}
	fcflstp->num_items++;
	mutex_exit(&fcflstp->fcflist_mutex);
	BNXEF_DEBUG(dsp, FIP, -1, "%s%d: FCF added to list, num_items 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance);
	return (BNXEF_SUCCESS);
}

/*
 * Remove the given fcf from the fcf list
 */

int
bnxef_fcf_list_remove(bnxef_driver_state_t *dsp, bnxef_fcf_info_t *fcfp)
{
	bnxef_fcf_list_t *fcflstp;
	bnxef_fcf_info_t *fcfcp; /* Current FCF while walking */
	bnxef_fcf_info_t *fcfprevp;	/* Previous FCF while walking */
	int i, found = 0;
	int ret  = BNXEF_FAILURE;

	ASSERT(dsp != NULL);
	ASSERT(fcfp != NULL);

	fcflstp = &dsp->fcf_list;

	ASSERT(fcflstp->num_items > 0);
	mutex_enter(&fcflstp->fcflist_mutex);
	fcfcp = fcflstp->head;
	ASSERT(fcfcp != NULL);
	fcfprevp = NULL;
	for (i = 0; i < fcflstp->num_items; i++) {
		if (fcfcp == fcfp) {
			found++;
			break;
		}
		fcfprevp = fcfcp;
		fcfcp = fcfcp->next;
		ASSERT(fcfcp != NULL);
	}

	if (found) {
		if (fcflstp->num_items == 1) {
			/* There is only one item */
			fcflstp->head = NULL;
			fcflstp->tail = NULL;
		} else {
			if (fcfprevp == NULL) {
				if (fcfp == fcflstp->head) {
					fcflstp->head = fcfp->next;
				}
			} else
				fcfprevp->next = fcfp->next;
			if (fcflstp->tail == fcfp)
				fcflstp->tail = fcfprevp;
		}
		fcflstp->num_items--;
		ret = BNXEF_SUCCESS;
	}
	mutex_exit(&fcflstp->fcflist_mutex);

	return (ret);
}

/*
 * Get the least priority numbered FCF which will be the highest priority FCF
 * for operation. Eventually we can play with this to choose from management
 * software or through config file.
 */
void
bnxef_select_fcf(bnxef_driver_state_t *dsp)
{
	ASSERT(mutex_owned(&dsp->state_mutex));
	dsp->fcf_ident = bnxef_identify_highest_priority_fcf(dsp);
	mutex_enter(&dsp->fcf_ident->fcf_mutex);
	dsp->fcf_ident->fcf_state |= FCF_SELECTED_FOR_LOGIN;
	mutex_exit(&dsp->fcf_ident->fcf_mutex);
}
