/*
 * The information contained in this file is confidential and proprietary to
 * Broadcom Corporation.  No part of this file may be reproduced or
 * distributed, in any form or by any means for any purpose, without the
 * express written permission of Broadcom Corporation.
 *
 * (c) COPYRIGHT 2010-2012 Broadcom Corporation, ALL RIGHTS RESERVED.
 */

#pragma ident "(C) 2010-2012 Broadcom Corporation"
#pragma ident " $File$$Revision$$Change$$DateTime$"
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/stat.h>
#include <sys/pci.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>
#include <sys/cmn_err.h>
#include <sys/file.h>
#include <sys/cred.h>
#include <sys/byteorder.h>
#include <sys/atomic.h>
#include <sys/scsi/scsi.h>
#include <sys/modhash.h>
#include <sys/sunldi.h>
#include <sys/kstat.h>
#include <sys/kmem.h>
#include <sys/ethernet.h>
#include <sys/stream.h>
#include <sys/sunldi.h>
/*
 * leadville header files
 */
#include <sys/fibre-channel/fc_types.h>
#include <sys/fibre-channel/impl/fc_fcaif.h>
#include <bnxe_binding.h>
#include <bnxef_tq_param.h>
#include <bnxef_fcf_info.h>
#include <bnxef_hsi.h>
#include <bnxef_fcoe_task.h>
#include <bnxef_fcoe_session.h>
#include <bnxef_state.h>
#include <bnxef_fip.h>
#include <bnxef_fcoe.h>
#include <bnxef_debug.h>

extern ddi_dma_attr_t bnxef_dma_attr;
extern ddi_device_acc_attr_t bnxef_dev_acc_attr;
int bnxef_get_num_free_list_items(bnxef_driver_state_t *dsp);
int bnxef_task_bd_table_free(fc_packet_t *pktp, uint32_t stage);
int bnxef_task_session_list_find(bnxef_session_t *sessionp,
	uint16_t task_id);
int bnxef_task_bd_table_pre_free(bnxef_driver_state_t *dsp,
		bnxef_fcoe_task_t *taskp, uint32_t stage);
bnxef_fcoe_task_t *
bnxef_task_alloc(bnxef_driver_state_t *dsp, int sleepflag);
void
bnxef_fcoe_task_inc_ref_cnt(bnxef_fcoe_task_t *taskp);
void
bnxef_fcoe_task_dec_ref_cnt(bnxef_driver_state_t *dsp,
	bnxef_fcoe_task_t *taskp);
/*
 * The following provides the utility routines to allocate cache and also to
 * manage the free and session list given the session list head.
 */

/*
 * The following is the cache constructor for the Task structures.
 */
/* ARGSUSED */
int
bnxef_task_cache_constructor(void *bufp, void *args, int kmflags)
{

	char name[KSTAT_STRLEN];
	bnxef_fcoe_task_t *taskp = (bnxef_fcoe_task_t *)bufp;
	bnxef_driver_state_t *dsp = (bnxef_driver_state_t *)args;
	uint32_t refcnt;

	NOTE(ARGUNUSED(kmflags))

	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	refcnt = dsp->task_ref_cnt;
	dsp->task_ref_cnt++;
	mutex_exit(&dsp->state_mutex);

	bzero((uint8_t *)bufp, sizeof (bnxef_fcoe_task_t));
	(void) snprintf(name, KSTAT_STRLEN, "task%d-%d-mutex",
		dsp->instance, refcnt);
	mutex_init(&taskp->task_mutex, name, MUTEX_DRIVER, NULL);

	(void) snprintf(name, KSTAT_STRLEN, "task%d-%d-cv",
		dsp->instance, refcnt);
	cv_init(&taskp->task_cv, name, CV_DRIVER, NULL);

	return (0);
}


/*
 * Task cache destructor
 */
/*ARGSUSED*/
void
bnxef_task_cache_destructor(void *bufp, void *args)
{

	bnxef_fcoe_task_t *taskp  = (bnxef_fcoe_task_t *)bufp;

	NOTE(ARGUNUSED(args))

	mutex_destroy(&taskp->task_mutex);
	cv_destroy(&taskp->task_cv);

}

/*
 * Task Cache create routine. only called from attach routine.
 */
int
bnxef_task_cache_create(bnxef_driver_state_t *dsp)
{

	char buf[KSTAT_STRLEN];

	ASSERT(dsp != NULL);

	(void) snprintf(buf, KSTAT_STRLEN, "task%d_cache", dsp->instance);
	dsp->task_cache = kmem_cache_create(buf, sizeof (bnxef_fcoe_task_t),
		8, bnxef_task_cache_constructor, bnxef_task_cache_destructor,
		NULL, dsp, 0, KM_SLEEP);
	if (dsp->task_cache == NULL) {
		cmn_err(CE_PANIC, "%s%d Unable to allocate task cache\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		return (DDI_FAILURE);
	}

	return (DDI_SUCCESS);
}


/*
 * This function will destroy the cache created at attach time. This routine can
 * only be called from detach.
 */
int
bnxef_task_cache_destroy(bnxef_driver_state_t *dsp)
{

	ASSERT(dsp != NULL);

	kmem_cache_destroy(dsp->task_cache);

	return (DDI_SUCCESS);
}


/*
 * Initialize the free list header to instantiate free list
 */

int
bnxef_task_free_list_init(bnxef_driver_state_t *dsp)
{

	bnxef_task_list_t *freelstp;
	char	name[KSTAT_STRLEN];

	ASSERT(dsp != NULL);

	freelstp = &dsp->bnxef_task_free_list;
	bzero((uint8_t *)freelstp, sizeof (bnxef_task_list_t));
	snprintf(name, KSTAT_STRLEN, "taskfreelist%d-mutex", dsp->instance);
	mutex_init(&freelstp->tasklist_mutex, name, MUTEX_DRIVER, NULL);
	snprintf(name, KSTAT_STRLEN, "taskfreelist%d-cv", dsp->instance);
	cv_init(&freelstp->tasklist_cv, name, CV_DRIVER, NULL);
	return (DDI_SUCCESS);
}


/*
 * This routine will destroy the task list mutex and return success.
 */
int
bnxef_task_free_list_fini(bnxef_driver_state_t *dsp)
{

	bnxef_task_list_t *freelstp;

	ASSERT(dsp != NULL);

	freelstp = &dsp->bnxef_task_free_list;

	mutex_destroy(&freelstp->tasklist_mutex);
	cv_destroy(&freelstp->tasklist_cv);

	return (DDI_SUCCESS);
}


/*
 * This initializes the task list initiated on a session.
 */
int
bnxef_task_session_list_init(bnxef_driver_state_t *dsp,
		bnxef_session_t *sessionp)
{

	bnxef_task_list_t *tasklstp;
	char	name[KSTAT_STRLEN];


	ASSERT(dsp != NULL);
	ASSERT(sessionp != NULL);

	tasklstp = &sessionp->sessiontasklist;

	ASSERT(tasklstp != NULL);

	bzero(tasklstp, sizeof (bnxef_task_list_t));
	snprintf(name, KSTAT_STRLEN, "tasksessionlist%d-%dmutex",
			dsp->instance, sessionp->con_id);
	mutex_init(&tasklstp->tasklist_mutex, name, MUTEX_DRIVER, NULL);
	snprintf(name, KSTAT_STRLEN, "tasksessionlist%d-%dmutex",
			dsp->instance, sessionp->con_id);
	cv_init(&tasklstp->tasklist_cv, name, CV_DRIVER, NULL);


	return (DDI_SUCCESS);
}


/*
 * This will destroy the mutex in session task list.
 */

int
bnxef_task_session_list_fini(bnxef_session_t *sessionp)
{

	bnxef_task_list_t *tasklstp;
	bnxef_driver_state_t *dsp;
	bnxef_port_t	*portp;


	ASSERT(sessionp != NULL);
	portp = sessionp->lport;
	ASSERT(portp != NULL);
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	tasklstp = &sessionp->sessiontasklist;

	mutex_destroy(&tasklstp->tasklist_mutex);
	cv_destroy(&tasklstp->tasklist_cv);

	return (DDI_SUCCESS);
}


/*
 * THis routine will return the current number of entries in free list.
 */
int
bnxef_get_num_free_list_items(bnxef_driver_state_t *dsp)
{
		bnxef_task_list_t *tasklstp;
		int num_items;


		ASSERT(dsp != NULL);
		tasklstp = &dsp->bnxef_task_free_list;

		mutex_enter(&tasklstp->tasklist_mutex);
		num_items = tasklstp->num_items;
		mutex_exit(&tasklstp->tasklist_mutex);

		return (num_items);
}

/*
 * This routine is called at the attach time to create a free list
 */
int
bnxef_task_list_add(bnxef_driver_state_t *dsp,
		bnxef_fcoe_task_t *taskp)
{

	bnxef_task_list_t	*tasklstp;

	ASSERT(dsp != NULL);
	ASSERT(taskp != NULL);
	tasklstp = &dsp->bnxef_task_free_list;

	mutex_enter(&tasklstp->tasklist_mutex);
	if ((tasklstp->head == NULL) && (tasklstp->tail == NULL)) {
		/* This list is empty, insert this elememt as first element */
		tasklstp->head = taskp;
		tasklstp->tail = taskp;
	} else {
		ASSERT(tasklstp->tail != NULL);
		ASSERT(tasklstp->tail->nextp == NULL);
		tasklstp->tail->nextp = taskp;
		tasklstp->tail = taskp;
	}
	tasklstp->num_items++;
	mutex_exit(&tasklstp->tasklist_mutex);
	return (BNXEF_SUCCESS);
}

/*
 * This will remove the first item from the list and return that entry.
 */

bnxef_fcoe_task_t *
bnxef_task_list_get_first(bnxef_driver_state_t *dsp)
{

	bnxef_fcoe_task_t *taskp;
	bnxef_task_list_t *tasklstp;

	ASSERT(dsp != NULL);

	tasklstp = &dsp->bnxef_task_free_list;

	mutex_enter(&tasklstp->tasklist_mutex);

	taskp = tasklstp->head;
	if (taskp != NULL) {
		tasklstp->head = taskp->nextp;
		if (tasklstp->head == NULL) {
			tasklstp->tail = NULL;
		}
		taskp->nextp = NULL;
		tasklstp->num_items--;
	} else {
		ASSERT(tasklstp->tail == NULL);
		ASSERT(tasklstp->num_items == 0);
	}
	mutex_exit(&tasklstp->tasklist_mutex);

	return (taskp);
}


bnxef_fcoe_task_t *
bnxef_rrq_task_alloc(bnxef_driver_state_t *dsp, int sleep)
{
		int num_items;
		bnxef_fcoe_task_t *taskp;
	bnxef_task_list_t *tasklstp;

	tasklstp = &dsp->bnxef_task_free_list;

	num_items = bnxef_get_num_free_list_items(dsp);
	do {
		if (num_items < (BNXEF_NUM_TASKS_ALLOWED_FOR_LV_IOS-0x80)) {
			mutex_enter(&tasklstp->tasklist_mutex);
				/* This will wait until some one frees a task */
				cv_wait(&tasklstp->tasklist_cv,
					&tasklstp->tasklist_mutex);
				mutex_exit(&tasklstp->tasklist_mutex);
			} else break;
		num_items = bnxef_get_num_free_list_items(dsp);
	}
	while (num_items < (BNXEF_NUM_TASKS_ALLOWED_FOR_LV_IOS-0x80));

		/* Now allocate a task structure from the free list for use */
		taskp = bnxef_task_alloc(dsp, sleep);
		if (taskp == NULL) {
			BNXEF_DEBUG(dsp, GENERAL, -1,
				"%s%d: Unable to allocate task structure\n",
				BNXEF_DRIVER_NAME, dsp->instance);
			return (NULL);
		}

	return (taskp);

}

/*
 * This will wait for the resources to be available before it allocates
 * the task. This will wait on the free list cv untill resources become
 * available if the sleep flag indicates sleeping is allowed, if no
 * sleeping is allowed this will task or NULL when no resources are available.
 */

bnxef_fcoe_task_t *
bnxef_task_alloc(bnxef_driver_state_t *dsp, int sleepflag)
{
	bnxef_fcoe_task_t *taskp;
	bnxef_task_list_t *tasklstp;
	int retry = 0;

	ASSERT(dsp != NULL);

	tasklstp = &dsp->bnxef_task_free_list;
	taskp = bnxef_task_list_get_first(dsp);
	if (taskp == NULL) {
		if (sleepflag == KM_NOSLEEP) {
			return (NULL);
		} else {
			do {
			BNXEF_AUTO_DEBUG("%s%d: Waiting for Release, retry %d",
				BNXEF_DRIVER_NAME, dsp->instance, retry);
				mutex_enter(&tasklstp->tasklist_mutex);
				/* This will wait until some one frees a task */
				cv_wait(&tasklstp->tasklist_cv,
					&tasklstp->tasklist_mutex);
				mutex_exit(&tasklstp->tasklist_mutex);
				retry++;
				taskp = bnxef_task_list_get_first(dsp);
				ASSERT(retry <= BNXEF_MAX_TASK_ALLOC_RETRY);
				} while (taskp == NULL);
		}
	}
	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_ALLOCATED;
	BNXEF_AUTO_DEBUG("%s%d:Task allocated 0x%p",
			BNXEF_DRIVER_NAME, dsp->instance, taskp);
	taskp->sessionp = NULL;
	taskp->rec_timeout_cnt = 0;
	taskp->tq_params.reserved2 = 0;
	taskp->tq_params.reserved3 = 0;
	taskp->ref_cnt = 1;
	mutex_exit(&taskp->task_mutex);
	return (taskp);
}

/*
 * The following will free the entry into free list.
 */
void
bnxef_task_free(void *arg, bnxef_fcoe_task_t *taskp)
{

	bnxef_driver_state_t *dsp = (bnxef_driver_state_t *)arg;
	bnxef_task_list_t *tasklstp;

	ASSERT(dsp != NULL);

	BNXEF_DEBUG(dsp, GENERAL, -1, "%s%d: Freeing the Entry 0x%p",
		BNXEF_DRIVER_NAME, dsp->instance, taskp);
	mutex_enter(&taskp->task_mutex);
	taskp->ref_cnt--;
	if (taskp->ref_cnt > 0)  {
		mutex_exit(&taskp->task_mutex);
		return;
	}
	taskp->task_flags &= ~BNXEF_FCOE_TASK_ALLOCATED;
	taskp->task_flags = 0; /* All flags should be off at this point */
	taskp->tq_params.reserved2 = 0;
	taskp->tq_params.reserved3 = 0;
	mutex_exit(&taskp->task_mutex);

	tasklstp = &dsp->bnxef_task_free_list;


	(void) bnxef_task_list_add(dsp, taskp);
	mutex_enter(&tasklstp->tasklist_mutex);
	cv_signal(&tasklstp->tasklist_cv);
	mutex_exit(&tasklstp->tasklist_mutex);

}

void
bnxef_fcoe_task_inc_ref_cnt(bnxef_fcoe_task_t *taskp)
{

	mutex_enter(&taskp->task_mutex);
	taskp->ref_cnt++;
	mutex_exit(&taskp->task_mutex);
}

void
bnxef_fcoe_task_dec_ref_cnt(bnxef_driver_state_t *dsp, bnxef_fcoe_task_t *taskp)
{
	mutex_enter(&taskp->task_mutex);
	taskp->ref_cnt--;
	if (taskp->ref_cnt > 0) {
		mutex_exit(&taskp->task_mutex);
		return;
	}
	mutex_exit(&taskp->task_mutex);

	bnxef_task_free(dsp, taskp);
}

/*
 * The following routines handle the inclusion of task and deletion of tasks
 * into session specific task list. This inclusion and deletion will be very
 * frequent and it is possible that in each hash bucket there could be on an
 * average 200 such entries and in worst case it could be more than 8K.
 * Therefore this session specific list is doubly linked so that we can unlink
 * the task from the list without walking the complete list.
 */
int
bnxef_task_session_list_add(bnxef_session_t *sessionp,
	bnxef_fcoe_task_t *taskp)
{
		bnxef_task_list_t *tasklstp;
		bnxef_driver_state_t *dsp;
		bnxef_port_t		*portp;

		ASSERT(sessionp != NULL);
		ASSERT(taskp != NULL);
		ASSERT(taskp->sessionp == NULL);
	portp = sessionp->lport;
	ASSERT(portp != NULL);
	dsp = portp->statep;
	ASSERT(dsp != NULL);

		    mutex_enter(&sessionp->session_mutex);
		    if (sessionp->state & BNXEF_SESSION_CLEANUP_INPROGRESS) {
		mutex_exit(&sessionp->session_mutex);
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Session Cleanup In progress, not queuing task 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
		return (BNXEF_FAILURE);
		    }
		    mutex_exit(&sessionp->session_mutex);

	if (bnxef_task_session_list_find(sessionp, taskp->x_id) == 1) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Duplicate TAsk Id found 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
		debug_enter("Found Duplicate X_ID in list");
	}

	tasklstp = &sessionp->sessiontasklist;

		mutex_enter(&tasklstp->tasklist_mutex);
		if (sessionp->state & BNXEF_SESSION_CLEANUP_INPROGRESS) {
		mutex_exit(&tasklstp->tasklist_mutex);
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Returning, session Cleanup in progress %p",
			BNXEF_DRIVER_NAME, dsp->instance, sessionp);
		return (BNXEF_FAILURE);
	}
		taskp->sessionp = sessionp;
		if ((tasklstp->head == NULL) && (tasklstp->tail == NULL)) {
		tasklstp->head = taskp;
		tasklstp->tail = taskp;
		taskp->sprevp = NULL;
		taskp->snextp = NULL;
		} else  {
			ASSERT(taskp->snextp == NULL);
			ASSERT(tasklstp->tail != NULL);
			if (tasklstp->tail->snextp != NULL) {
			BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Found the BAd entry 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
		}
			ASSERT(tasklstp->tail->snextp == NULL);
			tasklstp->tail->snextp = taskp;
			taskp->sprevp = tasklstp->tail;
			tasklstp->tail = taskp;
	}
	taskp->tq_params.reserved2++;
	tasklstp->num_items ++;
	mutex_exit(&tasklstp->tasklist_mutex);

	return (BNXEF_SUCCESS);
}


/*
 * This will return the first item to kick start the search.
 */
bnxef_fcoe_task_t *
bnxef_task_session_list_get_first(bnxef_session_t *sessionp)
{
		bnxef_task_list_t *tasklstp;
		bnxef_fcoe_task_t *taskp  = NULL;

		ASSERT(sessionp != NULL);

		tasklstp = &sessionp->sessiontasklist;
		mutex_enter(&tasklstp->tasklist_mutex);
		if (tasklstp->num_items > 0) {
		taskp = tasklstp-> head;
		ASSERT(taskp != NULL);
	}
	mutex_exit(&tasklstp->tasklist_mutex);
	return (taskp);
}

int
bnxef_task_session_list_get_num_items(bnxef_session_t *sessionp)
{
	int items;
	bnxef_task_list_t *tasklstp;

	tasklstp = &sessionp->sessiontasklist;
	mutex_enter(&tasklstp->tasklist_mutex);
	items = tasklstp->num_items;
	mutex_exit(&tasklstp->tasklist_mutex);

	return (items);
}

int
bnxef_task_session_list_find(bnxef_session_t *sessionp,
	uint16_t task_id)
{
	bnxef_task_list_t	*tasklstp;
	bnxef_fcoe_task_t	*taskp;
	int	found = 0;

	ASSERT(sessionp != NULL);
	tasklstp = &sessionp->sessiontasklist;
	mutex_enter(&tasklstp->tasklist_mutex);
	taskp = tasklstp->head;
	while (taskp != NULL) {
		if (taskp-> x_id == task_id) {
			found++;
			break;
		}
		taskp = taskp->snextp;
	}
	mutex_exit(&tasklstp->tasklist_mutex);

	return (found);
}

/*
 * This will return the number of active I/Os queued in task list of this
 * session.
 */
int
bnxef_get_active_ios(bnxef_session_t *sessionp)
{
	bnxef_task_list_t *tasklstp;
	int num_ios;

	ASSERT(sessionp != NULL);

	tasklstp = &sessionp->sessiontasklist;

	mutex_enter(&tasklstp->tasklist_mutex);
	num_ios = tasklstp->num_items;
	mutex_exit(&tasklstp->tasklist_mutex);

	return (num_ios);
}

/*
 * The following will delink the taskp pointer from the session list.
 */
void
bnxef_task_session_list_remove(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_task_list_t *tasklstp;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);
	ASSERT(taskp->sessionp == sessionp);


#if 0
		    mutex_enter(&sessionp->session_mutex);
		    if (taskp->task_flags & BNXEF_FCOE_TASK_ELS) {
	if (sessionp->els_pending) {
		sessionp->els_pending = 0;
		cv_signal(&sessionp->session_els);
	}
		    }
		    mutex_exit(&sessionp->session_mutex);
#endif
	tasklstp = &sessionp->sessiontasklist;

	mutex_enter(&tasklstp->tasklist_mutex);
	if (tasklstp->head == tasklstp->tail) {
		ASSERT(tasklstp->head == taskp);
		ASSERT(tasklstp->tail == taskp);
		tasklstp->head = tasklstp->tail = NULL;
		tasklstp->num_items--;
		taskp->sessionp = NULL;
		taskp->sprevp = NULL;
		taskp->snextp = NULL;
		taskp->tq_params.reserved3 ++;
		mutex_exit(&tasklstp->tasklist_mutex);
		return;
	}
	/* THis is the last items in the list */
	if (taskp->snextp == NULL) {
		ASSERT(taskp->sprevp != NULL);
		tasklstp->tail = taskp->sprevp;
		taskp->sprevp  = NULL;
		ASSERT(tasklstp->tail != NULL);
		tasklstp->tail->snextp = NULL;
	} else  if (taskp->sprevp == NULL) {
		/* This is the first item in the list. */
		ASSERT(taskp->snextp != NULL);
		tasklstp->head = taskp->snextp;
		ASSERT(tasklstp->head != NULL);
		tasklstp->head->sprevp = NULL;
		taskp->snextp = NULL;
	} else {
		/* This items is neither the first nor the last */
		taskp->sprevp->snextp = taskp->snextp;
		taskp->snextp->sprevp = taskp->sprevp;
		taskp->snextp = NULL;
		taskp->sprevp = NULL;
	}

	tasklstp->num_items--;
	taskp->tq_params.reserved3 ++;
	taskp->sessionp = NULL;
	mutex_exit(&tasklstp->tasklist_mutex);

}

/*
 * This will allocate the BD table for each I/O command to be sent down.
 * This is only used for middle path command where the command information
 * can only be sent using the Mul_sges. This gets freed when the I/O completes.
 */
int
bnxef_task_cmd_bd_table_alloc(bnxef_session_t *sessionp, fc_packet_t *pktp)
{
	bnxef_fcoe_task_t *taskp;
	ddi_dma_attr_t *bddmaattr = &bnxef_dma_attr;
	ddi_device_acc_attr_t *bdaccattr = &bnxef_dev_acc_attr;
	bnxef_port_t	*portp;
	bnxef_driver_state_t *dsp;
	int ret, stage = 0;

	ASSERT(pktp != NULL);
	ASSERT(sessionp != NULL);

	taskp = (bnxef_fcoe_task_t *)pktp->pkt_fca_private;
	portp = sessionp->lport;
	ASSERT(portp != NULL);
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	BNXEF_DEBUG(dsp, SCSI, portp->id,
		"%s%d: Allocating DMA Handle for CMD BD table for task 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
	ret = ddi_dma_alloc_handle(dsp->dip, bddmaattr, DDI_DMA_SLEEP, NULL,
		&taskp->bd_dma_handle);
	if (ret != DDI_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to allocate task BD dma handle\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		goto bnxef_task_cmd_bd_table_alloc_error;
	}

	stage |= BNXEF_TASK_BD_TABLE_DMA_HANDLE_ALLOCATED;
	BNXEF_DEBUG(dsp, SCSI, portp->id,
		"%s%d: Allocating CMD DMA memory of length 0x%x for task 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->bd_length,
			taskp->x_id);
	ret = ddi_dma_mem_alloc(taskp->bd_dma_handle,
		(pktp->pkt_cmd_cookie_cnt + pktp->pkt_resp_cookie_cnt)
			* sizeof (struct fcoe_bd_ctx),
		bdaccattr, DDI_DMA_CONSISTENT, DDI_DMA_SLEEP, NULL,
		&taskp->bd_addr, (unsigned long *)&taskp->bd_length,
		&taskp->bd_dev_acc);
	if (ret != DDI_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%s: Unable to allocate CMD DMA memory\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		goto bnxef_task_cmd_bd_table_alloc_error;
	}
	stage |= BNXEF_TASK_BD_TABLE_MEM_ALLOCATED;
	memset(taskp->bd_addr, 0, taskp->bd_length);
	BNXEF_DEBUG(dsp, SCSI, portp->id,
		"%s%d: Binding CMD DMA memory allocated for Task id 0%x",
				BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);

	ret = ddi_dma_addr_bind_handle(taskp->bd_dma_handle,
		NULL, taskp->bd_addr, taskp->bd_length,
		DDI_DMA_CONSISTENT | DDI_DMA_RDWR, DDI_DMA_SLEEP, NULL,
		&taskp->bd_cookie, &taskp->bd_cookie_cnt);
	if (ret != DDI_DMA_MAPPED) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to bind CMD dma mem with handle\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		goto bnxef_task_cmd_bd_table_alloc_error;
	}

	ASSERT(taskp->bd_cookie_cnt == 1);
	stage |= BNXEF_TASK_BD_TABLE_MEM_BOUND;
	taskp->task_flags |= BNXEF_FCOE_TASK_BD_LIST_ALLOC;
	BNXEF_DEBUG(dsp, SCSI, portp->id,
		"%s%d: CMD DMA resources allocated for task 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
	return (BNXEF_SUCCESS);

bnxef_task_cmd_bd_table_alloc_error:
	(void) bnxef_task_bd_table_free(pktp, stage);
	return (BNXEF_FAILURE);
}


int
bnxef_task_bd_table_pre_alloc(bnxef_driver_state_t *dsp,
		bnxef_fcoe_task_t *taskp)
{
	ddi_dma_attr_t *bddmaattr = &bnxef_dma_attr;
	ddi_device_acc_attr_t *bdaccattr = & bnxef_dev_acc_attr;
	int ret, stage = 0;


	ASSERT(dsp != NULL);
	ASSERT(taskp != NULL);

	BNXEF_DEBUG(dsp, SCSI, -1,
		"%s%d: Allocating DMA Handle for BD table for task 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
	ret = ddi_dma_alloc_handle(dsp->dip, bddmaattr, DDI_DMA_SLEEP, NULL,
		&taskp->bd_dma_handle);
	if (ret != DDI_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to allocate task BD dma handle\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		goto bnxef_task_bd_table_pre_alloc_error;
	}

	stage |= BNXEF_TASK_BD_TABLE_DMA_HANDLE_ALLOCATED;
	BNXEF_DEBUG(dsp, SCSI, -1,
		"%s%d: Allocating DMA memory of length 0x%x for task 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->bd_length,
			taskp->x_id);
	ret = ddi_dma_mem_alloc(taskp->bd_dma_handle,
		BNXEF_PAGE_SIZE,
		bdaccattr, DDI_DMA_CONSISTENT, DDI_DMA_SLEEP, NULL,
		&taskp->bd_addr, (unsigned long *)&taskp->bd_length,
		&taskp->bd_dev_acc);
	if (ret != DDI_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%s: Unable to allocate memory\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		goto bnxef_task_bd_table_pre_alloc_error;
	}
	stage |= BNXEF_TASK_BD_TABLE_MEM_ALLOCATED;
	memset(taskp->bd_addr, 0, taskp->bd_length);
	BNXEF_DEBUG(dsp, SCSI, -1,
	"%s%d: Binding DMA memory allocated for Task id 0x%x: length 0x%x",
				BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id,
			taskp->bd_length);

	ret = ddi_dma_addr_bind_handle(taskp->bd_dma_handle,
		NULL, taskp->bd_addr, taskp->bd_length,
		DDI_DMA_CONSISTENT | DDI_DMA_RDWR, DDI_DMA_SLEEP, NULL,
		&taskp->bd_cookie, &taskp->bd_cookie_cnt);
	if (ret != DDI_DMA_MAPPED) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to bind dma mem with handle: ret 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, ret);
		goto bnxef_task_bd_table_pre_alloc_error;
	}

	ASSERT(taskp->bd_cookie_cnt == 1);
	stage |= BNXEF_TASK_BD_TABLE_MEM_BOUND;
	taskp->task_flags |= BNXEF_FCOE_TASK_BD_LIST_ALLOC;
	BNXEF_DEBUG(dsp, SCSI, -1,
		"%s%d: DMA resources allocated for task 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
	return (BNXEF_SUCCESS);

bnxef_task_bd_table_pre_alloc_error:
	(void) bnxef_task_bd_table_pre_free(dsp, taskp, stage);
	return (BNXEF_FAILURE);
}

/*
 * This will allocate the BD table for each I/O required and will be
 * freed when the I/O completes. Since this is not linked into the session
 * based task list, no one else can be looking into this task structure.
 * Therefore there is no need to hold the lock at this point in time. Once, it
 * is in that list one must protect its state with mutex.
 */
int
bnxef_task_bd_table_alloc(bnxef_session_t *sessionp, fc_packet_t *pktp)
{

	bnxef_fcoe_task_t *taskp;
	ddi_dma_attr_t *bddmaattr = &bnxef_dma_attr;
	ddi_device_acc_attr_t *bdaccattr = & bnxef_dev_acc_attr;
	bnxef_port_t	*portp;
	bnxef_driver_state_t *dsp;
	int ret, stage = 0;

	ASSERT(pktp != NULL);
	ASSERT(sessionp != NULL);

	taskp = (bnxef_fcoe_task_t *)pktp->pkt_fca_private;
	portp = sessionp->lport;
	ASSERT(portp != NULL);
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	BNXEF_DEBUG(dsp, SCSI, portp->id,
		"%s%d: Allocating DMA Handle for BD table for task 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
	ret = ddi_dma_alloc_handle(dsp->dip, bddmaattr, DDI_DMA_SLEEP, NULL,
		&taskp->bd_dma_handle);
	if (ret != DDI_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to allocate task BD dma handle\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		goto bnxef_task_bd_table_alloc_error;
	}

	stage |= BNXEF_TASK_BD_TABLE_DMA_HANDLE_ALLOCATED;
	BNXEF_DEBUG(dsp, SCSI, portp->id,
		"%s%d: Allocating DMA memory of length 0x%x for task 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->bd_length,
			taskp->x_id);
	ret = ddi_dma_mem_alloc(taskp->bd_dma_handle,
		BNXEF_PAGE_SIZE,
		// pktp->pkt_data_cookie_cnt * sizeof (struct fcoe_bd_ctx),
		bdaccattr, DDI_DMA_CONSISTENT, DDI_DMA_SLEEP, NULL,
		&taskp->bd_addr, (unsigned long *)&taskp->bd_length,
		&taskp->bd_dev_acc);
	if (ret != DDI_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%s: Unable to allocate memory\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		goto bnxef_task_bd_table_alloc_error;
	}
	stage |= BNXEF_TASK_BD_TABLE_MEM_ALLOCATED;
	memset(taskp->bd_addr, 0, taskp->bd_length);
	BNXEF_DEBUG(dsp, SCSI, portp->id,
		"%s%d: Binding DMA memory allocated for Task id 0%x",
				BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);

	ret = ddi_dma_addr_bind_handle(taskp->bd_dma_handle,
		NULL, taskp->bd_addr, taskp->bd_length,
		DDI_DMA_CONSISTENT | DDI_DMA_RDWR, DDI_DMA_SLEEP, NULL,
		&taskp->bd_cookie, &taskp->bd_cookie_cnt);
	if (ret != DDI_DMA_MAPPED) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to bind dma mem with handle\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		goto bnxef_task_bd_table_alloc_error;
	}

	ASSERT(taskp->bd_cookie_cnt == 1);
	stage |= BNXEF_TASK_BD_TABLE_MEM_BOUND;
	taskp->task_flags |= BNXEF_FCOE_TASK_BD_LIST_ALLOC;
	BNXEF_DEBUG(dsp, SCSI, portp->id,
		"%s%d: DMA resources allocated for task 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
	return (BNXEF_SUCCESS);

bnxef_task_bd_table_alloc_error:
	(void) bnxef_task_bd_table_free(pktp, stage);
	return (BNXEF_FAILURE);
}

int
bnxef_task_bd_table_pre_free(bnxef_driver_state_t *dsp,
		bnxef_fcoe_task_t *taskp, uint32_t stage)
{

	ddi_dma_handle_t	tempdmahandle;
	ddi_acc_handle_t	dev_acc_handle;
	int	 ret = BNXEF_FAILURE;

	ASSERT(taskp != NULL);
	ASSERT(dsp != NULL);

	mutex_enter(&taskp->task_mutex);
	tempdmahandle = taskp->bd_dma_handle;
	dev_acc_handle = taskp->bd_dev_acc;
	mutex_exit(&taskp->task_mutex);

	if ((stage & BNXEF_TASK_BD_TABLE_MEM_BOUND) ==
		BNXEF_TASK_BD_TABLE_MEM_BOUND) {
		ret = ddi_dma_unbind_handle(tempdmahandle);
		ASSERT(ret == DDI_SUCCESS);
	}

	if ((stage &  BNXEF_TASK_BD_TABLE_MEM_ALLOCATED) ==
		BNXEF_TASK_BD_TABLE_MEM_ALLOCATED) {
		ddi_dma_mem_free(&dev_acc_handle);
	}

	if ((stage & BNXEF_TASK_BD_TABLE_DMA_HANDLE_ALLOCATED) ==
		BNXEF_TASK_BD_TABLE_DMA_HANDLE_ALLOCATED) {
		ddi_dma_free_handle(&tempdmahandle);
	}

	if (ret == DDI_SUCCESS) {
		mutex_enter(&taskp->task_mutex);
		taskp->task_flags &= ~BNXEF_FCOE_TASK_BD_LIST_ALLOC;
		mutex_exit(&taskp->task_mutex);
		return (BNXEF_SUCCESS);
	} else ret = BNXEF_FAILURE;

	return (ret);

}
/*
 * This routine frees all the BD resources. While freeing this holds the
 * task mutex so that, timeout or other routines are not updating it at the
 * same time.
 */
int
bnxef_task_bd_table_free(fc_packet_t *pktp, uint32_t stage)
{
	bnxef_fcoe_task_t *taskp;
	ddi_dma_handle_t	tempdmahandle;
	ddi_acc_handle_t	dev_acc_handle;
	int	 ret = BNXEF_FAILURE;

	ASSERT(pktp != NULL);

	taskp = (bnxef_fcoe_task_t *)pktp->pkt_fca_private;
	mutex_enter(&taskp->task_mutex);
	tempdmahandle = taskp->bd_dma_handle;
	dev_acc_handle = taskp->bd_dev_acc;
	mutex_exit(&taskp->task_mutex);

	if ((stage & BNXEF_TASK_BD_TABLE_MEM_BOUND) ==
		BNXEF_TASK_BD_TABLE_MEM_BOUND) {
		ret = ddi_dma_unbind_handle(tempdmahandle);
		ASSERT(ret == DDI_SUCCESS);
	}

	if ((stage &  BNXEF_TASK_BD_TABLE_MEM_ALLOCATED) ==
		BNXEF_TASK_BD_TABLE_MEM_ALLOCATED) {
		ddi_dma_mem_free(&dev_acc_handle);
	}

	if ((stage & BNXEF_TASK_BD_TABLE_DMA_HANDLE_ALLOCATED) ==
		BNXEF_TASK_BD_TABLE_DMA_HANDLE_ALLOCATED) {
		ddi_dma_free_handle(&tempdmahandle);
	}

	if (ret == DDI_SUCCESS) {
		mutex_enter(&taskp->task_mutex);
		taskp->task_flags &= ~BNXEF_FCOE_TASK_BD_LIST_ALLOC;
		mutex_exit(&taskp->task_mutex);
		return (BNXEF_SUCCESS);
	} else ret = BNXEF_FAILURE;

	return (ret);
}
