/*
 * The information contained in this file is confidential and proprietary to
 * Broadcom Corporation.  No part of this file may be reproduced or
 * distributed, in any form or by any means for any purpose, without the
 * express written permission of Broadcom Corporation.
 *
 * (c) COPYRIGHT 2010-2012 Broadcom Corporation, ALL RIGHTS RESERVED.
 */

#pragma ident "(C) 2010-2012 Broadcom Corporation"
#pragma ident "$File$$Revision$$Change$$DateTime$"


#ifndef __BNXEF_FIP_H__
#define	__BNXEF_FIP_H__
#include <sys/types.h>
#include <sys/conf.h>
#include <sys/stream.h>
#include <sys/stropts.h>
#include <sys/errno.h>
#include <sys/cred.h>
#include <sys/poll.h>
#include <sys/modctl.h>
#include <sys/stat.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/ddifm.h>
#include <sys/fm/protocol.h>
#include <sys/fm/util.h>
#include <sys/fm/io/ddi.h>
#include <sys/pattr.h>
#include <sys/sysmacros.h>
#include <sys/strsun.h>
#include <sys/ksynch.h>
#include <sys/kstat.h>
#include <sys/strsubr.h>
#include <sys/ksynch.h>
#include <sys/ethernet.h>

#define	FIP_ETHERTYPE		0x8914
#define	FIP_FRAME_VER		0x10
#define	DEFAULT_FIP_PRIORITY	128
#define	DEFAULT_FC_MAP		0x0EFC00
#define	FKA_ADV_PERIOD		20	/* 2.5 * time FKA_ADV_PERIOD */
#define	FKA_VN_PERIOD		90

#define	BNXEF_FIP_FRAME_RETRY_COUNT	0x3

#define		VLAN_ID_SHIFT		0
#define	 VLAN_ID_SIZE		12
#define	 VLAN_ID_MASK		0x0fffu

#define	 VLAN_CFI_SHIFT		(VLAN_ID_SHIFT + VLAN_ID_SIZE)
#define	 VLAN_CFI_SIZE		1
#define	 VLAN_CFI_MASK		0x0001u

#define	 VLAN_PRI_SHIFT		(VLAN_CFI_SHIFT + VLAN_CFI_SIZE)
#define	 VLAN_PRI_SIZE		3
#define		VLAN_PRI_MASK		0x0007u

#define	VLAN_TCI(pri, cfi, vid) \
	(((pri) << VLAN_PRI_SHIFT) | \
	((cfi) << VLAN_CFI_SHIFT) | \
	((vid) << VLAN_ID_SHIFT))

#define	VLAN_ID(tci)		(((tci) >> VLAN_ID_SHIFT) & VLAN_ID_MASK)
#define	VLAN_CFI(tci)		(((tci) >> VLAN_CFI_SHIFT) & VLAN_CFI_MASK)
#define	VLAN_PRI(tci)		(((tci) >> VLAN_PRI_SHIFT) & VLAN_PRI_MASK)

typedef struct __fip_pdu_hdr__ {
	uint8_t		version;	/* Should be 1 i << 4 */
	uint8_t		reserved;
} __attribute__((packed)) fip_pdu_hdr_t;

typedef struct __fip_operation_fmt_hdr__ {
	uint32_t	fip_prot_code:16;
	uint32_t	reserved1:8;
	uint32_t	fip_subcode:8;
	uint32_t	desc_len:16;
	uint32_t	flags:16;
} fip_operation_fmt_hdr_t;

/* Flags definition */
#define	BNXEF_FIP_FLAGS_A	(0x01 << 2)
#define	BNXEF_FIP_FLAGS_S	(0x01 << 1)
#define	BNXEF_FIP_FLAGS_F	(0x01)
#define	BNXEF_FIP_FLAGS_FP	(0x01 << 15)
#define	BNXEF_FIP_FLAGS_SP	(0x01 << 14)

/* The following are the the protocol code */
#define	FIP_PROT_DISCOVERY	0x0001
#define	FIP_PROT_VLINK		0x0002
#define	FIP_PROT_VLINK_MAIN	0x0003
#define	FIP_PROT_VLAN		0x0004


/* the following are the protocol sub codes */
#define	FIP_PROT_SUB_DISC_SOLICIT	0x01
#define	FIP_PROT_SUB_DISC_ADV		0x02
#define	FIP_PROT_SUB_VLINK_INIT_REQ	0x01
#define	FIP_PROT_SUB_VLINK_RESP		0x02
#define	FIP_PROT_SUB_KEEP_ALIVE		0x01
#define	FIP_PROT_SUB_CLEAR_VLINK	0x02
#define	FIP_PROT_SUB_VLAN_REQUEST	0x01
#define	FIP_PROT_SUB_VLAN_NOTIF		0x02

/* The folllowing are FIP discriptor types */
#define	FIP_DESC_TYPE_NONE		0	/* Reserved */
#define	FIP_DESC_TYPE_PRIORITY		1	/* Priority */
#define	FIP_DESC_TYPE_MAC_ADDR		2	/* MAC Address */
#define	FIP_DESC_TYPE_FC_MAP		3	/* FC_MAP */
#define	FIP_DESC_NAME_IDENTIFIER	4	/* NAme Identifier */
#define	FIP_DESC_NAME_FABRIC		5	/* Fabric descriptor */
#define	FIP_DESC_MAX_FCOE_SIZE		6	/* MAx FCoE Size */
#define	FIP_DESC_FLOGI			7	/* Flogi */
#define	FIP_DESC_NPIV_FDISC		8	/* NPIV FDISC */
#define	FIP_DESC_LOGO			9	/* LOGO */
#define	FIP_DESC_ELP			10	/* ELP - May not be used */
#define	FIP_DESC_VX_PORT_ID		11	/* Vx port identification */
#define	FIP_DESC_FKA_ADV_PERIOD		12	/* Advertizement period */
#define	FIP_DESC_VENDOR_ID		13	/* Vendor ID */
#define	FIP_DESC_VLAN			14
#define	FIP_CRITICAL_DESC_TYPE_MAX	127

/*
 * The following are the standard length for different specified critical
 * descriptors. These lengths are in words of 32 bit size.
 */
#define	FIP_PRIORITY_DESC_LEN	1
#define	FIP_MAC_ADDR_DESC_LEN	2
#define	FIP_NAME_ID_DESC_LEN	3
#define	FIP_FABRIC_DESC_LEN	4
#define	FIP_FKA_ADV_PERIOD_DESC_LEN 2
#define	FIP_FLOGI_DESC_LEN	36
#define	FIP_NPIV_FDISC_DESC_LEN	36
#define	FIP_NPIV_FDISC_LS_RJT_DESC_LEN	9
#define	FIP_LOGO_DESC_LEN	11
#define	FIP_LOGO_LS_ACC_DESC_LEN 8
#define	FIP_VLAN_DESC_LEN	1
#define	FIP_VX_PORT_DESC_LEN	5

/* The following are the list of descriptors defined in FC-BB-5 specification */

/* priority Descriptor */
typedef struct __fip_priority_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	reserved:8;
	uint32_t	priority:8;
} fip_priority_desc_fmt_t;

/* FIP MAC Address descriptor */
typedef struct __fip_mac_addr_desc_fmt_ {
	uint32_t	type:8;			/* Type of descriptor */
	uint32_t	length:8;		/* Length of descriptor */
	uint32_t	mac_addr_msb1:8;	/* MSB of MAC address */
	uint32_t	mac_addr_msb0:8;	/* MSB of MAC address */

	uint32_t	mac_addr_lsb;		/* MAC Address LSB */
} fip_mac_addr_desc_fmt_t;

/* FIP FC MAP descriptor */
typedef  struct __fip_fc_map_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	reserved:16;
	uint32_t	reserved1:8;
	uint32_t	fc_map:24;
} fip_fc_map_desc_fmt_t;

/* FIP Name identifier descriptor format */
typedef  struct __fip_name_id_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	reserved:16;
	uint8_t		name[8];		/* Name identifier */
} fip_name_id_desc_fmt_t;

/* FIP Fabric descriptor format */
typedef struct  __fip_fabric_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	vf_id:16;
	uint32_t	fc_map;
	uint32_t	fabric_name_msb;
	uint32_t	fabric_name_lsb;
} fip_fabric_desc_fmt_t;

/* FCoE MAX Size Desc format */
typedef struct __fip_max_fcoe_size_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	max_fcoe_size:16;
} fip_max_fcoe_size_desc_fmt_t;

/* FIP FLOGI/FDISC/LSACC/LSRJT/LOGO Desc formats */
typedef struct __fip_fc_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	reserved:16;
	uint8_t		fc_info[1];
} fip_fc_desc_fmt_t;

/* FIP Vx Port Identification Descriptor */

typedef struct __fip_vx_port_desc_fmt__ {
	uint8_t		type;
	uint8_t		length;
	uint8_t		macaddr[6];
	uint32_t	address_id;
	uint8_t		port_name[8];
} fip_vx_port_desc_fmt_t;

/* FIP FKA_ADV_Period Descriptor Format */
typedef struct __fip_fka_adv_period_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	d:16;
	uint32_t	fka_adv_period;		/* In Millisec */
} fip_fka_adv_period_desc_fmt_t;

/* FIP Vendor ID descriptor format */
typedef struct __fip_vendor_id_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	reserved:16;
	uint32_t	vendor_id_msb;
	uint32_t	vendor_id_lsb;
} fip_vendor_id_desc_fmt_t;

/* FIP VLAN desc format */
typedef struct __fip_vlan_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	vlan_id:16;
}  fip_vlan_desc_fmt_t;

/* FIP Vendor specific descriptor format */
typedef struct __fip_vs_desc_fmt__ {
	uint32_t	type:8;
	uint32_t	length:8;
	uint32_t	reserved:16;
	uint32_t	vendor_id_msb;
	uint32_t	vendor_id_lsb;
	uint8_t		vs_info[1];
} fip_vs_desc_fmt_t;
typedef struct __bnxef_fip_frame__ {
		struct ether_header ether;
		struct ether_vlan_extinfo vlan;
		fip_pdu_hdr_t	pdu;
		fip_operation_fmt_hdr_t ophdr;
		uint8_t desc[1];
} __attribute__((packed)) bnxef_fip_frame_t;

typedef struct __bnxef_fip_frame_novlan__ {
	struct ether_header ether;
	fip_pdu_hdr_t	pdu;
	fip_operation_fmt_hdr_t  ophdr;
	uint8_t		desc[1];
} __attribute__((packed)) bnxef_fip_frame_novlan_t;

/* Exported Routines */
mblk_t *bnxef_prep_mblk_fip_vlan_request(bnxef_driver_state_t *dsp);
mblk_t *bnxef_prep_mblk_fip_disc_sol(bnxef_driver_state_t *dsp);
mblk_t *bnxef_prep_mblk_fip_flogi_fdisc(bnxef_driver_state_t *dsp,
	uint8_t opcode, uint16_t ox_id, uint8_t *port_name, uint8_t *node_name);
mblk_t *bnxef_prep_mblk_fip_logo(bnxef_driver_state_t *dsp,
	bnxef_lport_t *portp);
mblk_t *bnxef_prep_mblk_fip_keep_alive(bnxef_driver_state_t *dsp,
	bnxef_lport_t *portp);
#endif
