/*
 * The information contained in this file is confidential and proprietary to
 * Broadcom Corporation.  No part of this file may be reproduced or
 * distributed, in any form or by any means for any purpose, without the
 * express written permission of Broadcom Corporation.
 *
 * (c) COPYRIGHT 2010-2012 Broadcom Corporation, ALL RIGHTS RESERVED.
 */

#pragma ident "(C) 2010-2012 Broadcom Corporation"
#pragma ident " $File$$Revision$$Change$$DateTime$"
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/stat.h>
#include <sys/pci.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>
#include <sys/cmn_err.h>
#include <sys/file.h>
#include <sys/cred.h>
#include <sys/byteorder.h>
#include <sys/atomic.h>
#include <sys/scsi/scsi.h>
#include <sys/modhash.h>
#include <sys/sunldi.h>
#include <sys/kstat.h>
#include <sys/kmem.h>
#include <sys/ethernet.h>
#include <sys/stream.h>
#include <sys/sunldi.h>
/*
 * leadville header files
 */
#include <sys/fibre-channel/fc_types.h>
#include <sys/fibre-channel/impl/fc_fcaif.h>
#include <bnxe_binding.h>
#include <bnxef_tq_param.h>
#include <bnxef_fcf_info.h>
#include <bnxef_hsi.h>
#include <bnxef_fcoe_task.h>
#include <bnxef_fcoe_session.h>
#include <bnxef_state.h>
#include <bnxef_fip.h>
#include <bnxef_fcoe.h>
#include <bnxef_debug.h>


/*
 * The following handles the port related utility routines
 */

/*
 * The port cache constructor
 */
/*ARGSUSED*/
int
bnxef_port_cache_constructor(void *bufp, void *args, int flags)
{

	char  name[KSTAT_STRLEN];
	bnxef_port_t *portp = (bnxef_port_t *)bufp;
	bnxef_driver_state_t *dsp = (bnxef_driver_state_t *)args;

	NOTE(ARGUNUSED(flags))

	bzero((uint8_t *)bufp, sizeof (bnxef_port_t));
	(void) snprintf(name, KSTAT_STRLEN, "port%d-%d_mutex",
		dsp->instance, dsp->port_ref_cnt);
	mutex_init(&portp->port_mutex, name, MUTEX_DRIVER, NULL);
	(void) snprintf(name, KSTAT_STRLEN, "port%d-%d_cv",
		dsp->instance, dsp->port_ref_cnt);
	cv_init(&portp->id_cv, name, CV_DRIVER, NULL);
	(void) snprintf(name, KSTAT_STRLEN, "port%d-%d-shudown-cv",
		dsp->instance, dsp->port_ref_cnt++);
	cv_init(&portp->shutdown_cv, name, CV_DRIVER, NULL);

	return (0);
}

/*
 * The port cache destructor
 */
/*ARGSUSED*/
void
bnxef_port_cache_destructor(void *bufp, void *args)
{
	bnxef_port_t	*portp = (bnxef_port_t *)bufp;

	NOTE(ARGUNUSED(args))

	mutex_destroy(&portp->port_mutex);
	cv_destroy(&portp->id_cv);

}

/*
 * Port cache create routine, only called from attach, no locks held.
 */
int
bnxef_port_cache_create(bnxef_driver_state_t *dsp)
{

	char  buf[KSTAT_STRLEN];

	ASSERT(dsp != NULL);

	(void) snprintf(buf, KSTAT_STRLEN, "port%d_cache", dsp->instance);
	dsp->port_cache = kmem_cache_create(buf, sizeof (bnxef_port_t),
		8, bnxef_port_cache_constructor,
		bnxef_port_cache_destructor, NULL,
		dsp, 0, KM_SLEEP);

	if (dsp->port_cache == NULL) {
		cmn_err(CE_PANIC, "%s%d Unable to allocate port cache\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		return (DDI_FAILURE);
	}

	return (DDI_SUCCESS);
}
/*
 * This destroys the cache created at attach time. This routine can only be
 * called from detach.
 */
int
bnxef_port_cache_destroy(bnxef_driver_state_t *dsp)
{
	ASSERT(dsp != NULL);

	kmem_cache_destroy(dsp->port_cache);

	return (DDI_SUCCESS);
}



/*
 * This initializes the port list
 */
int
bnxef_port_list_init(bnxef_driver_state_t *dsp)
{

	bnxef_port_list_t *portlstp;
	char  name[KSTAT_STRLEN];

	ASSERT(dsp != NULL);

	portlstp = &dsp->port_list;

	bzero((uint8_t *)portlstp, sizeof (bnxef_port_list_t));
	snprintf(name, KSTAT_STRLEN, "portlist%d-mutex",
		dsp->instance);
	mutex_init(&portlstp->portlist_mutex, name, MUTEX_DRIVER, NULL);

	return (DDI_SUCCESS);
}

/*
 * THis routine will destroy the port_list mutex and return success.
 */
int
bnxef_port_list_fini(bnxef_driver_state_t *dsp)
{
	bnxef_port_list_t	 *portlstp;

	ASSERT(dsp != NULL);

	portlstp = &dsp->port_list;

	mutex_destroy(&portlstp->portlist_mutex);

	return (DDI_SUCCESS);
}


/*
 * The following are the routine to find a port with certain key.
 */

/*
 * This will identify the first port in the list with flags set.
 */
bnxef_port_t *
bnxef_port_list_find_flags(bnxef_driver_state_t *dsp, uint32_t flags)
{

	bnxef_port_list_t	*portlstp;
	bnxef_port_t		*portp;
	int i, found = 0;

	ASSERT(dsp != NULL);
	ASSERT(flags != 0);

	portlstp = &dsp->port_list;

	mutex_enter(&portlstp->portlist_mutex);
	portp = portlstp->head;
	if (portp == NULL) {
		mutex_exit(&portlstp->portlist_mutex);
		return (NULL);
	}
	ASSERT(portp != NULL);
	for (i = 0; i < portlstp->num_items; i++) {
		if ((portp->flags & flags) == flags) {
			found++;
			break;
		}
		portp = portp->next;
		if (portp == NULL) break;
	}

	mutex_exit(&portlstp->portlist_mutex);
	return (found? portp: NULL);
}

/*
 * This will get the first port from the list and return. Explict
 * call to remove the port need to be done to remove it from list.
 */
bnxef_port_t *
bnxef_get_first_port(bnxef_driver_state_t *dsp)
{

	bnxef_port_list_t	*portlstp;
	bnxef_port_t		*portp;

	ASSERT(dsp != NULL);

	portlstp = &dsp->port_list;

	mutex_enter(&portlstp->portlist_mutex);
	portp = portlstp->head;
	mutex_exit(&portlstp->portlist_mutex);
	return (portp);
}


/*
 * This will find the port with MAC address, vlan id and FC ID
 */
bnxef_port_t *
bnxef_port_list_find_id(bnxef_driver_state_t *dsp, uint8_t *macaddrp,
		uint16_t vlan_id, uint32_t fcid)
{

	bnxef_port_list_t	*portlstp;
	bnxef_port_t		*portp;
	int i, found = 0;
	int ret;

	ASSERT(dsp != NULL);
	ASSERT(macaddrp != NULL);

	BNXEF_DEBUG(dsp, GENERAL, -1,
		"%s%d: Entering Value Vlan id 0x%x, port_id 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, vlan_id, fcid);

	portlstp = &dsp->port_list;
	mutex_enter(&portlstp->portlist_mutex);
	portp = portlstp->head;
	if (portp == NULL) {
		mutex_exit(&portlstp->portlist_mutex);
		return (NULL);
	}

	ASSERT(portp != NULL);
	for (i = 0; i < portlstp->num_items; i++) {
		if (portp->flags & BNXEF_FC_PORT_ID_ACQUIRED) {
			BNXEF_AUTO_DEBUG("%s%d: Vlan id 0x%x, port_id 0x%x",
				BNXEF_DRIVER_NAME, dsp->instance, vlan_id,
				fcid);
			BNXEF_DEBUG(dsp, GENERAL, -1,
				"%s%d: Portp->Vlan id 0x%x, port->id 0x%x",
				BNXEF_DRIVER_NAME, dsp->instance,
				portp->vlan_id, portp->id);
			ret = bcmp(portp->mac_addr, macaddrp, 6);
			if ((portp->vlan_id == vlan_id) &&
			    (portp->id  == fcid) &&
			    (ret == 0)) {
				found++;
				break;
			}
		}
		portp = portp->next;
		if (portp == NULL) break;
	}

	mutex_exit(&portlstp->portlist_mutex);
	return (found? portp: NULL);
}


/*
 * This routine is to add the port to the end of the list
 */
int
bnxef_port_list_add(bnxef_driver_state_t *dsp, bnxef_port_t *portp)
{
	bnxef_port_list_t *portlstp;

	ASSERT(dsp != NULL);
	ASSERT(portp != NULL);

	portlstp = &dsp->port_list;

	mutex_enter(&portlstp->portlist_mutex);

	portp->next = NULL;
	if ((portlstp->head == NULL) && (portlstp->tail == NULL)) {
		/* The list is empty, insert this element as first element */
		portlstp->head = portp;
		portlstp->tail = portp;
	} else {
		ASSERT(portlstp->tail != NULL);
		ASSERT(portlstp->tail->next == NULL);
		portlstp->tail->next = portp;
		portlstp->tail = portp;
	}
	portlstp->num_items++;
	mutex_exit(&portlstp->portlist_mutex);
	return (BNXEF_SUCCESS);
}


/*
 * This routine removes the given port from the list
 */
int
bnxef_port_list_remove(bnxef_driver_state_t *dsp, bnxef_port_t *portp)
{
	bnxef_port_list_t *portlstp;
	bnxef_port_t *portcp; 		/* Current port while walking */
	bnxef_port_t *portprevp;	/* Previous port while walking */
	int i, found = 0;
	int ret  = BNXEF_FAILURE;

	ASSERT(dsp != NULL);
	ASSERT(portp != NULL);

	portlstp = &dsp->port_list;

	ASSERT(portlstp->num_items > 0);
	mutex_enter(&portlstp->portlist_mutex);
	portcp = portlstp->head;
	ASSERT(portcp != NULL);
	portprevp = NULL;
	for (i = 0; i < portlstp->num_items; i++) {
		if (portcp == portp) {
			found++;
			break;
		}
		portprevp = portcp;
		portcp = portcp->next;
	}

	if (found) {
		if (portlstp->num_items == 1) {
			/* There is only one item */
			portlstp->head = NULL;
			portlstp->tail = NULL;
		} else {
			if (portprevp != NULL)
				portprevp->next = portp->next;
			else
				portlstp->head = portp->next;

			if (portlstp->tail == portp)
				portlstp->tail = portprevp;
		}
		portlstp->num_items--;
		ret = BNXEF_SUCCESS;
	}
	portp->next = NULL;
	portp->flags = 0;
	mutex_exit(&portlstp->portlist_mutex);

	return (ret);
}
