/*
 * The information contained in this file is confidential and proprietary to
 * Broadcom Corporation.  No part of this file may be reproduced or
 * distributed, in any form or by any means for any purpose, without the
 * express written permission of Broadcom Corporation.
 *
 * (c) COPYRIGHT 2010-2012 Broadcom Corporation, ALL RIGHTS RESERVED.
 */

#pragma ident "(C) 2010-2012 Broadcom Corporation"
#pragma ident " $File$$Revision$$Change$$DateTime$"
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/stat.h>
#include <sys/pci.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>
#include <sys/cmn_err.h>
#include <sys/file.h>
#include <sys/cred.h>
#include <sys/byteorder.h>
#include <sys/atomic.h>
#include <sys/scsi/scsi.h>
#include <sys/modhash.h>
#include <sys/sunldi.h>
#include <sys/kstat.h>
#include <sys/kmem.h>
#include <sys/ethernet.h>
#include <sys/stream.h>
#include <sys/sunldi.h>
/*
 * leadville header files
 */
#include <sys/fibre-channel/fc_types.h>
#include <sys/fibre-channel/impl/fc_fcaif.h>
#include <sys/fibre-channel/ulp/fcp.h>
#include <bnxe_binding.h>
#include <bnxef_tq_param.h>
#include <bnxef_fcf_info.h>
#include <bnxef_hsi.h>
#include <bnxef_fcoe_task.h>
#include <bnxef_fcoe_session.h>
#include <bnxef_state.h>
#include <bnxef_fip.h>
#include <bnxef_fcoe.h>
#include <bnxef_debug.h>

/* External Prototypes */
extern int bnxef_init_task(bnxef_session_t *sessionp,  fc_packet_t *pktp);
extern int bnxef_init_mp_task(bnxef_session_t *sessionp, fc_packet_t *pktp);
extern void bnxef_add_2_sq(bnxef_session_t *sessionp, uint16_t xid);
extern void bnxef_timeout(void *arg);
extern void bnxef_ring_doorbell(bnxef_session_t *sessionp);
extern int bnxef_start_polling(bnxef_driver_state_t *dsp);
extern void bnxef_stop_polling(bnxef_driver_state_t *dsp);
extern mblk_t  *bnxef_prepare_fcoe_mblk_from_pkt(bnxef_driver_state_t *dsp,
		bnxef_port_t *portp, fc_packet_t *pktp);
extern void bnxef_complete_pkt(fc_packet_t *pktp, uint8_t state,
		uint8_t action, uint32_t reason, uint32_t expln);
extern void bnxef_big_endian(void *bufp, int size);
extern int bnxef_get_active_ios(bnxef_session_t *sessionp);
extern boolean_t bnxef_send_frame(bnxef_driver_state_t  *dsp,
		mblk_t *mp, uint16_t vlan_id);
extern int bnxef_get_num_free_list_items(bnxef_driver_state_t *dsp);
extern int bnxef_end_task(bnxef_driver_state_t *dsp, fc_packet_t *pktp);
extern void
bnxef_complete_polling(bnxef_session_t *sessionp, bnxef_fcoe_task_t *taskp);


int
bnxef_gpn_id_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp);
int
bnxef_da_id_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp);

/*
 * This will transport the SCSI commands provided for transporting.
 */
int
bnxef_scsi_transport_handler(bnxef_session_t *sessionp,
	bnxef_fcoe_task_t *taskp)
{
	bnxef_driver_state_t	 *dsp;
	fc_packet_t			*pktp;
	bnxef_port_t		*portp;
	fcp_cmd_t		cmd;
	bnxef_tq_params_t	*tqp;
	uint32_t		task_flags;
	timeout_id_t		tid;
	int ret, active_ios;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	dsp = portp->statep;
	ASSERT(dsp != NULL);

	pktp = taskp->pktp;
	ASSERT(pktp != NULL);


	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	if (dsp->physical_port_state &BNXEF_PHYPORT_OFFLINE) {
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Unable to Transport Physical Port Offline",
		BNXEF_DRIVER_NAME, dsp->instance);
		mutex_exit(&dsp->state_mutex);
		bnxef_end_task(dsp, pktp);
		return (FC_OFFLINE);
	}

	if (dsp->state & BNXEF_L2_UNLOAD_CALLED) {
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Unable to Transport Physical Port is unloaded",
		BNXEF_DRIVER_NAME, dsp->instance);
		mutex_exit(&dsp->state_mutex);
		bnxef_end_task(dsp, pktp);
		return (FC_OFFLINE);
	}
	mutex_exit(&dsp->state_mutex);

	mutex_enter(&portp->port_mutex);
	if (portp->flags & BNXEF_FC_PORT_SOL_BOUND) {
		if ((portp->flags & BNXEF_FC_PORT_SHUTDOWN_IN_PROGRESS) ||
			(portp->flags & BNXEF_FC_PORT_ID_NOT_ACQUIRED)) {
			BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to Transport, port shutdown in progress",
				BNXEF_DRIVER_NAME, dsp->instance);
			BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to Transport, Port ID Not acquired",
				BNXEF_DRIVER_NAME, dsp->instance);
			mutex_exit(&portp->port_mutex);
			bnxef_end_task(dsp, pktp);
			return (FC_TRAN_BUSY);
		}
	}
	mutex_exit(&portp->port_mutex);

	mutex_enter(&sessionp->session_mutex);
	if ((sessionp->state &
		(BNXEF_SESSION_OFFLOADED | BNXEF_SESSION_ENABLED)) == 0) {
		/* If it was not enabled or offloaded */
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to transport Session not offloaded\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		mutex_exit(&sessionp->session_mutex);
		bnxef_end_task(dsp, pktp);
		return (FC_TRAN_BUSY);
	}

	if (sessionp->state & BNXEF_TM_CLEANUP_IN_PROGRESS) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to Transport Session Reset in progress",
				BNXEF_DRIVER_NAME, dsp->instance);
		mutex_exit(&sessionp->session_mutex);
		bnxef_end_task(dsp, pktp);
		return (FC_TRAN_BUSY);
	}

	mutex_exit(&sessionp->session_mutex);


	/*
	 * If the session is offloaded and enabled, then throttle the number
	 * of I/Os per session based on number of active I/Os pending on
	 * this session.
	 */
		active_ios = bnxef_get_active_ios(sessionp);
		if (active_ios > BNXEF_MAX_ACTIVE_IOS_PER_SESSION) {
		BNXEF_DEBUG(dsp, GENERAL, -1,
			"%s%d: Unable Transport, throttling active IOs",
			BNXEF_DRIVER_NAME, dsp->instance);
		BNXEF_STATS(dsp, scsi_bsy_ret);
		mutex_enter(&sessionp->session_mutex);
		sessionp->state |= BNXEF_SESSION_CQ_ARMING_REQUIRED;
		mutex_exit(&sessionp->session_mutex);
		bnxef_end_task(dsp, pktp);
		return (FC_TRAN_BUSY);
	}



	/*
	 * Now check whether this command is just SCSI READ/WRITE or Task
	 * management command. For task management the transport method is
	 * different.
	 */
	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)&cmd,
		(uint8_t *)pktp->pkt_cmd, sizeof (fcp_cmd_t),
			DDI_DEV_AUTOINCR);

	mutex_enter(&taskp->task_mutex);
	if (cmd.fcp_cntl.cntl_kill_tsk || cmd.fcp_cntl.cntl_clr_aca ||
		cmd.fcp_cntl.cntl_reset_tgt || cmd.fcp_cntl.cntl_reset_lun ||
		cmd.fcp_cntl.cntl_clr_tsk) {
		taskp->task_flags |= BNXEF_FCOE_TASK_MGMT_CMD;
		if (cmd.fcp_cntl.cntl_reset_tgt) {
			mutex_enter(&sessionp->session_mutex);
			sessionp->state |= BNXEF_TM_CLEANUP_IN_PROGRESS;
			mutex_exit(&sessionp->session_mutex);
		}
	} else {
		taskp->task_flags |= BNXEF_FCOE_TASK_SCSI;
	}
	task_flags = taskp->task_flags;
	/* Update the pkt_cmdfhdr with this Ox_id allocated for this exchange */
	pktp->pkt_cmd_fhdr.ox_id = taskp->x_id;
	mutex_exit(&taskp->task_mutex);
	pktp->pkt_cmd_fhdr.rx_id = 0xffff;

	BNXEF_DEBUG(dsp, SCSI, -1, "%s%d: Sending SCSI command for Task 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);

	/*
	 * For Retried command task flags might have completed flag set
	 * reset it here. Because it may not call uninit_pkt.
	 */
	if (taskp->task_flags & BNXEF_FCOE_TASK_COMPLETED) {
		BNXEF_DEBUG(dsp, SCSI, -1,
			"%s%d: BNXEF_FCOE_TASK_COMPLETED ENABLED IN beginning",
			BNXEF_DRIVER_NAME, dsp->instance);
		taskp->task_flags &= ~BNXEF_FCOE_TASK_COMPLETED;
	}

	if (taskp->task_flags & BNXEF_FCOE_RECOVERY_IN_PROGRESS) {
		cmn_err(CE_NOTE,
			"%s%d: RECOVER_IN PROGRES set in beginning",
			BNXEF_DRIVER_NAME, dsp->instance);
		taskp->task_flags &= ~BNXEF_FCOE_RECOVERY_IN_PROGRESS;
	}

	/* Add it to Queue for now, but if there is a failure, remove it */
	if (sessionp != NULL) {
		/*
		 * There is already a session established. Include the task
		 * into session task list
		 */
		ret = bnxef_task_session_list_add(sessionp, taskp);
		if (ret != BNXEF_SUCCESS) {
			return (FC_TRAN_BUSY);
		}

	}
	taskp->start_lbolt = ddi_get_lbolt();
	/* Initialize the pkt information into fw task information */
	if (task_flags & BNXEF_FCOE_TASK_SCSI) {
		ret = bnxef_init_task(sessionp, pktp);
	} else {
		ret = bnxef_init_mp_task(sessionp, pktp);
	}

	if (ret != BNXEF_SUCCESS) {
		/* We should also cleanup, bd list allocation - To be Done */
		bnxef_task_session_list_remove(sessionp, taskp);
		return (FC_TRANSPORT_ERROR);
	}




	/* ARM the timer to catch run away condition */
	tqp = &taskp->tq_params;
	tqp->dsp = dsp;
	tqp->taskp = taskp;
	tqp->sessp = sessionp;
	tqp->portp = portp;
	tqp->mp  = NULL;
	tqp->type = BNXEF_SCSI_TIMEOUT_TYPE;


	tid = timeout(bnxef_timeout, tqp, BNXEF_SEC2TICK(pktp->pkt_timeout));
//	tid= timeout(bnxef_timeout, tqp, BNXEF_SEC2TICK(10));
	mutex_enter(&taskp->task_mutex);
	taskp->timerhdl = tid;
	taskp->task_flags |= (BNXEF_FCOE_TASK_TIMER_ARMED |
			BNXEF_FCOE_SUBMITTED_TO_FIRMWARE);
	mutex_exit(&taskp->task_mutex);

	BNXEF_STATS(dsp, fp_completion_tx);

	if ((pktp->pkt_tran_flags & FC_TRAN_NO_INTR) == FC_TRAN_NO_INTR) {
		mutex_enter(&taskp->task_mutex);
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Waiting for Polled mode comand to COmplete task id 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
		/* Now add the task XID to the send Q */
		bnxef_add_2_sq(sessionp, taskp->x_id);
		/* Now ring the door bell register appropriately */
		bnxef_ring_doorbell(sessionp);
		mutex_exit(&taskp->task_mutex);
		if (ddi_in_panic()) {
			(void) bnxef_complete_polling(sessionp, taskp);
		} else {
			mutex_enter(&taskp->task_mutex);
			/* Now wait for your event to show up */
			cv_wait(&taskp->task_cv, &taskp->task_mutex);
			BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Waking up and handling COmpletion Id 0x%x",
				BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
			/* If it wakes up */
			mutex_exit(&taskp->task_mutex);
		}
#if 0
		/* Complete this exchange initiated in polled mode */
		bnxef_start_polling(dsp);
		mutex_enter(&taskp->task_mutex);
		/* Now wait for your event to show up */
		cv_wait(&taskp->task_cv, &taskp->task_mutex);
		/* If it wakes up */
		mutex_exit(&taskp->task_mutex);
		bnxef_stop_polling(dsp);
#endif
	} else {
		/* Now add the task XID to the send Q */
		bnxef_add_2_sq(sessionp, taskp->x_id);
		/* Now ring the door bell register appropriately */
		bnxef_ring_doorbell(sessionp);
		mutex_enter(&taskp->task_mutex);
		taskp->post_lbolt = ddi_get_lbolt();
		mutex_exit(&taskp->task_mutex);
	}


	BNXEF_STATS(dsp, scsi_dispatched);

	return (FC_SUCCESS);
}

int
bnxef_rpn_id_tx_handler(bnxef_session_t *sessionp, bnxef_fcoe_task_t *taskp)
{
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;


	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0)
		ctp = kmem_zalloc(pktp->pkt_cmdlen, KM_SLEEP);
	ASSERT(ctp != NULL);

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);
//	cmn_err(CE_NOTE, "Taskp 0x%p, cmd_len 0x%x, Addr 0x%p",
//		taskp, pktp->pkt_cmdlen, ctp);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	bnxef_big_endian(taskp->payloadp, 20);

	return (BNXEF_SUCCESS);
}

int
bnxef_rnn_id_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;


	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0)
		ctp = kmem_zalloc(pktp->pkt_cmdlen, KM_SLEEP);
	ASSERT(ctp != NULL);

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	bnxef_big_endian(taskp->payloadp, 20);

	return (BNXEF_SUCCESS);
}

int
bnxef_rcs_id_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0)
		ctp = kmem_zalloc(pktp->pkt_cmdlen, KM_SLEEP);
	ASSERT(ctp != NULL);

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	bnxef_big_endian(taskp->payloadp, 24);

	return (BNXEF_SUCCESS);
}

int
bnxef_rft_id_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;
	    bnxef_driver_state_t *dsp;
	    ns_rfc_type_t	*rftidp;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	    dsp = (bnxef_driver_state_t *)portp->statep;
	    ASSERT(dsp != NULL);

	pktp = taskp->pktp;

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0)
		ctp = kmem_zalloc(pktp->pkt_cmdlen, KM_SLEEP);
	ASSERT(ctp != NULL);

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	    /* LINTED: alignment */
	    rftidp = (ns_rfc_type_t *)(taskp->payloadp +
		sizeof (fc_ct_header_t));
	    rftidp->rfc_types[3] = 0x00;
	    rftidp->rfc_types[2] = 0x00;
	    rftidp->rfc_types[1] = 0x01;
	    rftidp->rfc_types[0] = 0x00;
	    BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: RFT ID payload length 0x%x:", BNXEF_DRIVER_NAME,
		dsp->instance, pktp->pkt_cmdlen);
	bnxef_big_endian(taskp->payloadp, pktp->pkt_cmdlen);


	return (BNXEF_SUCCESS);
}

int
bnxef_rpt_id_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0)
		ctp = kmem_zalloc(pktp->pkt_cmdlen, KM_SLEEP);
	ASSERT(ctp != NULL);

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	bnxef_big_endian(taskp->payloadp, pktp->pkt_cmdlen);

	return (BNXEF_SUCCESS);
}

int
bnxef_rspn_id_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;
	uint8_t			fillbytes = 0;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0) {
		if ((pktp->pkt_cmdlen % 4) != 0)
			fillbytes = 4 - (pktp->pkt_cmdlen % 4);
		ctp = kmem_zalloc(pktp->pkt_cmdlen + fillbytes, KM_SLEEP);
		ASSERT(ctp != NULL);
	}

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen+fillbytes;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	if (fillbytes > 0) {
		/* LINTED: alignment */
		((fc_frame_hdr_t *)taskp->frame_hdrp)->f_ctl |=
				(fillbytes & 0x3);
	}
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	bnxef_big_endian(taskp->payloadp, 20);

	return (BNXEF_SUCCESS);
}
int
bnxef_rsnn_nn_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_driver_state_t  *dsp;
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;
	uint8_t			fillbytes = 0;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0) {
		BNXEF_DEBUG(dsp, FCCT, -1, "%s%d: pkt->cmdlen 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_cmdlen);
		if ((pktp->pkt_cmdlen % 4) != 0)
			fillbytes = 4 - (pktp->pkt_cmdlen % 4);
		ctp = kmem_zalloc(pktp->pkt_cmdlen + fillbytes, KM_SLEEP);
		ASSERT(ctp != NULL);
	}

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen+fillbytes;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	if (fillbytes > 0) {
		/* LINTED: alignment */
		((fc_frame_hdr_t *)taskp->frame_hdrp)->f_ctl |=
				(fillbytes & 0x3);
	}
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	bnxef_big_endian(taskp->payloadp, 16);

	return (BNXEF_SUCCESS);
}

int
bnxef_da_id_tx_handler(bnxef_session_t *sessionp,
	bnxef_fcoe_task_t *taskp)
{
	bnxef_driver_state_t  *dsp;
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;
	uint8_t			fillbytes = 0;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0) {
		BNXEF_DEBUG(dsp, FCCT, -1, "%s%d: pkt->cmdlen 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_cmdlen);
		if ((pktp->pkt_cmdlen % 4) != 0)
			fillbytes = 4 - (pktp->pkt_cmdlen % 4);
		ctp = kmem_zalloc(pktp->pkt_cmdlen + fillbytes, KM_SLEEP);
		ASSERT(ctp != NULL);
	}

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen+fillbytes;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	if (fillbytes > 0) {
		/* LINTED: alignment */
		((fc_frame_hdr_t *)taskp->frame_hdrp)->f_ctl |=
				(fillbytes & 0x3);
	}
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	bnxef_big_endian(taskp->payloadp, pktp->pkt_cmdlen);

	return (BNXEF_SUCCESS);



}

int
bnxef_gpn_id_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_driver_state_t  *dsp;
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;
	uint8_t			fillbytes = 0;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0) {
		BNXEF_DEBUG(dsp, FCCT, -1, "%s%d: pkt->cmdlen 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_cmdlen);
		if ((pktp->pkt_cmdlen % 4) != 0)
			fillbytes = 4 - (pktp->pkt_cmdlen % 4);
		ctp = kmem_zalloc(pktp->pkt_cmdlen + fillbytes, KM_SLEEP);
		ASSERT(ctp != NULL);
	}

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen+fillbytes;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	if (fillbytes > 0) {
		/* LINTED: alignment */
		((fc_frame_hdr_t *)taskp->frame_hdrp)->f_ctl |=
				(fillbytes & 0x3);
	}
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	bnxef_big_endian(taskp->payloadp, pktp->pkt_cmdlen);

	return (BNXEF_SUCCESS);

}

int
bnxef_gid_pn_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_driver_state_t  *dsp;
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;
	uint8_t			fillbytes = 0;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0) {
		BNXEF_DEBUG(dsp, FCCT, -1, "%s%d: pkt->cmdlen 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_cmdlen);
		if ((pktp->pkt_cmdlen % 4) != 0)
			fillbytes = 4 - (pktp->pkt_cmdlen % 4);
		ctp = kmem_zalloc(pktp->pkt_cmdlen + fillbytes, KM_SLEEP);
		ASSERT(ctp != NULL);
	}

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen+fillbytes;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	if (fillbytes > 0) {
		/* LINTED: alignment */
		((fc_frame_hdr_t *)taskp->frame_hdrp)->f_ctl |=
				(fillbytes & 0x3);
	}
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);
	bnxef_big_endian(taskp->payloadp, 16);

	return (BNXEF_SUCCESS);
}

int
bnxef_ga_nxt_tx_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_driver_state_t  *dsp;
	bnxef_port_t		*portp;
	fc_packet_t		*pktp;
	fc_frame_hdr_t		*fp;
	fc_ct_header_t		*ctp;
	uint8_t			fillbytes = 0;

	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	pktp = taskp->pktp;
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	fp = kmem_zalloc(sizeof (fc_frame_hdr_t), KM_SLEEP);
	ASSERT(fp != NULL);
	if (pktp->pkt_cmdlen > 0) {
		BNXEF_DEBUG(dsp, FCCT, -1, "%s%d: pkt->cmdlen 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_cmdlen);
		if ((pktp->pkt_cmdlen % 4) != 0)
			fillbytes = 4 - (pktp->pkt_cmdlen % 4);
		ctp = kmem_zalloc(pktp->pkt_cmdlen + fillbytes, KM_SLEEP);
		ASSERT(ctp != NULL);
	}

	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	taskp->frame_hdrp = (uint8_t *)fp;

	if (pktp->pkt_cmdlen > 0) {
		taskp->task_flags |= BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
		taskp->payloadsz = pktp->pkt_cmdlen+fillbytes;
		taskp->payloadp  = (uint8_t *)ctp;
	}
	mutex_exit(&taskp->task_mutex);

	/*
	 * Copy the frame header in the big endian format conducive for
	 * for transmission as is.
	 */
	bcopy((uint8_t *)&pktp->pkt_cmd_fhdr, (uint8_t *)taskp->frame_hdrp,
			sizeof (fc_frame_hdr_t));
	if (fillbytes > 0) {
		/* LINTED: alignment */
		((fc_frame_hdr_t *)taskp->frame_hdrp)->f_ctl |=
				(fillbytes & 0x3);
	}
	bnxef_big_endian((uint8_t *)taskp->frame_hdrp, sizeof (fc_frame_hdr_t));

	/*
	 * copy the CT Header along with the command specific information.
	 */

	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)taskp->payloadp,
		(uint8_t *)pktp->pkt_cmd, pktp->pkt_cmdlen,
		DDI_DEV_AUTOINCR);

	bnxef_big_endian(taskp->payloadp, pktp->pkt_cmdlen);

	return (BNXEF_SUCCESS);
}


/*
 * This handles the transport of FC-CT exchanges. The FC-CT exchanges are
 * always transported using the non offloaded path in this driver.
 */
int
bnxef_fcct_transport_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp)
{
	bnxef_driver_state_t		 *dsp;
	bnxef_port_t			*portp;
	fc_packet_t			*pktp;
	mblk_t				*mblkp;
	timeout_id_t			tid;
	bnxef_tq_params_t		*tqp;
	fc_ct_header_t			cthdr;
	int ret;


	ASSERT(sessionp != NULL);
	ASSERT(taskp != NULL);

	portp = sessionp->lport;
	ASSERT(portp != NULL);

	dsp = portp->statep;
	ASSERT(dsp != NULL);

	pktp = taskp->pktp;
	ASSERT(pktp != NULL);

	/* Update the pkt_cmdfhdr with this OX_id allocated for this exchange */
	mutex_enter(&taskp->task_mutex);
	pktp->pkt_cmd_fhdr.ox_id = taskp->x_id;
	mutex_exit(&taskp->task_mutex);
	pktp->pkt_cmd_fhdr.rx_id = 0xffff;


	ddi_rep_get8(pktp->pkt_cmd_acc, (uint8_t *)&cthdr,
		(uint8_t *)pktp->pkt_cmd,
		sizeof (fc_ct_header_t), DDI_DEV_AUTOINCR);

	switch (cthdr.ct_cmdrsp) {
	case NS_RPN_ID:
		bnxef_rpn_id_tx_handler(sessionp, taskp);
		break;
	case NS_RNN_ID:
		bnxef_rnn_id_tx_handler(sessionp, taskp);
		break;
	case NS_RCS_ID:
		bnxef_rcs_id_tx_handler(sessionp, taskp);
		break;
	case NS_RFT_ID:
		bnxef_rft_id_tx_handler(sessionp, taskp);
		break;
	case NS_RPT_ID:
		bnxef_rpt_id_tx_handler(sessionp, taskp);
		break;
	case NS_RSPN_ID:
		bnxef_rspn_id_tx_handler(sessionp, taskp);
		break;
	case NS_RSNN_NN:
		bnxef_rsnn_nn_tx_handler(sessionp, taskp);
		break;
	case NS_GA_NXT:
		bnxef_ga_nxt_tx_handler(sessionp, taskp);
		break;
	case NS_GID_PN:
		bnxef_gid_pn_tx_handler(sessionp, taskp);
		break;
		case NS_GPN_ID:
		bnxef_gpn_id_tx_handler(sessionp, taskp);
		break;
	case NS_DA_ID:
		bnxef_da_id_tx_handler(sessionp, taskp);
		break;
	default: {
		fc_ct_header_t cthdrrsp;

		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: The FCCT command received 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, cthdr.ct_cmdrsp);
		cthdrrsp.ct_inid = 0x0;
		cthdrrsp.ct_rev = 0x01;
		cthdrrsp.ct_reserved1 = 0;
		cthdrrsp.ct_options = 0;
		cthdrrsp.ct_fcssubtype = 0x02;
		cthdrrsp.ct_fcstype = 0xFC;
		cthdrrsp.ct_aiusize = 0x0;
		cthdrrsp.ct_cmdrsp = 0x8001;
		cthdrrsp.ct_vendor = 0x0;
		cthdrrsp.ct_expln = 0x0;
		cthdrrsp.ct_reason = 0x0B;
		cthdrrsp.ct_reserved2 = 0;

		pktp->pkt_reason = cthdrrsp.ct_reason;
		bnxef_big_endian(&cthdrrsp, sizeof (fc_ct_header_t));
		ddi_rep_put8(pktp->pkt_resp_acc, (uint8_t *)&cthdrrsp,
			(uint8_t *)pktp->pkt_resp, sizeof (fc_ct_header_t),
			DDI_DEV_AUTOINCR);
		bnxef_complete_pkt(pktp, FC_PKT_FS_RJT, 0, pktp->pkt_reason, 0);

		BNXEF_STATS(dsp, ct_dispatched);

		return (FC_SUCCESS);
			}

	}

	if (sessionp != NULL) {
		/*
		 * There is already a session established. Include the task
		 * into session task list
		 */
		ret = bnxef_task_session_list_add(sessionp, taskp);
		if (ret != BNXEF_SUCCESS)
			return (FC_TRAN_BUSY);
	}

	/* Prepare the Mblk for transporting using L2 services */
	mblkp = bnxef_prepare_fcoe_mblk_from_pkt(dsp, portp, pktp);
	if (mblkp == NULL) {
		bnxef_task_session_list_remove(sessionp, taskp);
		return (FC_TRANSPORT_ERROR);
	}



	/*
	 * There is no session created, therefore task is not queued up in it.
	 * therefore, plogi should require special handling when timeout
	 * happens.
	 */
	tqp = &taskp->tq_params;
	tqp->dsp = dsp;
	tqp->taskp = taskp;
	tqp->sessp = sessionp;
	tqp->mp  = mblkp;
	tqp->type = BNXEF_CT_TIMEOUT_TYPE;
	tqp->portp = portp;

	tid = timeout(bnxef_timeout, tqp, BNXEF_SEC2TICK(pktp->pkt_timeout/2));
	mutex_enter(&taskp->task_mutex);
	taskp->timerhdl = tid;
	taskp->task_flags |= (BNXEF_FCOE_TASK_TIMER_ARMED |
			BNXEF_FCOE_TASK_SUBMITTED_L2);
	mutex_exit(&taskp->task_mutex);

	/* Transmit the Mblk prepared */
	if (!bnxef_send_frame(dsp, mblkp, dsp->fcf_ident->vlan_id)) {
		bnxef_task_session_list_remove(sessionp, taskp);
		return (FC_TRANSPORT_ERROR);
	    }
	/*
	 * The assumption is when the LS_ACC or LS_RJT comes for this
	 * handler of such frame will set all the status for return to
	 * higher level layer in the packet structure.
	 */
	if ((pktp->pkt_tran_flags & FC_TRAN_NO_INTR) == FC_TRAN_NO_INTR) {
		debug_enter("Going through Polled mode");
		/* Complete this exchange initiated in polled mode */
		bnxef_start_polling(dsp);
		mutex_enter(&taskp->task_mutex);
		/* Now wait for your event to show up */
		cv_wait(&taskp->task_cv, &taskp->task_mutex);
		/* If it wakes up */
		mutex_exit(&taskp->task_mutex);
		bnxef_stop_polling(dsp);
	}

	BNXEF_STATS(dsp, ct_dispatched);

	return (FC_SUCCESS);
}


/*
 * This handles the FC-CT frames received from external entities.
 */
void
bnxef_fcct_rx_handler(bnxef_port_t *portp, bnxef_session_t *sessionp,
	bnxef_fcoe_task_t *taskp, mblk_t *mp, int flags)
{
	timeout_id_t	tid;
	fc_packet_t	*pktp;
	bnxef_fcoe_frame_t	*fcoep;
	fc_frame_hdr_t		*fcp;
	uint8_t			*ctrsp;
	bnxef_driver_state_t	*dsp;
	int			size;
	fc_ct_header_t		*cthdrp;
	bnxef_fcoe_novlan_frame_t *fcoenovlanp;


	ASSERT(portp != NULL);
	ASSERT(sessionp != NULL);
	ASSERT(mp != NULL);
	ASSERT(taskp != NULL);

	dsp = portp->statep;
	ASSERT(dsp != NULL);

	pktp = taskp->pktp;

	/* Now the task is completed, shut off the timer enabled */
	mutex_enter(&taskp->task_mutex);
	if (taskp->task_flags & BNXEF_FCOE_TASK_TIMER_ARMED) {
		tid = taskp->timerhdl;
	}
	taskp->task_flags &= ~ BNXEF_FCOE_TASK_TIMER_ARMED;
	mutex_exit(&taskp->task_mutex);

	/* Cancel the timer */
	untimeout(tid);

	ASSERT(pktp->pkt_fca_private == taskp);

	BNXEF_DEBUG(dsp, FCCT, -1, "%s%d: Flags 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, flags);

	if (flags) {
		/* LINTED: alignment */
		fcoep = (bnxef_fcoe_frame_t *)mp->b_rptr;
		ASSERT(fcoep != NULL);
		/* LINTED: alignment */
		fcp = (fc_frame_hdr_t *)fcoep->desc;
		ASSERT(fcp != NULL);
	} else {
		/* LINTED: alignment */
		fcoenovlanp = (bnxef_fcoe_novlan_frame_t *)mp->b_rptr;
		ASSERT(fcoenovlanp != NULL);
		/* LINTED: alignment */
		fcp = (fc_frame_hdr_t *)fcoenovlanp->desc;
		ASSERT(fcp != NULL);
	}

	ctrsp = (uint8_t *)(fcp +1);
	/* This is the FC Payload size received */
	size = (mp->b_wptr-8) - ctrsp;

	ddi_rep_put8(pktp->pkt_resp_acc, (uint8_t *)ctrsp,
		(uint8_t *)pktp->pkt_resp, size, DDI_DEV_AUTOINCR);

	if (sessionp != NULL) {
		/* Remove the task from session list */
		bnxef_task_session_list_remove(sessionp, taskp);
	}

	/* LINTED: alignment */
	cthdrp = (fc_ct_header_t *)ctrsp;
	bnxef_big_endian(cthdrp, sizeof (fc_ct_header_t));

	if (cthdrp->ct_cmdrsp == FS_ACC_IU) {
		BNXEF_DEBUG(dsp, ERROR, -1, "%s%d: FS_ACC received 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
		pktp->pkt_resp_resid = 0;
		/* Now complete the pkt and return */
		bnxef_complete_pkt(pktp, FC_PKT_SUCCESS, 0, 0, 0);
	} else {
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: FS_RJT reason 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, cthdrp->ct_reason);
		pktp->pkt_reason = cthdrp->ct_reason;
		bnxef_complete_pkt(pktp, FC_PKT_FS_RJT, 0,
			pktp->pkt_reason, cthdrp->ct_expln);
	}

	BNXEF_STATS(dsp, l2_fcoe_rx_free);
	BNXEF_STATS(dsp, l2_rx_free);
	/* Free the message block */
	freemsg(mp);
}
