/*
 * The information contained in this file is confidential and proprietary to
 * Broadcom Corporation.  No part of this file may be reproduced or
 * distributed, in any form or by any means for any purpose, without the
 * express written permission of Broadcom Corporation.
 *
 * (c) COPYRIGHT 2010-2012 Broadcom Corporation, ALL RIGHTS RESERVED.
 */

#pragma ident "(C) 2010-2012 Broadcom Corporation "
#pragma ident "$File$$Revision$$Change$$DateTime$"

#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/stat.h>
#include <sys/pci.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>
#include <sys/cmn_err.h>
#include <sys/file.h>
#include <sys/cred.h>
#include <sys/byteorder.h>
#include <sys/atomic.h>
#include <sys/scsi/scsi.h>
#include <sys/modhash.h>
#include <sys/sunldi.h>
#include <sys/kstat.h>
#include <sys/kmem.h>
#include <sys/ethernet.h>
#include <sys/stream.h>
#include <sys/sunldi.h>
/*
 * leadville header files
 */
#include <sys/fibre-channel/fc_types.h>
#include <sys/fibre-channel/impl/fc_fcaif.h>
#include <sys/fibre-channel/ulp/fcp.h>
#include <bnxe_binding.h>
#include <bnxef_tq_param.h>
#include <bnxef_fcf_info.h>
#include <bnxef_hsi.h>
#include <bnxef_fcoe_task.h>
#include <bnxef_fcoe_session.h>
#include <bnxef_state.h>
#include <bnxef_fip.h>
#include <bnxef_fcoe.h>
#include <bnxef_debug.h>
#include <bnxef_seq_recov.h>
#include <57xx_fcoe_constants.h>

extern bnxef_fcoe_task_t *bnxef_task_alloc(bnxef_driver_state_t *, int flags);
extern int bnxef_pkt_cmd_res_alloc(bnxef_tq_params_t *tqp, fc_packet_t *pktp);
extern void bnxef_complete_pkt(fc_packet_t *pktp, uint8_t state,
		uint8_t action, uint32_t reason, uint32_t expln);
extern int bnxef_pkt_resp_res_alloc(bnxef_tq_params_t *tqp, fc_packet_t *pktp);
extern void bnxef_pkt_cmd_res_free(fc_packet_t *pktp, int stage);
extern void bnxef_pkt_resp_res_free(fc_packet_t *pktp,  int stage);
extern int bnxef_init_mp_task(bnxef_session_t *sessionp, fc_packet_t *pktp);
extern void bnxef_add_2_sq(bnxef_session_t *sessionp, uint16_t xid);
extern void bnxef_ring_doorbell(bnxef_session_t *sessionp);
extern void bnxef_big_endian(void *bufp, int size);
extern int bnxef_cleanup_task(bnxef_session_t *sessionp,
	bnxef_fcoe_task_t *taskp);
extern int bnxef_initiate_abts(bnxef_session_t *sessionp,
	bnxef_fcoe_task_t *taskp);
extern void bnxef_timeout(void *arg);
extern int bnxef_init_task(bnxef_session_t *sessionp, fc_packet_t *pktp);
extern int bnxef_task_bd_table_free(fc_packet_t *pktp, uint32_t stage);
extern void
bnxef_fcoe_task_dec_ref_cnt(bnxef_driver_state_t *dsp,
	bnxef_fcoe_task_t *taskp);
extern void
bnxef_fcoe_task_inc_ref_cnt(bnxef_fcoe_task_t *taskp);

void bnxef_rec_timeout(void *arg);
void bnxef_rec_timeout_handler(void *arg);
void bnxef_srr_timeout(void *arg);
void bnxef_srr_timeout_handler(void *arg);
void bnxef_cleanup_sequence(bnxef_session_t *sessionp,
	bnxef_fcoe_task_t *taskp, uint32_t reloff);


/*
 * This file contains all the code needed to support
 * the Sequence recovery for tapes.
 */

void
bnxef_send_rec(void *arg)
{
	bnxef_session_t	 *sessionp;
	bnxef_port_t		 *portp;
	bnxef_port_t		 *rportp;
	bnxef_driver_state_t	 *dsp;
	bnxef_fcoe_task_t		*taskp;
	bnxef_fcoe_task_t		*rectaskp;
	fc_packet_t			*pktp;
	la_els_rec_t			recpayload;
	bnxef_tq_params_t	*tqp;
	int ret;
	timeout_id_t		tid;
	struct fcoe_task_ctx_entry 	*fwtaskp;
	uint16_t		rx_id;

	tqp = (bnxef_tq_params_t *)arg;
	ASSERT(tqp != NULL);
	sessionp = tqp->sessp;
	portp = tqp->portp;
	dsp	= tqp->dsp;
	taskp = tqp->taskp;
	fwtaskp = taskp->fwtaskp;

	ASSERT(sessionp != NULL);
	ASSERT(portp != NULL);
	ASSERT(dsp != NULL);
	ASSERT(taskp != NULL);

	rportp = sessionp->rport;
	ASSERT(rportp != NULL);


	if (dsp->physical_port_state & BNXEF_PHYPORT_OFFLINE) {
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Physical port offline, cannot continue",
		BNXEF_DRIVER_NAME, dsp->instance);
		return;
	}

	if ((taskp->task_flags & BNXEF_FCOE_TASK_ALLOCATED) == 0) {
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Already I/O Was completed",
		BNXEF_DRIVER_NAME, dsp->instance);
		return;
	}
	if ((taskp->sec_flags & BNXEF_REC_SENT)) {
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Already REC is in progress",
		BNXEF_DRIVER_NAME, dsp->instance);
		return;
	}

	if ((taskp->sec_flags & BNXEF_SRR_SENT)) {
		BNXEF_DEBUG(dsp, ERROR, -1,
		"SRR In progress, REC not sent",
		BNXEF_DRIVER_NAME, dsp->instance);
		return;
	}

	BNXEF_DEBUG(dsp, ERROR, -1, "%s%d: Sending REC for TAsk 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
	pktp = kmem_zalloc(sizeof (fc_packet_t), KM_SLEEP);
	ASSERT(pktp != NULL);
	pktp->pkt_cmd_fhdr.d_id = rportp->id;
	pktp->pkt_cmd_fhdr.s_id = portp->id;
	pktp->pkt_cmd_fhdr.type = FC_TYPE_EXTENDED_LS;
	pktp->pkt_cmd_fhdr.rx_id = 0xffff;
	pktp->pkt_cmd_fhdr.r_ctl = R_CTL_ELS_REQ;
	pktp->pkt_cmd_fhdr.ro	=  0;
	pktp->pkt_cmd_fhdr.f_ctl = F_CTL_FIRST_SEQ | F_CTL_END_SEQ |
				F_CTL_SEQ_INITIATIVE;
	pktp->pkt_timeout	 =  2 * BNXEF_RA_TOV;
	pktp->pkt_tran_flags |= (FC_TRAN_CLASS3 | FC_TRAN_NO_INTR);
	pktp->pkt_cmdlen = sizeof (la_els_rec_t);
	pktp->pkt_rsplen = sizeof (la_els_rec_reply_t);

	rectaskp = bnxef_task_alloc(dsp, KM_SLEEP);
	ASSERT(rectaskp != NULL);

	rectaskp->pktp = pktp;
	rectaskp->task_flags |= BNXEF_FCOE_TASK_REC | BNXEF_FCOE_TASK_ELS;

	pktp->pkt_fca_private = rectaskp;
	pktp->pkt_cmd_fhdr.ox_id =  rectaskp->x_id;
	memset(&recpayload, 0, sizeof (la_els_rec_t));
	recpayload.ls_code = LA_ELS_REC;
	recpayload.s_id		= htonl(portp->id);
	recpayload.ox_id = htons(taskp->x_id);
	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: The Rx ID received from the Task context 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance,
		htons(fwtaskp->rxwr_txrd.var_ctx.rx_id));
	rx_id = LE16_TO_CPU(fwtaskp->rxwr_txrd.var_ctx.rx_id);
	if (sessionp->state & BNXEF_SESSION_OFFLOADED)
		recpayload.rx_id = htons(rx_id);
	else
		recpayload.rx_id = 0xffff;

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: pktp->pkt_resp_len 0x%x before cmd_res_alloc",
		BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_rsplen);
	ret = bnxef_pkt_cmd_res_alloc(tqp, pktp);
	if (ret != BNXEF_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to allocate packet cmd resources\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		bnxef_task_free(dsp, rectaskp);
		kmem_free(pktp, sizeof (fc_packet_t));
		/* For the Original Packet, silently tell Abort Completed */
		bnxef_task_session_list_remove(taskp->sessionp, taskp);
		bnxef_complete_pkt(taskp->pktp, FC_PKT_TRAN_ERROR,
			0, FC_REASON_ABORT_FAILED, 0);
		return;
	}

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: pktp->pkt_resp_len 0x%x before ddi_rep_put8 ",
		BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_rsplen);
	/* copy the rec payload data into cmd DMA buffer */
	ddi_rep_put8(pktp->pkt_cmd_acc, (uint8_t *)&recpayload,
		(uint8_t *)pktp->pkt_cmd, sizeof (la_els_rec_t),
		DDI_DEV_AUTOINCR);

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: pktp->pkt_resp_len 0x%x after ddi_rep_put8 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_rsplen);
	ret = bnxef_pkt_resp_res_alloc(tqp, pktp);
	if (ret != BNXEF_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to allocate pkt resp resources for REC\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		bnxef_pkt_cmd_res_free(pktp, BNXEF_PKT_CMD_ALL);
		bnxef_task_free(dsp, rectaskp);
		kmem_free(pktp, sizeof (fc_packet_t));
		/* For the Original Packet, silently tell Abort Completed */
		bnxef_task_session_list_remove(taskp->sessionp, taskp);
		bnxef_complete_pkt(taskp->pktp, FC_PKT_TRAN_ERROR,
				0, FC_REASON_ABORT_FAILED, 0);
		return;
	}

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: REC PAcket Set up with Required information",
		BNXEF_DRIVER_NAME, dsp->instance);
	/*
	 * We can also get the original X_ID from REC payload , but done
	 * this way here.
	 */
	rectaskp->task_flags |= BNXEF_FCOE_SEC_X_ID_VALID;
	rectaskp->secondary_x_id = taskp->x_id; /* Back link to original task */
	taskp->sec_flags |= BNXEF_REC_SENT;

	if (dsp->physical_port_state & BNXEF_PHYPORT_OFFLINE) {
		/*
		 * Now if the Link has gone down, do not set the timeout
		 * for RRQ donot set the RRQ out as well.
		 */
		bnxef_pkt_cmd_res_free(pktp, BNXEF_PKT_CMD_ALL);
		bnxef_pkt_resp_res_free(pktp, BNXEF_PKT_RESP_ALL);
		bnxef_task_free(dsp, rectaskp);
		kmem_free(pktp, sizeof (fc_packet_t));
		/* For the Original Packet, silently tell Abort Completed */
		bnxef_task_session_list_remove(taskp->sessionp, taskp);
		bnxef_complete_pkt(taskp->pktp, FC_PKT_TRAN_ERROR,
				0, FC_REASON_ABORT_FAILED, 0);
		return;
	}


	/* ARM the timer to catch run away condition */
	tqp = &rectaskp->tq_params;
	tqp->dsp = dsp;
	tqp->taskp = rectaskp;
	tqp->sessp = sessionp;
	tqp->mp  = NULL;
	tqp->type = BNXEF_REC_TIMEOUT_TYPE;
	tqp->portp = portp;

	tid = timeout(bnxef_rec_timeout, tqp,
			BNXEF_SEC2TICK(pktp->pkt_timeout));


	if (sessionp->state & BNXEF_SESSION_OFFLOADED) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Initializing the Task structure for REC",
			BNXEF_DRIVER_NAME, dsp->instance);
		/* Init mp task */
		bnxef_init_mp_task(sessionp, pktp);

		/* Update the SQ to start this */
		bnxef_add_2_sq(sessionp, rectaskp->x_id);

		mutex_enter(&rectaskp->task_mutex);
		rectaskp->timerhdl = tid;
		rectaskp->task_flags |= (BNXEF_FCOE_TASK_TIMER_ARMED |
					BNXEF_FCOE_SUBMITTED_TO_FIRMWARE);
		mutex_exit(&rectaskp->task_mutex);

		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Ringing the door bell to send REC",
			BNXEF_DRIVER_NAME, dsp->instance);

		bnxef_fcoe_task_inc_ref_cnt(taskp);
		ret = bnxef_task_session_list_add(sessionp, rectaskp);
		if (ret != BNXEF_SUCCESS) {
			debug_enter("Session Cleanup During REC sending");
		}

		/* Now ring the door bell register appropriately */
		bnxef_ring_doorbell(sessionp);
	} else
		ASSERT(0);

}


void
bnxef_send_srr(void *arg)
{
	bnxef_session_t	 *sessionp;
	bnxef_port_t		 *portp;
	bnxef_port_t		 *rportp;
	bnxef_driver_state_t	 *dsp;
	bnxef_fcoe_task_t		*taskp;
	bnxef_fcoe_task_t		*srrtaskp;
	fc_packet_t			*pktp;
	la_fcpls_srr_t			srrpayload;
	bnxef_tq_params_t	*tqp;
	int ret;
	timeout_id_t		tid;
	struct fcoe_task_ctx_entry 	*fwtaskp;
	uint32_t		reloff;
	uint8_t			rctliu;

	tqp = (bnxef_tq_params_t *)arg;
	ASSERT(tqp != NULL);
	sessionp = tqp->sessp;
	portp = tqp->portp;
	dsp	= tqp->dsp;
	taskp = tqp->taskp;
	fwtaskp = taskp->fwtaskp;
	reloff = tqp->reserved1;
	rctliu = tqp->reserved2;

	ASSERT(sessionp != NULL);
	ASSERT(portp != NULL);
	ASSERT(dsp != NULL);
	ASSERT(taskp != NULL);

	rportp = sessionp->rport;
	ASSERT(rportp != NULL);

	if (dsp->physical_port_state & BNXEF_PHYPORT_OFFLINE) {
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Physical port offline, cannot continue",
		BNXEF_DRIVER_NAME, dsp->instance);
		return;
	}

	if ((taskp->task_flags & BNXEF_FCOE_TASK_ALLOCATED) == 0) {
		BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Already I/O Was completed",
		BNXEF_DRIVER_NAME, dsp->instance);
		return;
	}
	ASSERT((taskp->sec_flags & BNXEF_REC_SENT) == 0);

	BNXEF_DEBUG(dsp, ERROR, -1, "%s%d: Sending SRR for TAsk 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
	pktp = kmem_zalloc(sizeof (fc_packet_t), KM_SLEEP);
	ASSERT(pktp != NULL);
	pktp->pkt_cmd_fhdr.d_id = rportp->id;
	pktp->pkt_cmd_fhdr.s_id = portp->id;
	pktp->pkt_cmd_fhdr.type = FC_TYPE_SCSI_FCP;
	pktp->pkt_cmd_fhdr.rx_id = 0xffff;
	pktp->pkt_cmd_fhdr.r_ctl = R_CTL_FC4_SVC | R_CTL_UNSOL_CONTROL;
	pktp->pkt_cmd_fhdr.ro	=  0;
	pktp->pkt_cmd_fhdr.f_ctl = F_CTL_FIRST_SEQ | F_CTL_END_SEQ |
				F_CTL_SEQ_INITIATIVE;
	pktp->pkt_timeout	 =  2 * BNXEF_RA_TOV;
	pktp->pkt_tran_flags |= (FC_TRAN_CLASS3 | FC_TRAN_NO_INTR);
	pktp->pkt_cmdlen = sizeof (la_fcpls_srr_t);
	pktp->pkt_rsplen = sizeof (la_fcpls_srr_reply_t);

	srrtaskp = bnxef_task_alloc(dsp, KM_SLEEP);
	ASSERT(srrtaskp != NULL);

	srrtaskp->pktp = pktp;
	srrtaskp->task_flags |= BNXEF_FCOE_TASK_SRR | BNXEF_FCOE_TASK_ELS;
	srrtaskp->orig_payloadp = NULL;

	pktp->pkt_fca_private = srrtaskp;
	pktp->pkt_cmd_fhdr.ox_id =  srrtaskp->x_id;
	memset(&srrpayload, 0, sizeof (la_fcpls_srr_t));
	srrpayload.ls_code = LA_FCPLS_SRR;
	srrpayload.ox_id = htons(taskp->x_id);
	srrpayload.reloff = htonl(reloff);
	srrpayload.rctl = rctliu;

	if (sessionp->state & BNXEF_SESSION_OFFLOADED)
		srrpayload.rx_id =
			htons(LE16_TO_CPU(fwtaskp->rxwr_txrd.var_ctx.rx_id));
	else
		srrpayload.rx_id = 0xffff;

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: pktp->pkt_resp_len 0x%x before cmd_res_alloc",
		BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_rsplen);
	ret = bnxef_pkt_cmd_res_alloc(tqp, pktp);
	if (ret != BNXEF_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to allocate packet cmd resources\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		bnxef_task_free(dsp, srrtaskp);
		kmem_free(pktp, sizeof (fc_packet_t));
		/* For the Original Packet, silently tell Abort Completed */
		bnxef_task_session_list_remove(taskp->sessionp, taskp);
		bnxef_complete_pkt(taskp->pktp, FC_PKT_TRAN_ERROR,
				0, FC_REASON_ABORT_FAILED, 0);
		return;
	}

	BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: pktp->pkt_resp_len 0x%x before ddi_rep_put8 ",
		BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_rsplen);
	/* copy the  payload data into cmd DMA buffer */
	ddi_rep_put8(pktp->pkt_cmd_acc, (uint8_t *)&srrpayload,
		(uint8_t *)pktp->pkt_cmd, sizeof (la_fcpls_srr_t),
		DDI_DEV_AUTOINCR);

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: pktp->pkt_resp_len 0x%x after ddi_rep_put8 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, pktp->pkt_rsplen);
	ret = bnxef_pkt_resp_res_alloc(tqp, pktp);
	if (ret != BNXEF_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to allocate pkt resp resources for SRR\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		bnxef_pkt_cmd_res_free(pktp, BNXEF_PKT_CMD_ALL);
		bnxef_task_free(dsp, srrtaskp);
		kmem_free(pktp, sizeof (fc_packet_t));
		/* For the Original Packet, silently tell Abort Completed */
		bnxef_task_session_list_remove(taskp->sessionp, taskp);
		bnxef_complete_pkt(taskp->pktp, FC_PKT_TRAN_ERROR,
				0, FC_REASON_ABORT_FAILED, 0);
		return;
	}

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: RRQ PAcket Set up with Required information",
		BNXEF_DRIVER_NAME, dsp->instance);
	/*
	 * We can also get the original X_ID from RRQ payload , but done
	 * this way here.
	 */
	srrtaskp->task_flags |= BNXEF_FCOE_SEC_X_ID_VALID;
	srrtaskp->secondary_x_id = taskp->x_id; /* Back link to original task */
	taskp->sec_flags |= BNXEF_SRR_SENT;

	if (dsp->physical_port_state & BNXEF_PHYPORT_OFFLINE) {
		/*
		 * Now if the Link has gone down, do not set the timeout
		 * for RRQ donot set the RRQ out as well.
		 */
		bnxef_pkt_cmd_res_free(pktp, BNXEF_PKT_CMD_ALL);
		bnxef_pkt_resp_res_free(pktp, BNXEF_PKT_RESP_ALL);
		bnxef_task_free(dsp, srrtaskp);
		kmem_free(pktp, sizeof (fc_packet_t));
		/* For the Original Packet, silently tell Abort Completed */
		bnxef_task_session_list_remove(taskp->sessionp, taskp);
		bnxef_complete_pkt(taskp->pktp, FC_PKT_TRAN_ERROR,
			0, FC_REASON_ABORT_FAILED, 0);
		return;
	}

	/* Set the original Task's timer - Again */
	tqp = &taskp->tq_params;
	tqp->dsp = dsp;
	tqp->taskp = taskp;
	tqp->sessp = sessionp;
	tqp->portp = portp;
	tqp->mp = NULL;
	tqp->type = BNXEF_SCSI_TIMEOUT_TYPE;
	tid = timeout(bnxef_timeout, tqp,
			BNXEF_SEC2TICK(taskp->pktp->pkt_timeout));
	mutex_enter(&taskp->task_mutex);
	taskp->timerhdl = tid;
	taskp->task_flags |= (BNXEF_FCOE_TASK_TIMER_ARMED |
				BNXEF_FCOE_SUBMITTED_TO_FIRMWARE);
	mutex_exit(&taskp->task_mutex);

	/* ARM the timer to catch run away condition */
	tqp = &srrtaskp->tq_params;
	tqp->dsp = dsp;
	tqp->taskp = srrtaskp;
	tqp->sessp = sessionp;
	tqp->mp  = NULL;
	tqp->type = BNXEF_SRR_TIMEOUT_TYPE;
	tqp->portp = portp;

	tid = timeout(bnxef_srr_timeout, tqp,
		BNXEF_SEC2TICK(pktp->pkt_timeout));

	if (sessionp->state & BNXEF_SESSION_OFFLOADED) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Initializing the Task structure for SRR",
			BNXEF_DRIVER_NAME, dsp->instance);
		/* Init mp task */
		bnxef_init_mp_task(sessionp, pktp);

		/* Update the SQ to start this */
		bnxef_add_2_sq(sessionp, srrtaskp->x_id);

		mutex_enter(&srrtaskp->task_mutex);
		srrtaskp->timerhdl = tid;
		srrtaskp->task_flags |= (BNXEF_FCOE_TASK_TIMER_ARMED |
					BNXEF_FCOE_SUBMITTED_TO_FIRMWARE);
		mutex_exit(&srrtaskp->task_mutex);

		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Ringing the door bell to send SRR",
			BNXEF_DRIVER_NAME, dsp->instance);

		bnxef_fcoe_task_inc_ref_cnt(taskp);
		ret = bnxef_task_session_list_add(sessionp, srrtaskp);
		if (ret != BNXEF_SUCCESS) {
			debug_enter("Session Cleanup During SRR sending");
		}

		/* Now ring the door bell register appropriately */
		bnxef_ring_doorbell(sessionp);
	} else
		ASSERT(0);

}


/*
 * This handles the LS_ACC received for REC sent out.
 */
void
bnxef_mp_rec_ls_acc_rx_handler(bnxef_driver_state_t *dsp, fc_packet_t *pktp)
{
	bnxef_fcoe_task_t *rectaskp;
	la_els_rec_reply_t *recreplyp;
	/* The original task which started this Sequence recovery */
	bnxef_fcoe_task_t *otaskp;
	fc_packet_t	*opktp;
	struct fcoe_err_report_entry *errp;
	bnxef_session_t *sessionp;
	uint32_t	 reloff;
	uint8_t		rctliu;
	bnxef_tq_params_t *tqp;
	bnxef_port_t *portp;
	uint8_t		seq_cleanup_comp = 0;


	ASSERT(dsp != NULL);
	ASSERT(pktp != NULL);

	rectaskp = (bnxef_fcoe_task_t *)pktp->pkt_fca_private;
	bnxef_big_endian(pktp->pkt_resp, 4);

	sessionp = rectaskp->sessionp;
	ASSERT(sessionp != NULL);

	portp = (bnxef_port_t *)sessionp->lport;
	ASSERT(portp != NULL);

	rectaskp->orig_payloadp = NULL;
	/* Do the byte swapping ritual for x86 platform */
	/* LINTED: alignment */
	recreplyp = (la_els_rec_reply_t *)pktp->pkt_resp;
	recreplyp->ox_id = htons(recreplyp->ox_id);
	recreplyp->rx_id = htons(recreplyp->rx_id);
	recreplyp->org_addr_id = htonl(recreplyp->org_addr_id);
	recreplyp->resp_addr_id = htonl(recreplyp->resp_addr_id);
	recreplyp->fc4value = htonl(recreplyp->fc4value);
	recreplyp->estat = htonl(recreplyp->estat);
	BNXEF_DEBUG(dsp, ERROR, -1, "%s%d: Estat received from Target 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, recreplyp->estat);
	otaskp = dsp->taskarray[rectaskp->secondary_x_id];

	/*
	 * If the sequence initiative is not held by target,
	 * we have work to do
	 */
	if ((recreplyp->estat & SEQUENCE_INITIATIVE_HELD_BY_TARGET) == 0) {
			if ((otaskp->task_flags &
				BNXEF_FCOE_TASK_COMPLETED) == 0) {
			ASSERT(rectaskp->task_flags &
				BNXEF_FCOE_SEC_X_ID_VALID);
			otaskp->rec_timeout_cnt = 0;
			errp = (struct fcoe_err_report_entry *)&otaskp->error;
			ASSERT(otaskp->task_flags &  BNXEF_FCOE_TASK_SCSI);
			if (otaskp->sec_flags & BNXEF_REC_SENT)
				otaskp->sec_flags &= ~BNXEF_REC_SENT;
			opktp = otaskp->pktp;
			if (opktp == NULL) {
				/*
				 * I/O is complete donot
				 * check anything-return
				 */
				bnxef_fcoe_task_dec_ref_cnt(dsp, otaskp);
				return;
			}
			/* Check whether it is a Write SCSI command */
			if (opktp->pkt_tran_type == FC_PKT_FCP_WRITE) {
				if (recreplyp->fc4value ==
					opktp->pkt_datalen) {
					rctliu = 0x7;
					/* FCP Reponse-to be retransmitted */
					reloff = 0;
				} else {
					/*
					 * We have to start transmitting from
					 * the offset represented by fc4value.
					 */
					bnxef_cleanup_sequence(sessionp,
						otaskp, recreplyp->fc4value);
					seq_cleanup_comp = 1;
					reloff = recreplyp->fc4value;
					rctliu = 0x5; /* XFER_RDY */
				}
			} else if (opktp->pkt_tran_type == FC_PKT_FCP_READ) {
				if (LE32_TO_CPU(errp->data.rx_buf_off) ==
					opktp->pkt_datalen) {
					rctliu = 0x7; /* FCP Response */
					reloff = 0;
				} else {
					/*
					 * We have to request retransmission
					 * starting from this offset
					 */
					bnxef_cleanup_sequence(sessionp,
						otaskp,
					LE32_TO_CPU(errp->data.rx_buf_off));
					rctliu = 0x01; /* Data IU */
					reloff =
					LE32_TO_CPU(errp->data.rx_buf_off);
					seq_cleanup_comp = 1;
				}
			}
			/*
			 * Now send the SRR to get the
			 * retransmission started
			 * for teh original task
			 */
			tqp = &otaskp->tq_params;
			tqp->dsp = dsp;
			tqp->taskp = otaskp;
			tqp->sessp = sessionp;
			tqp->portp = portp;
			tqp->reserved1 = reloff;
			tqp->reserved2 = rctliu;

			if (!seq_cleanup_comp)
				bnxef_send_srr(tqp);
		}

	}
	bnxef_fcoe_task_dec_ref_cnt(dsp, otaskp);
}


void
bnxef_mp_rec_ls_rjt_rx_handler(bnxef_driver_state_t *dsp, fc_packet_t *pktp)
{
	la_els_rjt_t *rjtp;
	bnxef_fcoe_task_t *rectaskp;
	bnxef_fcoe_task_t *otaskp;
	struct fcoe_err_report_entry *errp;
	fc_packet_t *opktp;
	bnxef_session_t *sessionp;
	bnxef_tq_params_t *tqp;
	timeout_id_t	tid;
	int ret;
			    fcp_cmd_t		cmd;


	ASSERT(dsp != NULL);
	ASSERT(pktp != NULL);

	bnxef_big_endian(pktp->pkt_resp, 4);
	/* LINTED: alignment */
	rjtp = (la_els_rjt_t *)pktp->pkt_resp;
	ASSERT(rjtp != NULL);

	rectaskp = (bnxef_fcoe_task_t *)pktp->pkt_fca_private;
	ASSERT(rectaskp != NULL);

	sessionp = rectaskp->sessionp;
	ASSERT(sessionp != NULL);
	ASSERT(rectaskp->task_flags & BNXEF_FCOE_SEC_X_ID_VALID);
	otaskp =  dsp->taskarray[rectaskp->secondary_x_id];
	ASSERT(otaskp != NULL);
	opktp = otaskp->pktp;
	ASSERT(opktp != NULL);
	/* This count is used only when it times out */
	otaskp->rec_timeout_cnt = 0;


	/* Decrement the reference count */
	bnxef_fcoe_task_dec_ref_cnt(dsp, otaskp);

	if (((rjtp->reason == BNXEF_LA_RJT_LOGICAL_ERR) ||
		(rjtp->reason == BNXEF_LA_RJT_UNABLE_TO_PERFORM)) &&
			(rjtp->reserved == 0x17)) {
		if (rectaskp->task_flags & BNXEF_FCOE_SEC_X_ID_VALID) {
			/* Check whether the error rx_buf and tx_buf are zero */
			errp = &otaskp->error;
			if ((errp->data.tx_buf_off == 0) &&
				(errp->data.rx_buf_off == 0)) {
				/* This means the COMMAND IU is dropped */
				ret = bnxef_cleanup_task(sessionp, otaskp);
						ASSERT(ret == BNXEF_SUCCESS);

				if (dsp->task_bd_table_pre_alloc == 0) {
					/*
					 * Free the BD descriptors
					 * previously allocated
					 */
					if ((otaskp->task_flags &
					BNXEF_FCOE_TASK_BD_LIST_ALLOC) ==
					BNXEF_FCOE_TASK_BD_LIST_ALLOC) {
						ret = bnxef_task_bd_table_free(
						pktp,
						BNXEF_TASK_BD_TABLE_ALL);
						ASSERT(ret == BNXEF_SUCCESS);
					}
				}

				if (sessionp) {
					bnxef_task_session_list_remove(
						sessionp,
						otaskp);
				}

				/*
				 * Free the Old task, we need to try
				 * with new task Id
				 */
				bnxef_task_free(dsp, otaskp);

				/* Allocate the new Task */
				otaskp = bnxef_task_alloc(dsp, KM_SLEEP);
				ASSERT(otaskp != NULL);

				/*
				 * Set the mapping properly
				 * to the Original packet
				 */
				opktp->pkt_fca_private = (void *)otaskp;
				otaskp->pktp = opktp;

				/*
				 * Now check whether this command is just
				 * SCSI READ/WRITE or Task
				 * management command. For task management the
				 * transport method is
				 * different.
				 */
				ddi_rep_get8(opktp->pkt_cmd_acc,
					(uint8_t *)&cmd,
					(uint8_t *)opktp->pkt_cmd,
					sizeof (fcp_cmd_t),
					DDI_DEV_AUTOINCR);

				mutex_enter(&otaskp->task_mutex);
				if (cmd.fcp_cntl.cntl_kill_tsk ||
					cmd.fcp_cntl.cntl_clr_aca ||
					cmd.fcp_cntl.cntl_reset_tgt ||
					cmd.fcp_cntl.cntl_reset_lun ||
					cmd.fcp_cntl.cntl_clr_tsk) {
					otaskp->task_flags |=
						BNXEF_FCOE_TASK_MGMT_CMD;
				} else {
					otaskp->task_flags |=
						BNXEF_FCOE_TASK_SCSI;
				}
				/*
				 * Update the pkt_cmdfhdr with this
				 * Ox_id allocated for this exchange
				 */
				opktp->pkt_cmd_fhdr.ox_id = otaskp->x_id;
				mutex_exit(&otaskp->task_mutex);
				opktp->pkt_cmd_fhdr.rx_id = 0xffff;


				/* re-initialize the same task */
				if (otaskp->task_flags &
					BNXEF_FCOE_TASK_MGMT_CMD)
					bnxef_init_mp_task(sessionp, opktp);
				else
					bnxef_init_task(sessionp, opktp);

				if (sessionp) {
					ret =
					bnxef_task_session_list_add(sessionp,
					otaskp);
				}
				/* Re-arm the timer */
				tqp = &otaskp->tq_params;
				/* ARM the timer to catch run away condition */
				tqp->dsp = dsp;
				tqp->taskp = otaskp;
				tqp->sessp = sessionp;
				tqp->portp = sessionp->lport;
				tqp->mp  = NULL;
				tqp->type = BNXEF_SCSI_TIMEOUT_TYPE;

				tid = timeout(bnxef_timeout,
					tqp,
					BNXEF_SEC2TICK(opktp->pkt_timeout));
				mutex_enter(&otaskp->task_mutex);
				otaskp->timerhdl = tid;
				otaskp->task_flags |=
					(BNXEF_FCOE_TASK_TIMER_ARMED |
					BNXEF_FCOE_SUBMITTED_TO_FIRMWARE);
				mutex_exit(&otaskp->task_mutex);

				/*
				 * Re-submit the same task on
				 * the SQ and ring door bell
				 */
				/* Now add the task XID to the send Q */
				bnxef_add_2_sq(sessionp, otaskp->x_id);
				/*
				 * Now ring the door bell
				 * egister appropriately
				 */
				bnxef_ring_doorbell(sessionp);
			}
		}
	}

}



void
bnxef_mp_srr_ls_acc_rx_handler(bnxef_driver_state_t *dsp, fc_packet_t *pktp)
{
	ASSERT(dsp != NULL);
	ASSERT(pktp != NULL);
	bnxef_fcoe_task_t *taskp, *otaskp;

	taskp = (bnxef_fcoe_task_t *)pktp->pkt_fca_private;
	otaskp = dsp->taskarray[taskp->secondary_x_id];
	ASSERT(otaskp != NULL);


	/* Nothing to be done */
	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: MP SRR LS_ACC Received",
		BNXEF_DRIVER_NAME, dsp->instance);
	bnxef_fcoe_task_dec_ref_cnt(dsp, otaskp);

}

void
bnxef_mp_srr_ls_rjt_rx_handler(bnxef_driver_state_t *dsp, fc_packet_t *pktp)
{
	la_els_rjt_t *rjtp;
	bnxef_fcoe_task_t *srrtaskp;
	bnxef_fcoe_task_t *taskp;

	ASSERT(dsp != NULL);
	ASSERT(pktp != NULL);

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: MP SRR LS RJT Received",
		BNXEF_DRIVER_NAME, dsp->instance);

	bnxef_big_endian(pktp->pkt_resp, 4);

	/* LINTED: alignment */
	rjtp = (la_els_rjt_t *)pktp->pkt_resp;
	srrtaskp = (bnxef_fcoe_task_t *)pktp->pkt_fca_private;
	ASSERT(srrtaskp != NULL);
	taskp = dsp->taskarray[srrtaskp->secondary_x_id];
	ASSERT(taskp != NULL);

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: LS RJT reason 0x%x, Expln 0x%x, VU 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance,
		rjtp->reason, rjtp->reserved, rjtp->vu);

	/* If it is unable to supply the requested data */
	if (rjtp->reserved ==  0x2A)  {
		int ret;
		timeout_id_t tid;
		int tid_valid;
		/*
		 * Allow the original packet to go through
		 * recovery using ABTS
		 */
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: SRR Was rejected with unable to supply data",
			BNXEF_DRIVER_NAME, dsp->instance);
		mutex_enter(&taskp->task_mutex);
		if (taskp->task_flags & BNXEF_FCOE_TASK_TIMER_ARMED) {
			tid = taskp->timerhdl;
			taskp->task_flags &= ~BNXEF_FCOE_TASK_TIMER_ARMED;
			tid_valid = 1;
		}
		taskp->task_flags |= BNXEF_FCOE_RECOVERY_IN_PROGRESS;
		mutex_exit(&taskp->task_mutex);
		bnxef_fcoe_task_dec_ref_cnt(dsp, taskp);
		if (tid_valid)
			untimeout(tid);

		ret = bnxef_initiate_abts((bnxef_session_t *)taskp->sessionp,
					taskp);
		if (ret != BNXEF_SUCCESS) {
			BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to Initiate ABTS for for task id 0x%x",
				BNXEF_DRIVER_NAME, dsp->instance);
			return;
		}

	} else
		bnxef_fcoe_task_dec_ref_cnt(dsp, taskp);
}


void
bnxef_rec_timeout(void *arg)
{

	int ret;
	bnxef_tq_params_t *tqp = (bnxef_tq_params_t *)arg;
	bnxef_driver_state_t *dsp;
	bnxef_port_t *portp;

	ASSERT(tqp != NULL);
	dsp = tqp->dsp;
	ASSERT(dsp != NULL);
	portp = tqp->portp;
	ASSERT(portp != NULL);

	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	if (dsp->state & BNXEF_PORT_CVL_PROCESSING_INPROGRESS) {
		mutex_exit(&dsp->state_mutex);
		return;
	}

	if (dsp->physical_port_state == BNXEF_PHYPORT_OFFLINE) {
		mutex_exit(&dsp->state_mutex);
		return;
	}
	mutex_exit(&dsp->state_mutex);
	/*
	 * If the shutdown is in progress, donot do anything
	 * just return, the shutdown process will clear this I/O.
	 */
	mutex_enter(&portp->port_mutex);
	if (portp->flags & BNXEF_FC_PORT_SHUTDOWN_IN_PROGRESS) {
		mutex_exit(&portp->port_mutex);
		return;
	}
	mutex_exit(&portp->port_mutex);

	ret = ddi_taskq_dispatch(dsp->tq,
		bnxef_rec_timeout_handler, (void *)tqp, DDI_NOSLEEP);
	if (ret != DDI_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
		"%s%d: Unable to schedule the REC timeout handler\n",
		BNXEF_DRIVER_NAME, dsp->instance);
		return;
	}


}

void
bnxef_rec_timeout_handler(void *arg)
{
	bnxef_tq_params_t *tqp = (bnxef_tq_params_t *)arg;
	bnxef_driver_state_t *dsp;
	bnxef_session_t	*sessionp;
	bnxef_port_t	*portp;
	uint16_t	orig_task_id;
	int		 tid_valid = 0;
	timeout_id_t	tid;
	uint8_t		rec_failed_count;
	bnxef_fcoe_task_t *taskp, *otaskp;
	int ret;

	dsp = tqp->dsp;
	sessionp = tqp->sessp;
	portp = tqp->portp;
	taskp = tqp->taskp;


	/* When REC Timeout happens, we need to initiate the Abts */
	ASSERT(sessionp != NULL);
	ASSERT(dsp != NULL);
	ASSERT(taskp != NULL);
	ASSERT(portp != NULL);


	// debug_enter("Timeout");
	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	if (dsp->state & BNXEF_PORT_CVL_PROCESSING_INPROGRESS) {
		mutex_exit(&dsp->state_mutex);
		return;
	}
	if (dsp->physical_port_state == BNXEF_PHYPORT_OFFLINE) {
		mutex_exit(&dsp->state_mutex);
		return;
	}
	mutex_exit(&dsp->state_mutex);

	/*
	 * If the shutdown is in progress, donot do anything
	 * just return, the shutdown process will clear this I/O.
	 */
	mutex_enter(&portp->port_mutex);
	if (portp->flags & BNXEF_FC_PORT_SHUTDOWN_IN_PROGRESS) {
		mutex_exit(&portp->port_mutex);
		return;
	}
	mutex_exit(&portp->port_mutex);


	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_RECOVERY_IN_PROGRESS;
	taskp->task_flags &= ~BNXEF_FCOE_TASK_TIMER_ARMED;
	mutex_exit(&taskp->task_mutex);

	/*
	 * Save the secondary Id, from the REC task to idenity
	 * the original task
	 */
	orig_task_id = taskp->secondary_x_id;
	otaskp = dsp->taskarray[orig_task_id];
	bnxef_fcoe_task_dec_ref_cnt(dsp, otaskp);
	mutex_enter(&otaskp->task_mutex);
	if (otaskp->sec_flags & BNXEF_REC_SENT)
		otaskp->sec_flags &= ~BNXEF_REC_SENT;
	otaskp->rec_timeout_cnt += 1;
	rec_failed_count = otaskp->rec_timeout_cnt;
	/* If the timeout has happened for second time */
	if (otaskp->rec_timeout_cnt == 2) {
		/* Abort the original command and move on */
		if (otaskp->task_flags & BNXEF_FCOE_TASK_TIMER_ARMED)  {
			tid = otaskp->timerhdl;
			tid_valid = 1;
			otaskp->task_flags &= ~BNXEF_FCOE_TASK_TIMER_ARMED;
		}
		otaskp->task_flags |= BNXEF_FCOE_RECOVERY_IN_PROGRESS;
		mutex_exit(&otaskp->task_mutex);
		if (tid_valid)
			untimeout(tid);
		ret = bnxef_initiate_abts(sessionp, otaskp);
		if (ret != BNXEF_SUCCESS) {
			BNXEF_DEBUG(dsp, ERROR, -1,
				"%s%d: Unable to initiate ABTS for Orig task",
				BNXEF_DRIVER_NAME, dsp->instance);
		}

	} else mutex_exit(&otaskp->task_mutex);


	/*
	 * This will take through the complete state machine to
	 * complete the task in error.
	 */
	ret = bnxef_initiate_abts(sessionp, taskp);
	if (ret != BNXEF_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to initiate ABTS\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		return;
	}

	if (rec_failed_count < 2) {
		/* Now resend the REC as a different exchange */
		bnxef_send_rec((void *)&otaskp->tq_params);
	}

}


void
bnxef_srr_timeout(void *arg)
{

	int ret;
	bnxef_tq_params_t *tqp = (bnxef_tq_params_t *)arg;
	bnxef_driver_state_t *dsp;
	bnxef_port_t *portp;



	ASSERT(tqp != NULL);
	dsp = tqp->dsp;
	ASSERT(dsp != NULL);
	portp = tqp->portp;
	ASSERT(portp != NULL);

	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	if (dsp->state & BNXEF_PORT_CVL_PROCESSING_INPROGRESS) {
		mutex_exit(&dsp->state_mutex);
		return;
	}

	if (dsp->physical_port_state == BNXEF_PHYPORT_OFFLINE) {
		mutex_exit(&dsp->state_mutex);
		return;
	}
	mutex_exit(&dsp->state_mutex);
	/*
	 * If the shutdown is in progress, donot do anything
	 * just return, the shutdown process will clear this I/O.
	 */
	mutex_enter(&portp->port_mutex);
	if (portp->flags & BNXEF_FC_PORT_SHUTDOWN_IN_PROGRESS) {
		mutex_exit(&portp->port_mutex);
		return;
	}
	mutex_exit(&portp->port_mutex);

	ret = ddi_taskq_dispatch(dsp->tq,
		bnxef_srr_timeout_handler, (void *)tqp, DDI_NOSLEEP);
	if (ret != DDI_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
		"%s%d: Unable to schedule the REC timeout handler\n",
		BNXEF_DRIVER_NAME, dsp->instance);
		return;
	}


}

/*
 * SRR timeout happens. Initiate ABTS for the SRR. Then also initate the
 * ABTS for the original task to which this SRR was sent.
 */
void
bnxef_srr_timeout_handler(void *arg)
{
	bnxef_tq_params_t *tqp = (bnxef_tq_params_t *)arg;
	bnxef_driver_state_t *dsp;
	bnxef_session_t	*sessionp;
	bnxef_port_t	*portp;
	uint16_t	orig_task_id;
	bnxef_fcoe_task_t *taskp, *otaskp;
	int 		ret;
	uint32_t 	task_flags;
	timeout_id_t	tid;

	dsp = tqp->dsp;
	sessionp = tqp->sessp;
	portp = tqp->portp;
	taskp = tqp->taskp;


	/* When REC Timeout happens, we need to initiate the Abts */
	ASSERT(sessionp != NULL);
	ASSERT(dsp != NULL);
	ASSERT(taskp != NULL);
	portp = sessionp->lport;
	ASSERT(portp != NULL);

	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	if (dsp->state & BNXEF_PORT_CVL_PROCESSING_INPROGRESS) {
		mutex_exit(&dsp->state_mutex);
		return;
	}
	if (dsp->physical_port_state == BNXEF_PHYPORT_OFFLINE) {
		mutex_exit(&dsp->state_mutex);
		return;
	}
	mutex_exit(&dsp->state_mutex);

	/*
	 * If the shutdown is in progress, donot do anything
	 * just return, the shutdown process will clear this I/O.
	 */
	mutex_enter(&portp->port_mutex);
	if (portp->flags & BNXEF_FC_PORT_SHUTDOWN_IN_PROGRESS) {
		mutex_exit(&portp->port_mutex);
		return;
	}
	mutex_exit(&portp->port_mutex);


	mutex_enter(&taskp->task_mutex);
	taskp->task_flags |= BNXEF_FCOE_RECOVERY_IN_PROGRESS;
	taskp->task_flags &= ~BNXEF_FCOE_TASK_TIMER_ARMED;
	mutex_exit(&taskp->task_mutex);

	/*
	 * Save the secondary Id, from the SRR task to idenity
	 * the original task
	 */
	orig_task_id = taskp->secondary_x_id;
	/* Now initiate the ABTS for the Original Task */
	otaskp = dsp->taskarray[orig_task_id];

	bnxef_fcoe_task_dec_ref_cnt(dsp, otaskp);

	/* Send the ABTS for the SRR task */
	ret = bnxef_initiate_abts(sessionp, taskp);
	if (ret != BNXEF_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to initiate ABTS for SRR task",
			BNXEF_DRIVER_NAME, dsp->instance);
	}

	mutex_enter(&otaskp->task_mutex);
	task_flags = otaskp->task_flags;
	tid = otaskp->timerhdl;
	mutex_exit(&otaskp->task_mutex);

	if (task_flags & BNXEF_FCOE_TASK_TIMER_ARMED) {
		untimeout(tid);
		mutex_enter(&otaskp->task_mutex);
		otaskp->task_flags &=
			~BNXEF_FCOE_TASK_TIMER_ARMED;
		mutex_exit(&otaskp->task_mutex);
	}

	mutex_enter(&otaskp->task_mutex);
	otaskp->task_flags |= BNXEF_FCOE_RECOVERY_IN_PROGRESS;
	mutex_exit(&otaskp->task_mutex);

	ret = bnxef_initiate_abts(sessionp, otaskp);
	if (ret != BNXEF_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable initiate ABTS for Org Task",
			BNXEF_DRIVER_NAME, dsp->instance);
	}
}




void
bnxef_cleanup_sequence(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp, uint32_t reloff)
{
	uint8_t task_type = FCOE_TASK_TYPE_SEQUENCE_CLEANUP;
	uint32_t context_id;
	struct fcoe_task_ctx_entry *fwtskp;
	bnxef_driver_state_t *dsp;
	bnxef_port_t *portp;
	bnxef_fcoe_task_t *seqcleanuptaskp;
	struct fcoe_bd_ctx *bdp;
	uint32_t orig_rel_off = reloff;
	fc_packet_t	*opktp;
	struct fcoe_task_ctx_entry *ofwtskp;
	int i, ret;
	uint64_t phyaddr;
	bnxef_tq_params_t *tqp;


	ASSERT(sessionp != NULL);
	portp = (bnxef_port_t *)sessionp->lport;
	context_id = sessionp->context_id;
	ASSERT(portp != NULL);
	dsp = (bnxef_driver_state_t *)portp->statep;
	ASSERT(dsp != NULL);

	seqcleanuptaskp = bnxef_task_alloc(dsp, KM_SLEEP);
	ASSERT(seqcleanuptaskp != NULL);
	fwtskp = seqcleanuptaskp->fwtaskp;
	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Sequence Cleanup Task id 0x%x, reoff 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance,
		seqcleanuptaskp->x_id, reloff);
	mutex_enter(&seqcleanuptaskp->task_mutex);
	tqp = &seqcleanuptaskp->tq_params;
	/*
	 * The original task pointer -
	 * this will be used to send SRR
	 */
	tqp->taskp = taskp;

	fwtskp->txwr_rxrd.const_ctx.tx_flags =
		FCOE_TASK_TX_STATE_SEQUENCE_CLEANUP <<
		FCOE_TCE_TX_WR_RX_RD_CONST_TX_STATE_SHIFT;
	fwtskp->txwr_rxrd.const_ctx.init_flags =
		(task_type <<
			FCOE_TCE_TX_WR_RX_RD_CONST_TASK_TYPE_SHIFT);
	fwtskp->rxwr_txrd.const_ctx.init_flags =
		CPU_TO_LE32(context_id <<
				FCOE_TCE_RX_WR_TX_RD_CONST_CID_SHIFT);
	fwtskp->txwr_rxrd.union_ctx.cleanup.ctx.cleaned_task_id =
		CPU_TO_LE16(taskp->x_id);
	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Cleaned Task Id 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, taskp->x_id);
	fwtskp->txwr_rxrd.union_ctx.cleanup.ctx.rolled_tx_seq_cnt =
			CPU_TO_LE16(0);
	fwtskp->txwr_rxrd.union_ctx.cleanup.ctx.rolled_tx_data_offset =
			CPU_TO_LE32(reloff);

	opktp = taskp->pktp;
	ASSERT(opktp != NULL);
	ofwtskp = taskp->fwtaskp;
	ASSERT(reloff < opktp->pkt_datalen);
	/* LINTED: alignment */
	bdp = (struct fcoe_bd_ctx *)taskp->bd_addr;
	for (i = 0; i < taskp->actual_bd_entries; i++) {
		if (reloff < LE16_TO_CPU(bdp->buf_len))
			break;
		reloff -= LE16_TO_CPU(bdp->buf_len);
		bdp += 1;
	}
	ASSERT(i < taskp->actual_bd_entries);
	phyaddr = taskp->bd_cookie.dmac_laddress +
			i * sizeof (struct fcoe_bd_ctx);
	/* Check whether it is a Write SCSI command */
	if (opktp->pkt_tran_type == FC_PKT_FCP_WRITE) {
		if (ofwtskp->txwr_rxrd.const_ctx.init_flags &
			(1 << FCOE_TCE_TX_WR_RX_RD_CONST_CACHED_SGE_SHIFT)) {
			struct fcoe_bd_ctx *fcoe_bd_tbl =
				/* LINTED: alignment */
				(struct fcoe_bd_ctx *)taskp->bd_addr;
			uint64_t phyaddr =
			(uint64_t)(
			(uint64_t)LE32_TO_CPU(fcoe_bd_tbl->buf_addr_hi) << 32) |
			(uint64_t)LE32_TO_CPU((fcoe_bd_tbl->buf_addr_lo));
			BNXEF_DEBUG(dsp, ERROR, -1,
				"%s%d: Task bd Addr 0x%p, buf_add_hi 0x%x,\
				buf_addr_lo 0x%x, phyaddr 0x%p, buf len 0x%x",
				BNXEF_DRIVER_NAME, dsp->instance,
				taskp->bd_addr,
				fcoe_bd_tbl->buf_addr_hi,
				fcoe_bd_tbl->buf_addr_lo,
				phyaddr, fcoe_bd_tbl->buf_len);
			fwtskp->txwr_only.sgl_ctx.cached_sge.cur_buf_addr.lo =
			CPU_TO_LE32((uint32_t)((uint64_t)(phyaddr + reloff)));
			fwtskp->txwr_only.sgl_ctx.cached_sge.cur_buf_addr.hi =
			CPU_TO_LE32((uint32_t)
				((uint64_t)(phyaddr + reloff) >> 32));
			fwtskp->txwr_only.sgl_ctx.cached_sge.cur_buf_rem =
				CPU_TO_LE16(
				(LE16_TO_CPU(fcoe_bd_tbl->buf_len)
					- reloff));
			BNXEF_DEBUG(dsp, ERROR, -1,
				"%s%d:  buf_add_hi 0x%x, buf_addr_lo 0x%x,\
					buf_len 0x%x",
				BNXEF_DRIVER_NAME, dsp->instance,
				fcoe_bd_tbl->buf_addr_hi,
				fcoe_bd_tbl->buf_addr_lo, fcoe_bd_tbl->buf_len);
		} else {
			/*
			 * For write if we use only mul_sgl and do not
			 * use cached SGEs, then do the following adjustments.
			 */
			fwtskp->txwr_only.sgl_ctx.sgl.mul_sgl.cur_sge_addr.lo =
					CPU_TO_LE32((uint32_t)phyaddr);
			fwtskp->txwr_only.sgl_ctx.sgl.mul_sgl.cur_sge_addr.hi =
			CPU_TO_LE32((uint32_t)((uint64_t)phyaddr  >> 32));
			fwtskp->txwr_only.sgl_ctx.sgl.mul_sgl.sgl_size =
				taskp->actual_bd_entries;
			fwtskp->txwr_only.sgl_ctx.sgl.mul_sgl.cur_sge_off =
				CPU_TO_LE16(reloff);
			fwtskp->txwr_only.sgl_ctx.sgl.mul_sgl.cur_sge_idx = i;
		}
	} else {
		if (ofwtskp->txwr_rxrd.const_ctx.init_flags &
			(1 << FCOE_TCE_TX_WR_RX_RD_CONST_CACHED_SGE_SHIFT)) {
			if (i == 0) {
				struct fcoe_bd_ctx *fcoe_bd_tbl =
					/* LINTED: alignment */
					(struct fcoe_bd_ctx *)taskp->bd_addr;
				uint64_t phyaddr =
					(uint64_t)
					LE32_TO_CPU((uint64_t)
					fcoe_bd_tbl->buf_addr_hi << 32) |
				(uint64_t)LE32_TO_CPU(fcoe_bd_tbl->buf_addr_lo);
				BNXEF_DEBUG(dsp, ERROR, -1,
				"%s%d: Task bd Addr 0x%p,\
					buf_add_hi 0x%x, buf_addr_lo 0x%x,\
					phyaddr 0x%p, buf len 0x%x",
					BNXEF_DRIVER_NAME, dsp->instance,
					taskp->bd_addr,
					fcoe_bd_tbl->buf_addr_hi,
					fcoe_bd_tbl->buf_addr_lo,
					phyaddr, fcoe_bd_tbl->buf_len);
				fwtskp->rxwr_only.union_ctx.
					read_info.sgl_ctx.
					cached_sge.cur_buf_addr.lo =
					CPU_TO_LE32((uint32_t)
					((uint64_t)(phyaddr + reloff)));
				fwtskp->rxwr_only.union_ctx.
					read_info.sgl_ctx.
					cached_sge.cur_buf_addr.hi =
					CPU_TO_LE32((uint32_t)
					((uint64_t)(phyaddr + reloff) >> 32));
				fwtskp->rxwr_only.union_ctx.
					read_info.sgl_ctx.
					cached_sge.cur_buf_rem =
					CPU_TO_LE16(
					(LE16_TO_CPU(fcoe_bd_tbl->buf_len) -
						reloff));


				fcoe_bd_tbl += 1;
				/*
				 * Here we Update already existing entries into
				 * second buffer, therefore fcoe_bd_tbl entries
				 * are written without swapping for endianness.
				 * please be aware of it when changing.
				 */
				fwtskp->rxwr_only.union_ctx.read_info.
					sgl_ctx.cached_sge.second_buf_addr.lo =
					fcoe_bd_tbl->buf_addr_lo;
				fwtskp->rxwr_only.union_ctx.read_info.
					sgl_ctx.cached_sge.second_buf_addr.hi =
					fcoe_bd_tbl->buf_addr_hi;
				fwtskp->rxwr_only.union_ctx.read_info.
					sgl_ctx.cached_sge.second_buf_rem =
					fcoe_bd_tbl->buf_len;
				BNXEF_DEBUG(dsp, ERROR, -1,
					"%s%d:  buf_add_hi 0x%x, \
					buf_addr_lo 0x%x, buf_len 0x%x",
					BNXEF_DRIVER_NAME, dsp->instance,
					fcoe_bd_tbl->buf_addr_hi,
					fcoe_bd_tbl->buf_addr_lo,
					fcoe_bd_tbl->buf_len);
			} else if (i == 1) {
				struct fcoe_bd_ctx *fcoe_bd_tbl =
					/* LINTED: alignment */
					(struct fcoe_bd_ctx *)taskp->bd_addr;

				/*
				 * This false in the second
				 * entry in the bd table
				 */
				fcoe_bd_tbl += 1;

				uint64_t phyaddr =
				(uint64_t)
				LE32_TO_CPU(
				((uint64_t)fcoe_bd_tbl->buf_addr_hi << 32)) |
			LE32_TO_CPU((uint64_t)(fcoe_bd_tbl->buf_addr_lo));
				BNXEF_DEBUG(dsp, ERROR, -1,
				"%s%d: 2. Task bd Addr 0x%p, buf_add_hi 0x%x,\
				buf_addr_lo 0x%x, phyaddr 0x%p, buf len 0x%x",
					BNXEF_DRIVER_NAME, dsp->instance,
					taskp->bd_addr,
					fcoe_bd_tbl->buf_addr_hi,
					fcoe_bd_tbl->buf_addr_lo,
					phyaddr, fcoe_bd_tbl->buf_len);
				fwtskp->rxwr_only.union_ctx.read_info.
				sgl_ctx.cached_sge.cur_buf_addr.lo =
				CPU_TO_LE32(
				(uint32_t)((uint64_t)(phyaddr + reloff)));
				fwtskp->rxwr_only.union_ctx.read_info.
					sgl_ctx.cached_sge.cur_buf_addr.hi =
				CPU_TO_LE32(
				(uint32_t)((uint64_t)(phyaddr + reloff) >> 32));
				fwtskp->rxwr_only.union_ctx.read_info.
					sgl_ctx.cached_sge.cur_buf_rem =
					CPU_TO_LE16(
					(LE16_TO_CPU(fcoe_bd_tbl->buf_len) -
					reloff));
				BNXEF_DEBUG(dsp, ERROR, -1,
					"%s%d:  2. buf_add_hi 0x%x,\
						buf_addr_lo 0x%x, buf_len 0x%x",
					BNXEF_DRIVER_NAME, dsp->instance,
					fcoe_bd_tbl->buf_addr_hi,
					fcoe_bd_tbl->buf_addr_lo,
					fcoe_bd_tbl->buf_len);


				fwtskp->rxwr_only.union_ctx.read_info.
				sgl_ctx.cached_sge.second_buf_addr.lo = 0;
				fwtskp->rxwr_only.union_ctx.read_info.
				sgl_ctx.cached_sge.second_buf_addr.hi = 0;
				fwtskp->rxwr_only.union_ctx.read_info.
				sgl_ctx.cached_sge.second_buf_rem =  0;
			} else {
				ASSERT(0);
			}
		} else {
			BNXEF_DEBUG(dsp, ERROR, -1,
				"%s%d: Mul SGE case phyaddr 0x%p,\
				bd entries 0x%x, reloff 0x%x, idx 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, phyaddr,
			taskp->actual_bd_entries, reloff, i);
			/* Multiple SGE are used for READ */
			fwtskp->rxwr_only.union_ctx.read_info.
					sgl_ctx.sgl.mul_sgl.cur_sge_addr.lo =
					CPU_TO_LE32((uint32_t)phyaddr);
			fwtskp->rxwr_only.union_ctx.read_info.
				sgl_ctx.sgl.mul_sgl.cur_sge_addr.hi =
			CPU_TO_LE32((uint32_t)((uint64_t)phyaddr  >> 32));
			fwtskp->rxwr_only.union_ctx.read_info.
					sgl_ctx.sgl.mul_sgl.sgl_size =
						taskp->actual_bd_entries;
			fwtskp->rxwr_only.union_ctx.read_info.sgl_ctx.
				sgl.mul_sgl.cur_sge_off =
				CPU_TO_LE16(reloff);
			fwtskp->rxwr_only.union_ctx.read_info.sgl_ctx.
					sgl.mul_sgl.cur_sge_idx = i;
		}
		memset(&fwtskp->rxwr_only.rx_seq_ctx, 0,
			sizeof (struct fcoe_rx_seq_ctx));
		fwtskp->rxwr_only.rx_seq_ctx.low_exp_ro =
			CPU_TO_LE32(orig_rel_off);
		fwtskp->rxwr_only.rx_seq_ctx.high_exp_ro =
			CPU_TO_LE32(orig_rel_off);
	}
	seqcleanuptaskp->task_flags |= BNXEF_FCOE_TASK_SEQ_CLEANUP;
	mutex_exit(&seqcleanuptaskp->task_mutex);

	ret = ddi_dma_sync(
		dsp->task_dma_handle[seqcleanuptaskp->dma_handle_idx],
		(seqcleanuptaskp->x_id %32) *
			sizeof (struct fcoe_task_ctx_entry),
			sizeof (struct fcoe_task_ctx_entry),
			DDI_DMA_SYNC_FORDEV);
	ASSERT(ret == DDI_SUCCESS);


	/* Add to the SQ entryt for firmware to work on */
	bnxef_add_2_sq(sessionp, seqcleanuptaskp->x_id);
	/* Now ring the door bell */
	bnxef_ring_doorbell(sessionp);

}
