
#ifndef __BNXEF_TRACE_EVENTS_H__
#define	__BNXEF_TRACE_EVENTS_H__

#include <sys/types.h>
#include <sys/conf.h>
#include <sys/stream.h>
#include <sys/stropts.h>
#include <sys/errno.h>
#include <sys/cred.h>
#include <sys/poll.h>
#include <sys/modctl.h>
#include <sys/stat.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/ddifm.h>
#include <sys/fm/protocol.h>
#include <sys/fm/util.h>
#include <sys/fm/io/ddi.h>
#include <sys/pattr.h>
#include <sys/sysmacros.h>
#include <sys/strsun.h>
#include <sys/ksynch.h>
#include <sys/kstat.h>
#include <sys/strsubr.h>
#include <sys/ksynch.h>
#include <note.h>


typedef struct bnxef_trace_events {
	uint16_t	event_id;
	int16_t		task_id;
	int16_t		secondary_task_id;
	uint16_t	reserved0;
	uint64_t	lbolt;
	uint64_t	reserved1;
	uint64_t	reserved2;
	uint64_t	reserved3;
	uint64_t	reserved4;
} bnxef_trace_events_t;


/*
 * List of events tracked and the semantics
 */
#define	BNXEF_SCSI_TASK_INIT			0x0001
#define	BNXEF_SCSI_TASK_COMPLETE		0x0002
#define	BNXEF_SCSI_TASK_TIMEOUT			0x0003
#define	BNXEF_SCSI_TASK_ERRORS			0x0004
#define	BNXEF_SCSI_TASK_ABTS_INITIATED		0x0005
#define	BNXEF_SCSI_TASK_BAACC_RECVD		0x0006
#define	BNXEF_SCSI_TASK_ABTS_TIMEOUT		0x0007
#define	BNXEF_SCSI_TASK_RRQ_INITIATED		0x0008
#define	BNXEF_SCSI_TASK_RRQ_COMPLETED		0x0009
#define	BNXEF_SCSI_TASK_RRQ_TIMEOUT		0x000A
#define	BNXEF_SCSI_TASK_CLEANUP_INIT		0x000B
#define	BNXEF_SCSI_TASK_CLEANUP_COMP		0x000C
#define	BNXEF_SCSI_TASK_SENSE_COMPLETED		0x000D
#define	BNXEF_SCSI_TASK_BARJT_RECVD		0x000E
#define	BNXEF_SCSI_TASK_RRQ_LS_RJT		0x000F
#define	BNXEF_SCSI_TASK_COMP_WHILE_RECOV	0x0010
#define	BNXEF_SCSI_TASK_ABTS_TASK_FREED		0x0011


/* List of Session Related events */
#define	BNXEF_SESSION_ENABLE_REQ	0x0030
#define	BNXEF_SESSION_ENABLE_COMP	0x0031
#define	BNXEF_SESSION_DISABLE_REQ	0x0032
#define	BNXEF_SESSION_DISABLE_COMP	0x0033
#define	BNXEF_SESSION_OFFLOAD_REQ	0x0034
#define	BNXEF_SESSION_OFFLOAD_COMP	0x0035
#define	BNXEF_SESSION_DESTROY_REQ	0x0036
#define	BNXEF_SESSION_DESTROY_COMP	0x0037

#ifdef DEBUG
#define	BNXEF_TRACE_EVENT(a, b, c, d, e, f, g, h, m) {\
	if (a->bnxef_debug & BNXEF_TRACE_EVENTS) {\
		mutex_enter(&a->global_event_log_mutex); \
		a->global_event_logp[a->gev_log_idx].event_id = b; \
		a->global_event_logp[a->gev_log_idx].task_id = c; \
		a->global_event_logp[a->gev_log_idx].secondary_task_id = d; \
		a->global_event_logp[a->gev_log_idx].reserved0 = e; \
		a->global_event_logp[a->gev_log_idx].lbolt = ddi_get_lbolt(); \
		a->global_event_logp[a->gev_log_idx].reserved1 = (uint64_t)f; \
		a->global_event_logp[a->gev_log_idx].reserved2 = (uint64_t)g; \
		a->global_event_logp[a->gev_log_idx].reserved3 = (uint64_t)h; \
		a->global_event_logp[a->gev_log_idx].reserved4 = (uint64_t)m; \
		a->gev_log_idx += 1; \
		a->gev_log_idx %= a->bnxef_max_trace_events; \
		mutex_exit(&a->global_event_log_mutex); \
	}\
}
#else
#define	BNXEF_TRACE_EVENT(a, b, c, d, e, f, g, h, m)
#endif


#endif
