/*
 * The information contained in this file is confidential and proprietary to
 * Broadcom Corporation.  No part of this file may be reproduced or
 * distributed, in any form or by any means for any purpose, without the
 * express written permission of Broadcom Corporation.
 *
 * (c) COPYRIGHT 2010-2012 Broadcom Corporation, ALL RIGHTS RESERVED.
 */

#pragma ident "(C) 2010-2012 Broadcom Corporation"
#pragma ident "$File$$Revision$$Change$$DateTime$"
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/stat.h>
#include <sys/pci.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>
#include <sys/cmn_err.h>
#include <sys/file.h>
#include <sys/cred.h>
#include <sys/byteorder.h>
#include <sys/atomic.h>
#include <sys/scsi/scsi.h>
#include <sys/modhash.h>
#include <sys/sunldi.h>
#include <sys/kstat.h>
#include <sys/kmem.h>
#include <sys/ethernet.h>
#include <sys/stream.h>
#include <sys/sunldi.h>
/*
 * leadville header files
 */
#include <sys/fibre-channel/fc_types.h>
#include <sys/fibre-channel/impl/fc_fcaif.h>
#include <bnxe_binding.h>
#include <bnxef_tq_param.h>
#include <bnxef_fcf_info.h>
#include <bnxef_hsi.h>
#include <bnxef_fcoe_task.h>
#include <bnxef_fcoe_session.h>
#include <bnxef_state.h>
#include <bnxef_fip.h>
#include <bnxef_fcoe.h>
#include <bnxef_debug.h>
#include <fw_defs.h>

// #define TEST_LS_RJT

/* Externals */
extern void  * bnxef_state;
extern int bnxef_unsol_buf_list_init(bnxef_port_t *portp);
extern int bnxef_session_list_init(bnxef_driver_state_t *dsp,
	bnxef_port_t *portp, uint8_t hashbucket);
extern int bnxef_session_list_fini(bnxef_driver_state_t *dsp,
	bnxef_port_t *portp, uint8_t hashbucket);
extern int bnxef_unsol_buf_list_fini(bnxef_port_t *portp);
extern int bnxef_fip_sm_send_flogi_fdisc(bnxef_driver_state_t *dsp,
	uint8_t opcode, uint16_t ox_id, uint8_t *port_name, uint8_t *node_name);
extern void bnxef_select_fcf(bnxef_driver_state_t *dsp);
extern int bnxef_fip_sm_send_logo(bnxef_driver_state_t *dsp,
	bnxef_lport_t *portp);
extern bnxef_fcoe_task_t *bnxef_task_alloc(bnxef_driver_state_t *dsp,
	int sleepflag);
extern int bnxef_unsol_buf_list_remove(bnxef_port_t *portp,
	bnxef_unsol_buf_t *unsolbufp);
extern int bnxef_unsol_buf_list_add(bnxef_port_t *portp,
	bnxef_unsol_buf_t *unsolbup);
extern int
bnxef_initiate_abts(bnxef_session_t *sessionp, bnxef_fcoe_task_t *taskp);
extern void bnxef_complete_pkt(fc_packet_t *pktp, uint8_t state,
		uint8_t action, uint32_t reason, uint32_t expln);
extern int bnxef_scsi_transport_handler(bnxef_session_t *sessionp,
		bnxef_fcoe_task_t *taskp);
extern int bnxef_fcct_transport_handler(bnxef_session_t *sessionp,
			bnxef_fcoe_task_t *taskp);
extern bnxef_session_t *bnxef_session_find_remote_port(bnxef_port_t *portp,
		int16_t vlan_id, uint32_t d_id, uint8_t *macaddr);
extern void bnxef_fip_disc_adv_enforce(void *arg);
extern void bnxef_timeout_fip_ka(void *arg);
extern void bnxef_cleanup_port(bnxef_port_t *portp);
extern int bnxef_get_num_free_list_items(bnxef_driver_state_t *dsp);



void
bnxef_fc_port_online_startup(void *arg)
{
	bnxef_port_t *portp = (bnxef_port_t *)arg;

	/* Allow the startup to complete before informing state ONLINE */
	drv_usecwait(1000);
	portp->pbind.port_statec_cb((opaque_t)portp->pbind.port_handle,
		FC_STATE_ONLINE);
}
/*
 * This will indicate the higher level layer that the FC port is online.
 */
void
bnxef_fc_port_online(void *arg)
{
	bnxef_port_t *portp = (bnxef_port_t *)arg;

	drv_usecwait(1000);
	portp->pbind.port_statec_cb((opaque_t)portp->pbind.port_handle,
		FC_STATE_10GBIT_SPEED|FC_STATE_ONLINE);
}

void
bnxef_fc_port_offline(void *arg)
{
	bnxef_port_t *portp = (bnxef_port_t *)arg;
	uint32_t flags;

	mutex_enter(&portp->port_mutex);
	flags = portp->flags;
	mutex_exit(&portp->port_mutex);
	if ((flags & BNXEF_FC_PORT_SOL_BOUND) == BNXEF_FC_PORT_SOL_BOUND) {
		portp->pbind.port_statec_cb((opaque_t)portp->pbind.port_handle,
			FC_STATE_OFFLINE);
	}
}

void
bnxef_fc_port_reset(void *arg)
{
	bnxef_port_t *portp = (bnxef_port_t *)arg;

	portp->pbind.port_statec_cb((opaque_t)portp->pbind.port_handle,
		FC_STATE_LIP_LBIT_SET);
}




static void
bnxef_populate_hba_fru_details(bnxef_driver_state_t *dsp,
		fc_fca_port_info_t *pip)
{
	dev_info_t *pdip;
	fca_port_attrs_t	*pattrp;
	int instance;

	pdip = ddi_get_parent(dsp->dip);

	ASSERT(dsp != NULL);
	ASSERT(pip != NULL);

	pattrp = &pip->pi_attrs;
	(void) snprintf(pattrp->manufacturer, FCHBA_MANUFACTURER_LEN,
		"Broadcom Inc.");
	(void) snprintf(pattrp->driver_name, FCHBA_DRIVER_NAME_LEN,
		"%s", BNXEF_DRIVER_NAME);
	(void) snprintf(pattrp->driver_version, FCHBA_DRIVER_VERSION_LEN,
		"%s", BNXEF_FCOE_VERSION);

	if (!dsp->callbacks.prvCtl(pdip, PRV_CTL_BOARD_SERNUM,
		(void *)pattrp->serial_number,
		FCHBA_SERIAL_NUMBER_LEN)) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to get the board serial number",
			BNXEF_DRIVER_NAME, dsp->instance);
		(void) strcpy(pattrp->serial_number, "N/A");
	}

	(void) strcpy(pattrp->hardware_version, "Converged NIC Rev 2");

	if (!dsp->callbacks.prvCtl(pdip, PRV_CTL_BOARD_TYPE,
			(void *)pattrp->model,
			FCHBA_MODEL_LEN)) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to get the board type",
			BNXEF_DRIVER_NAME, dsp->instance);
		(void) strcpy(pattrp->model, "10Gb/sec Converged NIC");
	}

	(void) strcpy(pattrp->model_description, "None");

	(void) sprintf(pattrp->firmware_version,
		    "%d.%d.%d",
		    BCM_5710_FW_MAJOR_VERSION,
		    BCM_5710_FW_MINOR_VERSION,
		    BCM_5710_FW_REVISION_VERSION);

	if (!dsp->callbacks.prvCtl(pdip, PRV_CTL_BOOTCODE_VERSION,
			(void *)pattrp->option_rom_version,
			FCHBA_OPTION_ROM_VERSION_LEN)) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to get the option rom version",
			BNXEF_DRIVER_NAME, dsp->instance);
		(void) strcpy(pattrp->option_rom_version, "Not Known");
	}

	pattrp->vendor_specific_id = 0x14e4;
	pattrp->max_frame_size = 2136;
	pattrp->supported_cos = 0x10000000;
	pattrp->supported_speed = FC_HBA_PORTSPEED_10GBIT;
	instance = ddi_get_instance(dsp->dip);
	pattrp->hba_fru_details.high = (short)((instance & 0xFFFF0000) >> 8);
	pattrp->hba_fru_details.low = (short)(instance & 0x0000FFFF);

#if defined(__S11) || defined(__S12)
	(void) sprintf((char *)pattrp->sym_port_name, "%s %s %s %s",
			"Broadcom", pattrp->model, pattrp->driver_name,
			pattrp->driver_version);
#endif
}

void
bnxef_offline_bind_port_handler(bnxef_driver_state_t *dsp,
	bnxef_port_t *portp, fc_fca_port_info_t *pip,
	fc_fca_bind_info_t *bip)
{
	uint8_t portname[8];
	uint8_t nodename[8];

	ASSERT(dsp != NULL);
	ASSERT(portp != NULL);
	ASSERT(pip != NULL);
	ASSERT(bip != NULL);

	mutex_enter(&portp->port_mutex);
#if defined(__S11) || defined(__S12)
	if (bip->port_npiv) {
		portp->flags |= (BNXEF_FC_PORT_VN_PORT);
		bcopy((uint8_t *)&bip->port_nwwn, (uint8_t *)nodename,
			sizeof (la_wwn_t));
		bcopy((uint8_t *)&bip->port_pwwn, (uint8_t *)portname,
			sizeof (la_wwn_t));
		bcopy(portname, &portp->plogi_info.nport_ww_name.raw_wwn[0],
			sizeof (la_wwn_t));
		bcopy(nodename, &portp->plogi_info.node_ww_name.raw_wwn[0],
			sizeof (la_wwn_t));
	} else {
#endif
		portp->flags |= (BNXEF_FC_PORT_N_PORT);
		bcopy(&dsp->logi_params.nport_ww_name.raw_wwn[0],
			portname, sizeof (la_wwn_t));
		bcopy(&dsp->logi_params.node_ww_name.raw_wwn[0],
				nodename, sizeof (la_wwn_t));
#if defined(__S11) || defined(__S12)
	}
#endif

	/* Now copy this Plogi information into portinfo structure */
	bcopy(&portp->plogi_info, &pip->pi_login_params,
		sizeof (la_els_logi_t));

	pip->pi_s_id.port_id = 0;
	pip->pi_port_state |= FC_STATE_10GBIT_SPEED | FC_STATE_OFFLINE;
	portp->flags |= BNXEF_FC_PORT_SOL_BOUND;
	mutex_exit(&portp->port_mutex);

	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	dsp->num_fc_ports++;
	mutex_exit(&dsp->state_mutex);

	bnxef_populate_hba_fru_details(dsp, pip);

}

/*
 * The following routines are the transport routine registered with the
 * leadville stack.
 */

/*
 * This binds the port and provides the port related information to
 * the leadville stack. The handle retuned is nothing but the port handle
 * allocated. This also gets the ID using FIP infrastructure and gets the port
 * ready for action. After binding when the FLOGI comes, we just fake success
 * and return the appropriate ID.
 */
opaque_t
bnxef_bind_port(dev_info_t *dip, fc_fca_port_info_t *pip,
	fc_fca_bind_info_t *bip)
{
	bnxef_driver_state_t *dsp;
	int		instance;
	bnxef_port_t	*portp;
	int		 i, j, ret;
	timeout_id_t	tid, tid2;
	uint8_t	portname[8];
	uint8_t	  nodename[8];
	uint8_t		retry = 0;
	uint8_t		cmd;
	uint8_t		fdisc_sent = 0;
	uint32_t	 fip_state;

	ASSERT(pip != NULL);
	ASSERT(bip != NULL);

	instance = ddi_get_instance(dip);
	dsp = ddi_get_soft_state(bnxef_state, instance);
	if (dsp == NULL) {
		cmn_err(CE_WARN, "%s: Unable to get driver state-%d instance\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		pip->pi_error = FC_BADPORT;
		return (NULL);
	}
	BNXEF_DEBUG(dsp, GENERAL, -1,
		"%s%d: Entering %s",
		BNXEF_DRIVER_NAME, dsp->instance, __func__);

	/* Validate whether the request to bind is within limits */
	if (bip->port_num > dsp->num_fc_ports) {
		BNXEF_DEBUG(dsp, ERROR, -1, "%s%d: Request out of bounds\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		pip->pi_error = FC_OUTOFBOUNDS;
		return (NULL);
	}

	/* Allocate a port structure and try to initialize fields */
	portp = kmem_cache_alloc(dsp->port_cache, KM_SLEEP);
	if (portp == NULL) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to allocate port structure\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		pip->pi_error = FC_BADPORT;
		return (NULL);
	}
	portp->flags = 0;
	portp->fip_state = 0;

	/* Initialize the unsolicited buffer list */
	ret = bnxef_unsol_buf_list_init(portp);
	if (ret != BNXEF_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to init unsol buffer list \n",
			BNXEF_DRIVER_NAME, dsp->instance);
		kmem_cache_free(dsp->port_cache, portp);
		pip->pi_error = FC_BADPORT;
		return (NULL);
	}

	/* Initialize the session hash buckets associated with this port */
	for (i = 0; i < BNXEF_MAX_HASH_BUCKETS; i++) {
		ret = bnxef_session_list_init(dsp, portp,  i);
		if (ret != BNXEF_SUCCESS) {
			BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Unable to init session hash bucket %d\n",
			BNXEF_DRIVER_NAME, dsp->instance, i);
			for (j = 0; j < i; j++)
				(void) bnxef_session_list_fini(dsp, portp, j);
			bnxef_unsol_buf_list_fini(portp);
			kmem_cache_free(dsp->port_cache, portp);
			pip->pi_error = FC_BADPORT;
			return (NULL);
		}
	}

	mutex_enter(&portp->port_mutex);
	portp->statep = dsp;

	/* Copy the bind information passed in into the port structure */
	bcopy((uint8_t *)bip, (uint8_t *)&portp->pbind,
		sizeof (fc_fca_bind_info_t));

	/* Set the Login Params supported and send it to Leadville stack */
	bcopy((uint8_t *)&dsp->logi_params, (uint8_t *)&portp->plogi_info,
			sizeof (la_els_logi_t));



	/* RNID parameters to be filled in, TBD- Not done here yet */
	pip->pi_rnid_params.status = FC_FAILURE;

	portp->flags |= (BNXEF_FC_PORT_ID_NOT_ACQUIRED);

	mutex_exit(&portp->port_mutex);

	ret = bnxef_port_list_add(dsp, portp);
	ASSERT(ret == BNXEF_SUCCESS);

	portp->fip_task = bnxef_task_alloc(dsp, KM_SLEEP);
	ASSERT(portp->fip_task != NULL);
	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	if ((dsp->physical_port_state & BNXEF_PHYPORT_ONLINE) == 0) {
		mutex_exit(&dsp->state_mutex);
		bnxef_offline_bind_port_handler(dsp, portp, pip, bip);
		return ((opaque_t)portp);
	}
	if (dsp->fcf_ident == NULL) {
		ret = cv_timedwait(&dsp->fcf_cv, &dsp->state_mutex,
				ddi_get_lbolt() + BNXEF_SEC2TICK(15));
		if (ret < 0) {
			mutex_exit(&dsp->state_mutex);
			bnxef_offline_bind_port_handler(dsp, portp, pip, bip);
			return ((opaque_t)portp);
		}
	}
	ASSERT(dsp->fcf_ident != NULL);
	mutex_exit(&dsp->state_mutex);

	mutex_enter(&portp->port_mutex);
	portp->vlan_id = dsp->fcf_ident->vlan_id;
	mutex_exit(&portp->port_mutex);

#ifdef TEST_LS_ACC_DROP
			vlan_id = dsp->fcf_ident->vlan_id;
#endif
	for (retry = 0; retry < 120; retry++) {
		fdisc_sent = 0;
		MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
		if ((dsp->physical_port_state & BNXEF_PHYPORT_ONLINE) == 0) {
			mutex_exit(&dsp->state_mutex);
			bnxef_offline_bind_port_handler(dsp, portp, pip, bip);
			return ((opaque_t)portp);
		}
		if (dsp->state & BNXEF_IMPLICIT_FABRIC_LOGO_IN_PROGRESS) {
			mutex_exit(&dsp->state_mutex);
			bnxef_offline_bind_port_handler(dsp, portp, pip, bip);
			return ((opaque_t)portp);
		}
		mutex_exit(&dsp->state_mutex);

#if defined(__S11) || defined(__S12)
		if (bip->port_npiv) {
			bcopy((uint8_t *)&bip->port_nwwn, (uint8_t *)nodename,
				sizeof (la_wwn_t));
			bcopy((uint8_t *)&bip->port_pwwn, (uint8_t *)portname,
				sizeof (la_wwn_t));
			bcopy(portname,
				&portp->plogi_info.nport_ww_name.raw_wwn[0],
				sizeof (la_wwn_t));
			bcopy(nodename,
				&portp->plogi_info.node_ww_name.raw_wwn[0],
				sizeof (la_wwn_t));
			/* THis is a npiv port */
			ret = bnxef_fip_sm_send_flogi_fdisc(dsp, LA_ELS_FDISC,
				portp->fip_task->x_id, portname, nodename);
			fdisc_sent = 1;
		} else {
#endif
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Sending FLOGI with Retry Count 0x%x",
				BNXEF_DRIVER_NAME, dsp->instance, retry);
#ifdef TEST_LS_ACC_DROP
			dsp->fcf_ident->vlan_id = 100;
			if (retry > 125)
				dsp->fcf_ident->vlan_id = vlan_id;
#endif
#ifdef TEST_LS_RJT
			cmd = LA_ELS_FDISC;
			if (retry > 125)
				cmd = LA_ELS_FLOGI;
#else
			cmd = LA_ELS_FLOGI;
#endif
			bcopy(&dsp->logi_params.nport_ww_name.raw_wwn[0],
				portname, sizeof (la_wwn_t));
			bcopy(&dsp->logi_params.node_ww_name.raw_wwn[0],
					nodename, sizeof (la_wwn_t));
			ret = bnxef_fip_sm_send_flogi_fdisc(dsp, cmd,
				portp->fip_task->x_id, portname, nodename);
#if defined(__S11) || defined(__S12)
		}
#endif
		if (ret != BNXEF_SUCCESS) {
			BNXEF_DEBUG(dsp, ERROR, -1,
				"%s%d: Unable to send FIP FLOGI frame\n",
				BNXEF_DRIVER_NAME, dsp->instance);
			if (fdisc_sent == 0) {
				bnxef_offline_bind_port_handler(dsp, portp,
					pip, bip);
				return ((opaque_t)portp);
			} else {
				for (i = 0; i < BNXEF_MAX_HASH_BUCKETS; i++)
					(void) bnxef_session_list_fini(dsp,
						portp,  i);
				bnxef_unsol_buf_list_fini(portp);
				(void) bnxef_port_list_remove(dsp, portp);
				kmem_cache_free(dsp->port_cache, portp);
				pip->pi_error = FC_BADPORT;
				return (NULL);
			}

		}

		mutex_enter(&portp->port_mutex);
#if defined(__S11) || defined(__S12)
		if (bip->port_npiv)
			portp->fip_state |= FIP_FDISC_SENT;
		else
#endif
			portp->fip_state |= FIP_FLOGI_SENT;
		ret = cv_timedwait(&portp->id_cv, &portp->port_mutex,
				ddi_get_lbolt()+ BNXEF_SEC2TICK(2));
		if (ret < 0) {
			/* Time out has happened when waiting for response */
			mutex_exit(&portp->port_mutex);
			BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Response to FLOGI or FDISC not received\n",
			BNXEF_DRIVER_NAME, dsp->instance);
			if (fdisc_sent == 0)
				continue;
			else
				mutex_enter(&portp->port_mutex);
		}

		if ((portp->fip_state & FIP_FLOGI_LS_RJT_RECEIVED) ||
			(portp->fip_state & FIP_FDISC_LS_RJT_RECEIVED)) {
			fip_state = portp->fip_state;
			mutex_exit(&portp->port_mutex);
			/*
			 * Now LS RJT has been received for a FIP FLOGI or
			 * FIP FIDSC
			 */
			BNXEF_DEBUG(dsp, ERROR, -1,
				"%s%d: LS RJT received, Reason %d, Action %d\
				fip_state 0x%x\n",
				BNXEF_DRIVER_NAME, dsp->instance,
				portp->fip_reason,
				portp->fip_action, fip_state);
			if (fip_state & FIP_FDISC_LS_RJT_RECEIVED)
				break;
			else
				continue;
		}
		if ((portp->fip_state & FIP_FLOGI_LS_ACC_RECEIVED) ||
			(portp->fip_state & FIP_FDISC_LS_ACC_RECEIVED)) {
			mutex_exit(&portp->port_mutex);
			break;
		}
		mutex_exit(&portp->port_mutex);

	}
	mutex_enter(&portp->port_mutex);


	if (((portp->fip_state & FIP_FLOGI_LS_ACC_RECEIVED) == 0) &&
		(portp->fip_state & FIP_FLOGI_SENT)) {
		mutex_exit(&portp->port_mutex);
		cmn_err(CE_WARN, "%s%d: Unable to login to switch - Retry",
			BNXEF_DRIVER_NAME, dsp->instance);
		bnxef_offline_bind_port_handler(dsp, portp, pip, bip);
		return ((opaque_t)portp);
	}

	if (((portp->fip_state & FIP_FDISC_LS_ACC_RECEIVED) == 0) &&
		(portp->fip_state & FIP_FDISC_SENT)) {
		mutex_exit(&portp->port_mutex);
		// bnxef_offline_bind_port_handler(dsp, portp, pip, bip);
		for (i = 0; i < BNXEF_MAX_HASH_BUCKETS; i++)
			(void) bnxef_session_list_fini(dsp, portp,  i);
		bnxef_unsol_buf_list_fini(portp);
		(void) bnxef_port_list_remove(dsp, portp);
		kmem_cache_free(dsp->port_cache, portp);
		pip->pi_error = FC_BADPORT;
		return (NULL);
	}


	/* Now copy this Plogi information into portinfo structure */
	bcopy(&portp->plogi_info, &pip->pi_login_params,
		sizeof (la_els_logi_t));

	pip->pi_s_id.port_id = portp->id;
	pip->pi_port_state |= FC_STATE_10GBIT_SPEED | FC_STATE_ONLINE;
	portp->flags |= BNXEF_FC_PORT_SOL_BOUND;
	mutex_exit(&portp->port_mutex);

#if defined(__S11) || defined(__S12)
	if (!bip->port_npiv) {
#endif
		/*
		 * Now  enforce that DISC advertizement should
		 * should come periodically from Nexus switch
		 */
		tid = timeout(bnxef_fip_disc_adv_enforce,
			(void *)dsp, BNXEF_SEC2TICK((3 * portp->vn_period)));
		MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
		dsp->fip_keep_alive_tmr_hdl = tid;
		dsp->fip_disc_status |= FIP_KEEP_ALIVE_TIMER_ARMED;
		mutex_exit(&dsp->state_mutex);
#if defined(__S11) || defined(__S12)
	}
#endif
#if defined(__S11) || defined(__S12)
		if (!bip->port_npiv) {
#endif
			portp->kaarg1.portp = portp;
			portp->kaarg1.flags = 0;
			tid = timeout(bnxef_timeout_fip_ka, &portp->kaarg1,
			BNXEF_SEC2TICK((dsp->fcf_ident->fka_adv_period/1000)));

			portp->kaarg2.portp = portp;
			portp->kaarg2.flags = VN_PORT_FLAG;
			tid2  = timeout(bnxef_timeout_fip_ka, &portp->kaarg2,
					BNXEF_SEC2TICK(FKA_VN_PERIOD));
			mutex_enter(&portp->port_mutex);
			portp->keep_alive_tmr2_hdl = tid2;
			portp->fip_state |= FIP_KEEP_ALIVE_TIMER2_ARMED;
			mutex_exit(&portp->port_mutex);
#if defined(__S11) || defined(__S12)
		} else {
			portp->kaarg1.portp = portp;
			portp->kaarg1.flags = 0;
			tid = timeout(bnxef_timeout_fip_ka, &portp->kaarg1,
				BNXEF_SEC2TICK(portp->vn_period));
		}
#endif
		mutex_enter(&portp->port_mutex);
			portp->keep_alive_tmr_hdl = tid;
		portp->fip_state |= FIP_VIRTUAL_LINK_MAINTENANCE_IN_PROGRESS;
		portp->fip_state |= FIP_KEEP_ALIVE_ARMED;
		portp->count = 0;
		mutex_exit(&portp->port_mutex);


	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	dsp->num_fc_ports++;
	/* Move the FIP state to VLINK Maint state */
	dsp->fip_disc_state = FIP_VLINK_MAINT_STATE;
	mutex_exit(&dsp->state_mutex);

	bnxef_populate_hba_fru_details(dsp, pip);
	/*
	 * Schedule a TAsk Queue so that, port state is indicated to
	 * higher level layer. If we have come this far that means the
	 * the physical port is online, so inform the bound port is online
	 * as well by calling the state change callback with Online event.
	 */
	ret = ddi_taskq_dispatch(dsp->tq, bnxef_fc_port_online_startup, portp,
			DDI_SLEEP);
	ASSERT(ret == DDI_SUCCESS);

	/* Everything is good, Now we have got the ID */
	return ((opaque_t)portp);
}


/*
 * The following routine unbinds the port indicated by the fca_handle.
 */
void
bnxef_unbind_port(opaque_t fca_handle)
{

	bnxef_port_t *portp = (bnxef_port_t *)fca_handle;
	bnxef_driver_state_t *dsp;
	int i, ret;

	ASSERT(portp != NULL);

	dsp = portp->statep;

	ASSERT(dsp != NULL);

	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	dsp->num_fc_ports --;
	mutex_exit(&dsp->state_mutex);

	BNXEF_DEBUG(dsp, GENERAL, -1,
		"%s%d: Unbind port called for 0x%p, for id 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, portp, portp->id);
	mutex_enter(&portp->port_mutex);
	/* If it not already bound by solaris, we donot want to do anything */
	if ((portp->flags & BNXEF_FC_PORT_SOL_BOUND) !=
		BNXEF_FC_PORT_SOL_BOUND) {
		/* Port is not bound, should not have happened */
		cmn_err(CE_PANIC, "%s%d:unbind request invalid \n",
			BNXEF_DRIVER_NAME, dsp->instance);
		mutex_exit(&portp->port_mutex);
	}
	portp->flags &= ~BNXEF_FC_PORT_SOL_BOUND;
	mutex_exit(&portp->port_mutex);

	if (portp->flags & BNXEF_FC_PORT_ID_ACQUIRED) {
		/*
		 * Send the  Fabric logo to give back the MAC address
		 * and port id
		 */
		ret =  bnxef_fip_sm_send_logo(dsp, portp);
		if (ret != BNXEF_SUCCESS) {
			BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to send FIP LOGO to clear Vlink\n",
			BNXEF_DRIVER_NAME, dsp->instance);
			mutex_enter(&portp->port_mutex);
		} else {
			/* Wait for Accept to come back */
			mutex_enter(&portp->port_mutex);
			portp->fip_state |= FIP_LOGO_SENT;
			ret = cv_timedwait(&portp->id_cv, &portp->port_mutex,
			ddi_get_lbolt() + BNXEF_SEC2TICK(1));
			if (ret < 0) {
				/*
				 * Time out has happened when
				 * waiting for response
				 */
				BNXEF_DEBUG(dsp, ERROR, portp->id,
				    "%s%d: Response to LOGO not\
					received port id 0x%x\n",
				BNXEF_DRIVER_NAME, dsp->instance, portp->id);
			}

			if ((portp->fip_state & FIP_LOGO_LS_RJT_RECEIVED)) {
				mutex_exit(&portp->port_mutex);
				/* Now LS RJT has been received for a LOGO */
				BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d: LS RJT received, Reason %d, Action %d\n",
					BNXEF_DRIVER_NAME, dsp->instance,
					portp->fip_reason,
					portp->fip_action);
			}
		}

		/* LS ACC has been received for LOGO, go for cleanup */
		/*
		 * Also check whether all unsolicited buffer have been freed
		 */
		if (portp->unsol_buf_header.num_items > 0) {
			BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d: Some Unsolicited buffers not freed,\
				unable to unbind\n",
				BNXEF_DRIVER_NAME, dsp->instance);
			mutex_exit(&portp->port_mutex);
		}
		mutex_exit(&portp->port_mutex);
	}


	bnxef_cleanup_port(portp);

	/* THis will deallocate the fip_task allocated  and reserved */
	bnxef_task_free(dsp, portp->fip_task);


	/* Uninit the unsolicited buffer olist */
	(void) bnxef_unsol_buf_list_fini(portp);

	/* Uninitalize all the hash buckets initialized to hold sessions */
	for (i = 0; i < BNXEF_MAX_HASH_BUCKETS; i++)
		(void) bnxef_session_list_fini(dsp, portp, i);


	(void) bnxef_port_list_remove(dsp, portp);
	kmem_cache_free(dsp->port_cache, portp);

}

int
bnxef_start_task(bnxef_driver_state_t *dsp, fc_packet_t *pktp, int sleep)
{
	int num_items;
	bnxef_fcoe_task_t *taskp;


	num_items = bnxef_get_num_free_list_items(dsp);
	if (num_items < BNXEF_NUM_TASKS_ALLOWED_FOR_LV_IOS) {
		pktp->pkt_fca_private = NULL;
		return (FC_NOMEM);
	}

	/* Now allocate a task structure from the free list for use */
	taskp = bnxef_task_alloc(dsp, sleep);
	if (taskp == NULL) {
		BNXEF_DEBUG(dsp, GENERAL, -1,
			"%s%d: Unable to allocate task structure\n",
			BNXEF_DRIVER_NAME, dsp->instance);
			pktp->pkt_fca_private = NULL;
		return (FC_NOMEM);
	}

	BNXEF_DEBUG(dsp, GENERAL, -1,
		"%s%d: TAsk pointer allocated 0x%p",
		BNXEF_DRIVER_NAME, dsp->instance, taskp);
	mutex_enter(&taskp->task_mutex);
	taskp->nextp = NULL;
	taskp->snextp = NULL;
	taskp->sprevp = NULL;
	taskp->sessionp = NULL;
	taskp->pktp = pktp;
	mutex_exit(&taskp->task_mutex);

	/* THis initializes the pointer space allocated by LV with taskp ptr */
	pktp->pkt_fca_private = (void *)taskp;

	return (FC_SUCCESS);
}

/*
 * The following routine allows to initialize the fc_packet private area for
 * the FCA driver.
 */
/* ARGSUSED2 */
int
bnxef_init_pkt(opaque_t fca_handle, fc_packet_t *pktp, int sleep)
{
	bnxef_driver_state_t *dsp;
	bnxef_port_t	 *portp;

	ASSERT(pktp != NULL);

	portp = (bnxef_port_t *)fca_handle;
	ASSERT(portp != NULL);
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	/* Check whether this port is bound */
	mutex_enter(&portp->port_mutex);
	if ((portp->flags &  BNXEF_FC_PORT_SOL_BOUND) !=
		BNXEF_FC_PORT_SOL_BOUND) {
		mutex_exit(&portp->port_mutex);
			pktp->pkt_fca_private = NULL;
		return (FC_UNBOUND);
	}
	portp->num_pkts_inited++;
	mutex_exit(&portp->port_mutex);

	pktp->pkt_fca_private = NULL;

	return (FC_SUCCESS);
}

int
bnxef_end_task(bnxef_driver_state_t *dsp, fc_packet_t *pktp)
{

	bnxef_fcoe_task_t *taskp;
	void *framep;
	void *payldp;
	void *orgpayldp;
	uint32_t payldsz;
	uint32_t taskflgs;

	taskp = (bnxef_fcoe_task_t *)pktp->pkt_fca_private;
	if (taskp == NULL) {
		BNXEF_DEBUG(dsp, GENERAL, -1, "%s%d: Task Pointer NuLL",
			BNXEF_DRIVER_NAME, dsp->instance);
		return (FC_BADPACKET);
	}



	mutex_enter(&taskp->task_mutex);
	framep = taskp->frame_hdrp;
	payldp = taskp->payloadp;
	payldsz = taskp->payloadsz;
	orgpayldp = taskp->orig_payloadp;
	taskflgs = taskp->task_flags;
	taskp->payloadp = NULL;
	taskp->frame_hdrp = NULL;
	taskp->payloadsz = 0;
	taskp->snextp = NULL;
	taskp->sprevp = NULL;
	taskp->pktp = NULL;
	mutex_exit(&taskp->task_mutex);

	if (taskflgs & BNXEF_FCOE_TASK_FRAME_HDR_ALLOC) {
		kmem_free(framep, sizeof (fc_frame_hdr_t));
	}

	if (taskflgs & BNXEF_FCOE_TASK_PAYLOAD_ALLOC) {
		kmem_free(payldp, payldsz);
	}

	if (taskflgs & BNXEF_FCOE_TASK_ORIG_PAYLOAD_ALLOC) {
		kmem_free(orgpayldp, payldsz);
	}
	mutex_enter(&taskp->task_mutex);
	if (taskflgs & BNXEF_FCOE_TASK_FRAME_HDR_ALLOC) {
		taskp->task_flags &= ~BNXEF_FCOE_TASK_FRAME_HDR_ALLOC;
	}
	if (taskflgs & BNXEF_FCOE_TASK_PAYLOAD_ALLOC) {
		taskp->task_flags &= ~BNXEF_FCOE_TASK_PAYLOAD_ALLOC;
	}
	if (taskflgs & BNXEF_FCOE_TASK_ORIG_PAYLOAD_ALLOC) {
		taskp->task_flags &= ~BNXEF_FCOE_TASK_ORIG_PAYLOAD_ALLOC;
	}
	if (taskflgs & BNXEF_FCOE_TASK_COMPLETED) {
		taskp->task_flags &= ~BNXEF_FCOE_TASK_COMPLETED;
		taskp->task_flags &= ~(BNXEF_FCOE_TASK_ELS |
					BNXEF_FCOE_TASK_ABTS |
					BNXEF_FCOE_TASK_SCSI |
					BNXEF_FCOE_TASK_RRQ |
					BNXEF_FCOE_TASK_UNSOLICITED_ELS);
	}
	mutex_exit(&taskp->task_mutex);

	BNXEF_DEBUG(dsp, GENERAL, -1,
		"%s%d: Freeing the Task pointer 0x%p",
		BNXEF_DRIVER_NAME, dsp->instance, taskp);
	if ((taskp->task_flags & BNXEF_FCOE_TASK_UNSOL_LOCAL_RESP) == 0) {
		bnxef_task_free(dsp, taskp);
		pktp->pkt_fca_private = NULL;
	} else {
		taskp->task_flags &= ~BNXEF_FCOE_TASK_UNSOL_LOCAL_RESP;
	}

	return (FC_SUCCESS);
}

/*
 * The following routine does the uninitialization of FCA private area.
 */
int
bnxef_un_init_pkt(opaque_t fca_handle, fc_packet_t *pktp)
{
	bnxef_driver_state_t *dsp;
	bnxef_port_t	 *portp;


	ASSERT(pktp != NULL);

	portp = (bnxef_port_t *)fca_handle;
	ASSERT(portp != NULL);
	dsp = portp->statep;
	ASSERT(dsp != NULL);

	/* Check whether this port is bound */
	mutex_enter(&portp->port_mutex);
	if ((portp->flags &  BNXEF_FC_PORT_SOL_BOUND) !=
		BNXEF_FC_PORT_SOL_BOUND) {
		mutex_exit(&portp->port_mutex);
		return (FC_UNBOUND);
	}
	if (portp->num_pkts_inited > 0)
		portp->num_pkts_inited--;
	mutex_exit(&portp->port_mutex);

	return (FC_SUCCESS);
}

/*
 * Unsolicited buffer allocation routine.
 */
int
bnxef_ub_alloc(opaque_t fca_handle, uint64_t tokens[], uint32_t size,
	uint32_t *countp, uchar_t type)
{
	bnxef_port_t *portp;
	bnxef_driver_state_t *dsp;
	uint32_t	i;
	fc_unsol_buf_t  *ubp;
	caddr_t		bufp;
	bnxef_unsol_buf_t *dubp;
	int	ret;

	ASSERT(countp != NULL);

	portp = (bnxef_port_t *)fca_handle;
	dsp =	portp->statep;

	ASSERT(portp != NULL);
	ASSERT(dsp != NULL);

	BNXEF_DEBUG(dsp, ERROR, -1,
		"%s%d: Ub alloc called Size 0x%x, Count 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, size, *countp);

	if (size > BNXEF_UB_SIZE_LIMIT) {
		*countp = 0;
		return (FC_NOMEM);
	}

	mutex_enter(&portp->port_mutex);
	if ((portp->flags & BNXEF_FC_PORT_SOL_BOUND) == 0) {
		mutex_exit(&portp->port_mutex);
		return (FC_UNBOUND);
	}
	mutex_exit(&portp->port_mutex);

	BNXEF_DEBUG(dsp, ERROR, -1, "%s%d: dsp->unsol_alloc_cnt 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, dsp->unsol_alloc_cnt);
	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	if (*countp  > BNXEF_UB_COUNT_LIMIT) {
		mutex_exit(&dsp->state_mutex);
		*countp = 0;
		return (FC_TOOMANY);
	}
	mutex_exit(&dsp->state_mutex);

	BNXEF_DEBUG(dsp, GENERAL, -1, "%s%d: Request to allocate %d buffers",
		BNXEF_DRIVER_NAME, dsp->instance, *countp);
	/*
	 * Allocate the requested unsolicited buffers
	 */
	for (i = 0; i < *countp; i++) {
		/* Allocate the fc_unsol_buf structure first */
		ubp = kmem_zalloc(sizeof (fc_unsol_buf_t), KM_NOSLEEP);
		if (ubp == NULL) {
			BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d: Unable to allocate unsol_buf_t struct\n"
				BNXEF_DRIVER_NAME, dsp->instance);
			goto bnxef_ub_alloc_failed;
		}

		bufp = kmem_zalloc(size, KM_NOSLEEP);
		if (bufp == NULL) {
			BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d: Unable to allocate buffer required\n",
				BNXEF_DRIVER_NAME, dsp->instance);
			kmem_free(ubp, sizeof (fc_unsol_buf_t));
			goto bnxef_ub_alloc_failed;
		}

		dubp = kmem_cache_alloc(dsp->unsol_buf_cache, KM_NOSLEEP);
		if (dubp == NULL) {
			BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d: Unable to allocate driver unsol buf\n",
				BNXEF_DRIVER_NAME, dsp->instance);
			kmem_free(ubp, sizeof (fc_unsol_buf_t));
			kmem_free(bufp, size);
			goto bnxef_ub_alloc_failed;
		}

		/* Fill in the required files in fc_unsol_buf_t structure */
		ubp->ub_fca_private = dubp;
		ubp->ub_buffer	 = bufp;
		ubp->ub_bufsize = size;
		ubp->ub_port_handle = fca_handle;


		/* Fill in all the information into the driver unsol buffer */
		mutex_enter(&dubp->unsol_mutex);
		dubp->unsol_bufp =  ubp;
		dubp->type  = type;
		dubp->nextp = NULL;
		dubp->portp = portp;
		dubp->flags = BNXEF_UB_RELEASED;
		mutex_exit(&dubp->unsol_mutex);

		tokens[i] = ubp->ub_token = (uint64_t)dubp;
		BNXEF_DEBUG(dsp, GENERAL, -1,
			"%s%d Token returned for Index %d is %p",
			BNXEF_DRIVER_NAME, dsp->instance, i, tokens[i]);

		/* Add this unsolbuffer into the unsol buf list maintained */
		ret = bnxef_unsol_buf_list_add(portp, dubp);
		if (ret != BNXEF_SUCCESS) {
			BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d; Unable to add to list\n",
				BNXEF_DRIVER_NAME, dsp->instance);
			kmem_cache_free(dsp->unsol_buf_cache, dubp);
			kmem_free(ubp, sizeof (fc_unsol_buf_t));
			kmem_free(bufp, size);
			goto bnxef_ub_alloc_failed;
		}
	}

	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	dsp->unsol_alloc_cnt += *countp;
	mutex_exit(&dsp->state_mutex);
	return (FC_SUCCESS);


bnxef_ub_alloc_failed:
	*countp = i;

	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	dsp->unsol_alloc_cnt += *countp;
	mutex_exit(&dsp->state_mutex);

	return (FC_NOMEM);
}


/*
 * Unsolicited Buffer release
 */
int
bnxef_ub_release(opaque_t fca_handle, uint32_t count, uint64_t tokens[])
{
	bnxef_port_t *portp;
	bnxef_driver_state_t *dsp;
	uint32_t	i;
	bnxef_unsol_buf_t *dubp;

	ASSERT(count > 0);

	portp = (bnxef_port_t *)fca_handle;
	dsp =	portp->statep;

	ASSERT(portp != NULL);
	ASSERT(dsp != NULL);

	BNXEF_DEBUG(dsp, ERROR, -1, "%s%d: Ub Release called for %d count",
		BNXEF_DRIVER_NAME, dsp->instance, count);
	/* Check whether the port is bound to solaris */
	mutex_enter(&portp->port_mutex);
	if ((portp->flags & BNXEF_FC_PORT_SOL_BOUND) == 0) {
		mutex_exit(&portp->port_mutex);
		return (FC_UNBOUND);
	}
	mutex_exit(&portp->port_mutex);

	for (i = 0; i < count; i++) {
		if (tokens[i] == NULL) {
			BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d: Bad token provided  at index %d\n",
				BNXEF_DRIVER_NAME, dsp->instance, i);
			goto bnxef_bad_token_error;
		}

		dubp = (bnxef_unsol_buf_t *)tokens[i];
		BNXEF_DEBUG(dsp, ERROR, -1, "%s%d: Release UBUF for token %p",
			BNXEF_DRIVER_NAME, dsp->instance, dubp);
		mutex_enter(&dubp->unsol_mutex);
		if ((dubp->flags & BNXEF_UB_RELEASED)) {
			mutex_exit(&dubp->unsol_mutex);
			BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d: Bad token provided for release at %d\n"
				BNXEF_DRIVER_NAME, dsp->instance, i);
			goto bnxef_bad_token_error;
		}

		dubp->flags |= BNXEF_UB_RELEASED;
		mutex_exit(&dubp->unsol_mutex);
	}

	return (FC_SUCCESS);

bnxef_bad_token_error:
	return (FC_UB_BADTOKEN);
}

/*
 * The following frees the unsolicited buffer allocated
 */
int
bnxef_ub_free(opaque_t fca_handle, uint32_t count, uint64_t tokens[])
{
	bnxef_port_t *portp;
	bnxef_driver_state_t *dsp;
	uint32_t	i;
	bnxef_unsol_buf_t *dubp;
	fc_unsol_buf_t  *ubp;
	int	 ret;

	portp = (bnxef_port_t *)fca_handle;
	dsp =	portp->statep;

	ASSERT(portp != NULL);
	ASSERT(dsp != NULL);

	BNXEF_DEBUG(dsp, GENERAL, -1,
		"%s%d: Entering Ub Free routine- count %d",
		BNXEF_DRIVER_NAME, dsp->instance, count);
	if (count == 0)
		return (FC_SUCCESS);
	/* Check whether the port is bound to solaris */
	mutex_enter(&portp->port_mutex);
	if ((portp->flags & BNXEF_FC_PORT_SOL_BOUND) == 0) {
		mutex_exit(&portp->port_mutex);
		return (FC_UNBOUND);
	}
	mutex_exit(&portp->port_mutex);

	BNXEF_DEBUG(dsp, GENERAL, -1, "%s%d: UB Release called for %d count",
			BNXEF_DRIVER_NAME, dsp->instance, count);
	for (i = 0; i < count; i++) {
		if (tokens[i] == NULL) {
			BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d: Bad token provided  at index %d\n",
				BNXEF_DRIVER_NAME, dsp->instance, i);
			goto bnxef_bad_token_error;
		}

#if 0
		dubp = (bnxef_unsol_buf_t *)tokens[i];
		mutex_enter(&dubp->unsol_mutex);
		if ((dubp->flags & BNXEF_UB_RELEASED) == 0) {
			mutex_exit(&dubp->unsol_mutex);
			BNXEF_DEBUG(dsp, ERROR, portp->id,
				"%s%d:Bad Unsol buf not with FCA at index %d\n",
				BNXEF_DRIVER_NAME, dsp->instance, i);
			goto bnxef_bad_token_error;
		}
		mutex_exit(&dubp->unsol_mutex);
#endif
	}

	BNXEF_DEBUG(dsp, GENERAL, -1,
		"%s%d: All %d count of ub buffer in released state",
		BNXEF_DRIVER_NAME, dsp->instance, count);

	for (i = 0; i < count; i++) {
		dubp = (bnxef_unsol_buf_t *)tokens[i];
		mutex_enter(&dubp->unsol_mutex);
		ubp  = dubp->unsol_bufp;
		mutex_exit(&dubp->unsol_mutex);

		ASSERT(ubp != NULL);

		kmem_free(ubp->ub_buffer, ubp->ub_bufsize);
		kmem_free(ubp, sizeof (fc_unsol_buf_t));
		/* Remove the driver unsolicited buffer from list */
		ret = bnxef_unsol_buf_list_remove(portp, dubp);
		if (ret != BNXEF_SUCCESS) {
			/* We cannot recover because we have freed some mem */
			cmn_err(CE_PANIC,
				"%s%d: Unable to remove from Unsol buf list\n",
				BNXEF_DRIVER_NAME, dsp->instance);
			goto  bnxef_bad_token_error;
		}

		BNXEF_DEBUG(dsp, GENERAL, -1, "%s%d: Token %p freed",
			BNXEF_DRIVER_NAME, dsp->instance, dubp);
		kmem_cache_free(dsp->unsol_buf_cache, (void *)dubp);
		MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
		dsp->unsol_alloc_cnt --;
		mutex_exit(&dsp->state_mutex);
	}
	BNXEF_DEBUG(dsp, GENERAL, -1, "%s%d: All ub buffers released",
		BNXEF_DRIVER_NAME, dsp->instance);

	return (FC_SUCCESS);

bnxef_bad_token_error:
	return (FC_UB_BADTOKEN);
}

/*
 * Get Capabilities
 */

int
bnxef_get_cap(opaque_t fca_handle, char *cap, void *bufp)
{
	bnxef_port_t *portp;
	bnxef_driver_state_t *dsp;
	uint32_t *rptr = (uint32_t *)bufp;
	svc_param_t *class3_param;
	int retval = FC_CAP_FOUND;

	ASSERT(cap != NULL);
	ASSERT(bufp != NULL);

	portp = (bnxef_port_t *)fca_handle;
	dsp = portp->statep;

	BNXEF_DEBUG(dsp, GENERAL, portp->id, "%s%d: Get cap called",
		BNXEF_DRIVER_NAME, dsp->instance);
	ASSERT(portp != NULL);
	ASSERT(dsp != NULL);
	mutex_enter(&portp->port_mutex);
	if ((portp->flags & BNXEF_FC_PORT_SOL_BOUND) == 0) {
		mutex_exit(&portp->port_mutex);
		return (FC_UNBOUND);
	}

	if (strcmp(cap, FC_NODE_WWN) == 0) {
		bcopy((uint8_t *)portp->plogi_info.node_ww_name.raw_wwn,
			(uint8_t *)bufp, 8);
	} else if (strcmp(cap, FC_LOGIN_PARAMS) == 0) {
		bcopy((uint8_t *)&portp->plogi_info, (uint8_t *)bufp,
				sizeof (la_els_logi_t));
	} else if (strcmp(cap, FC_CAP_UNSOL_BUF) == 0) {
		*rptr = (uint32_t)BNXEF_UB_COUNT_LIMIT;
	} else if (strcmp(cap, FC_CAP_NOSTREAM_ON_UNALIGN_BUF) == 0) {
		*rptr = (uint32_t)FC_ALLOW_STREAMING;
	} else if (strcmp(cap, FC_CAP_PAYLOAD_SIZE) == 0) {
		class3_param = &portp->plogi_info.class_3;
		*rptr = (uint32_t)class3_param->rcv_size;
	} else if (strcmp(cap, FC_CAP_POST_RESET_BEHAVIOR) == 0) {
		*rptr = (uint32_t)FC_RESET_RETURN_ALL;
	} else if (strcmp(cap, FC_CAP_FCP_DMA) == 0) {
		*rptr = FC_DVMA_SPACE;	/* Is this Good for x86 */
	} else
		retval = FC_CAP_ERROR;
	mutex_exit(&portp->port_mutex);
	return (retval);
}

/*
 * The following sets the capabilities that can be changed.
 */
int
bnxef_set_cap(opaque_t fca_handle, char *cap, void *bufp)
{
	bnxef_port_t *portp;
	bnxef_driver_state_t *dsp;

	ASSERT(cap != NULL);
	ASSERT(bufp != NULL);

	portp = (bnxef_port_t *)fca_handle;
	dsp = portp->statep;

	ASSERT(portp != NULL);
	ASSERT(dsp != NULL);
	mutex_enter(&portp->port_mutex);
	if ((portp->flags & BNXEF_FC_PORT_SOL_BOUND) == 0) {
		mutex_exit(&portp->port_mutex);
		return (FC_UNBOUND);
	}

	if (strcmp(cap, FC_NODE_WWN) == 0) {
		return (FC_CAP_FOUND);
	} else if (strcmp(cap, FC_LOGIN_PARAMS) == 0) {
		return (FC_CAP_FOUND);
	} else if (strcmp(cap, FC_CAP_UNSOL_BUF) == 0) {
		return (FC_CAP_FOUND);
	} else if (strcmp(cap, FC_CAP_PAYLOAD_SIZE) == 0) {
		return (FC_CAP_FOUND);
	} else if (strcmp(cap, FC_CAP_POST_RESET_BEHAVIOR) == 0) {
		return (FC_CAP_FOUND);
	}

	return (FC_CAP_ERROR);
}


/*
 * This will return the map if it is avilable. In our case there is no map
 * as we are an FCoE device.
 */
/*ARGSUSED*/
int
bnxef_getmap(opaque_t fca_handle, fc_lilpmap_t *mbp)
{

	NOTE(ARGUNUSED(fca_handle))
	NOTE(ARGUNUSED(mbp))
	return (FC_NOMAP);
}

/*
 * Abort a specific exchange
 */
int
bnxef_abort(opaque_t fca_handle, fc_packet_t *pktp, int flags)
{
	bnxef_port_t	 *portp;
	bnxef_fcoe_task_t *taskp;
	bnxef_session_t	  *sessionp;
	bnxef_driver_state_t *dsp;
	int ret;

	portp = (bnxef_port_t *)fca_handle;
	ASSERT(portp != NULL);

	dsp = portp->statep;
	ASSERT(dsp != NULL);

	taskp = pktp->pkt_fca_private;
	ASSERT(taskp != NULL);

	sessionp = taskp->sessionp;
	ASSERT(sessionp != NULL);

	/*
	 * Remember for FCP which we care, there are called with KM_SLEEP
	 * But for FCIP which we donot care it is called with KM_NOSLEEP
	 * therefore it is safe to assume that abort in our scenario will
	 * never be called with KM_NOSLEEP as we donot care abut fcip.
	 */
	ASSERT(flags == KM_SLEEP);

	cmn_err(CE_NOTE, "%s%d: %s called for Task 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, __func__, taskp->x_id);
	/*
	 * This will take through the complete state machine to
	 * complete the task in error.
	 */
	ret = bnxef_initiate_abts(sessionp, taskp);
	if (ret != BNXEF_SUCCESS) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Unable to initiate ABTS\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		return (FC_ABORT_FAILED);
	}

	return (FC_SUCCESS);
}


/*
 * This is the reset routine, we donot support any of the commands documented
 * Therefore return FC_FAILURE without checking anything.
 */
int
bnxef_reset(opaque_t fca_handle, uint32_t cmd)
{
	bnxef_port_t *portp = (bnxef_port_t *)fca_handle;
	bnxef_driver_state_t *dsp;

	ASSERT(portp != NULL);
	dsp = portp-> statep;
	ASSERT(dsp != NULL);

	BNXEF_DEBUG(dsp, GENERAL, portp->id,
		"%s%d: %s Unable to perform given cmd %d\n",
		BNXEF_DRIVER_NAME, dsp->instance, __func__, cmd);
	cmn_err(CE_NOTE, "%s%d: %s called on Port 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, __func__, portp->id);

	return (FC_FAILURE);
}


int
bnxef_port_manage(opaque_t fca_handle, fc_fca_pm_t *cmdp)
{

	bnxef_port_t *portp = (bnxef_port_t *)fca_handle;
	bnxef_driver_state_t *dsp;

	ASSERT(portp != NULL);
	dsp = portp->statep;

	ASSERT(dsp != NULL);

	/* we donot support any of the port manage command s */
	BNXEF_DEBUG(dsp, GENERAL, portp->id,
		"%s%d: %s Unable to perform given cmd %d\n",
		BNXEF_DRIVER_NAME, dsp->instance, __func__, *cmdp);


	return (FC_FAILURE);
}

/*
 * The following is the notify routine. But we donot support any of
 * the notify commands.
 */
int
bnxef_notify(opaque_t fca_handle, uint32_t cmd)
{
	bnxef_port_t *portp = (bnxef_port_t *)fca_handle;
	bnxef_driver_state_t *dsp;

	ASSERT(portp != NULL);
	dsp = portp->statep;

	ASSERT(dsp != NULL);

	/* we donot support any of the notify commands */
	BNXEF_DEBUG(dsp, GENERAL, portp->id,
		"%s%d: %s Unable to perform given cmd %d\n",
		BNXEF_DRIVER_NAME, dsp->instance, __func__, cmd);


	return (FC_FAILURE);
}


/*
 * This handles the SCSI command transport and FC_CT transport.
 */
int
bnxef_transport(opaque_t fca_handle, fc_packet_t *pktp)
{
	bnxef_driver_state_t	 *dsp;
	bnxef_port_t		 *portp;
	bnxef_fcoe_task_t	*taskp;
	uint8_t			phyport_state;
	int			ret = -1;
	uint32_t		port_flags;
	uint32_t		d_id;
	uint8_t			*macaddrp;
	uint16_t		vlan_id;
	bnxef_session_t		*sesp;
	uint32_t		state;


	portp = (bnxef_port_t *)fca_handle;
	ASSERT(portp != NULL);

	dsp = portp->statep;
	ASSERT(dsp != NULL);

	if (pktp->pkt_fca_private == NULL)
		bnxef_start_task(dsp, pktp, KM_SLEEP);

	taskp = pktp->pkt_fca_private;
	if (taskp == NULL) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: TAsk not allocated cannot transport \n",
			BNXEF_DRIVER_NAME, dsp->instance);
		/*
		 * THis is a workaround for fcp_watch thread timeout.
		 * under some stress test condition when we say we
		 * donot have task resources to handle, this guy
		 * prematurely timeout and some of the stress cases
		 * exit with I/O error. Only under stress when, we
		 * return we donot have resources, we make sure the
		 * fcp_watch thread does not identify wrongly timeout
		 * of a packet.
		 */
		if (pktp->pkt_timeout < 60) {
			pktp->pkt_timeout = 60 + 4;
		}
		return (FC_TRAN_BUSY);
	}

	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	phyport_state = dsp->physical_port_state;
	state = dsp->state;
	mutex_exit(&dsp->state_mutex);

	if (!phyport_state) {
		bnxef_end_task(dsp, pktp);
		return (FC_TRANSPORT_ERROR);
	}

	if (state & BNXEF_IMPLICIT_FABRIC_LOGO_IN_PROGRESS) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: Implicit fabric logo in progress - Bsy ret",
			BNXEF_DRIVER_NAME, dsp->instance);
		bnxef_end_task(dsp, pktp);
		return (FC_TRAN_BUSY);
	}

	mutex_enter(&portp->port_mutex);
	port_flags = portp->flags;
	mutex_exit(&portp->port_mutex);
	if ((port_flags & BNXEF_FC_PORT_SOL_BOUND) == 0) {
		BNXEF_DEBUG(dsp, ERROR, portp->id,
			"%s%d: Port not bound to solaris, cannot transport\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		bnxef_end_task(dsp, pktp);
		return (FC_UNBOUND);
	}

	/* Set up the response frame header */
	bcopy((void *)&pktp->pkt_cmd_fhdr, (void *)&pktp->pkt_resp_fhdr,
		sizeof (fc_frame_hdr_t));

	switch (pktp->pkt_tran_type) {
	case FC_PKT_NOP:
		/* No operation. Return Success, do not call pkt_comp */
		pktp->pkt_state = FC_PKT_SUCCESS;
		return (FC_SUCCESS);
	case  FC_PKT_OUTBOUND:
	case  FC_PKT_EXCHANGE:
		pktp->pkt_resp_fhdr.r_ctl = 0x03; /* Response to FC-CT */
		break;
	case  FC_PKT_FCP_READ:
	case  FC_PKT_FCP_WRITE:
		pktp->pkt_resp_fhdr.r_ctl = 0x07; /* FCP_RESP */
		/* Supported operation, continue processing */
		break;

	case  FC_PKT_INBOUND:
	case  FC_PKT_IP_WRITE:
	case  FC_PKT_BROADCAST:
	deafult:
		return (FC_TRANSPORT_ERROR);
	}

	pktp->pkt_resp_fhdr.d_id = portp->id;
	pktp->pkt_resp_fhdr.s_id = pktp->pkt_cmd_fhdr.d_id;
	pktp->pkt_resp_fhdr.r_ctl =
		R_CTL_EXTENDED_SVC | R_CTL_SOLICITED_CONTROL;
	pktp->pkt_resp_fhdr.f_ctl =
		F_CTL_XCHG_CONTEXT | F_CTL_LAST_SEQ | F_CTL_END_SEQ;

	/*
	 * If it comes here, that means session should have already
	 * been created and possibly offloaded. Find the session.
	 */
	macaddrp = portp->fcf->mac_addr;
	vlan_id  = portp->fcf->vlan_id;
	/* Determine whether the connection already exists for this D_ID */
	d_id = pktp->pkt_cmd_fhdr.d_id;

	/* find the session if it is already existing */
	sesp = bnxef_session_find_remote_port(portp, vlan_id,  d_id, macaddrp);
	if (sesp == NULL) {
		BNXEF_DEBUG(dsp, ERROR, -1,
			"%s%d: pktp 0x%p, d_id 0x%x, Vlan_id 0x%x",
			BNXEF_DRIVER_NAME, dsp->instance, pktp, d_id, vlan_id);
		/*
		 * We donot have connection,
		 * Therefore nothing can be transported
		 */
		bnxef_complete_pkt(pktp, FC_PKT_FAILURE,
			0, FC_REASON_NO_CONNECTION, 0);
		return (FC_SUCCESS);
	}

	mutex_enter(&taskp->task_mutex);
	taskp->tq_params.reserved1 = 0;
	mutex_exit(&taskp->task_mutex);

	/*
	 * Now check what type of exchange is expected to be started or
	 * finsihed. BAsed on this identification, one could trigget appropriate
	 * handlers.
	 */
	switch (pktp->pkt_cmd_fhdr.type) {
	case FC_TYPE_SCSI_FCP:
		ret = bnxef_scsi_transport_handler(sesp, taskp);
		break;
	case FC_TYPE_FC_SERVICES:
		ret = bnxef_fcct_transport_handler(sesp, taskp);
		break;
	default:
		BNXEF_DEBUG(dsp, SCSI, portp->id,
			"%s%d: Unknown type, cannot be handled\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		return (FC_TRANSPORT_ERROR);
	}

	return (ret);
}
