/*
 * The information contained in this file is confidential and proprietary to
 * Broadcom Corporation.  No part of this file may be reproduced or
 * distributed, in any form or by any means for any purpose, without the
 * express written permission of Broadcom Corporation.
 *
 * (c) COPYRIGHT 2010-2012 Broadcom Corporation, ALL RIGHTS RESERVED.
 */

#pragma ident "(C) 2010-2012 Broadcom Corporation"
#pragma ident "$File$$Revision$$Change$$DateTime$"
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/stat.h>
#include <sys/pci.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>
#include <sys/cmn_err.h>
#include <sys/file.h>
#include <sys/cred.h>
#include <sys/byteorder.h>
#include <sys/atomic.h>
#include <sys/scsi/scsi.h>
#include <sys/modhash.h>
#include <sys/sunldi.h>
#include <sys/kstat.h>
#include <sys/kmem.h>
#include <sys/ethernet.h>
#include <sys/stream.h>
#include <sys/sunldi.h>
/*
 * leadville header files
 */
#include <sys/fibre-channel/fc_types.h>
#include <sys/fibre-channel/impl/fc_fcaif.h>
#include <bnxe_binding.h>
#include <bnxef_tq_param.h>
#include <bnxef_fcf_info.h>
#include <bnxef_hsi.h>
#include <bnxef_fcoe_task.h>
#include <bnxef_fcoe_session.h>
#include <bnxef_state.h>
#include <bnxef_fip.h>
#include <bnxef_fcoe.h>
#include <bnxef_debug.h>

extern void bnxef_big_endian(void *bufp, int size);


/*
 * unsolicited buffer constructor.
 */
/*ARGSUSED*/
int
bnxef_unsol_buf_cache_constructor(void * bufp, void *args, int kmflags)
{

	char name[KSTAT_STRLEN];
	bnxef_unsol_buf_t *unsolbufp = (bnxef_unsol_buf_t *)bufp;
	bnxef_driver_state_t *dsp = (bnxef_driver_state_t *)args;
	uint32_t refcnt;

	NOTE(ARGUNUSED(kmflags))
	MUTEX_ENTER(dsp, &dsp->state_mutex, __func__, __LINE__);
	refcnt = dsp->unsol_buf_ref_cnt;
	dsp->unsol_buf_ref_cnt++;
	mutex_exit(&dsp->state_mutex);

	bzero((uint8_t *)bufp, sizeof (bnxef_unsol_buf_t));
	(void) snprintf(name, KSTAT_STRLEN, "unsolbuf%d_%d-mutex",
		dsp->instance, refcnt);
	mutex_init(&unsolbufp->unsol_mutex, name, MUTEX_DRIVER, NULL);
	return (0);
}

/*
 * Unsolicited buffer destructor
 */
/*ARGSUSED*/
void
bnxef_unsol_buf_cache_destructor(void *bufp, void *args)
{
	bnxef_unsol_buf_t *unsolbufp = (bnxef_unsol_buf_t *)bufp;

	NOTE(ARGUNUSED(args))

	mutex_destroy(&unsolbufp->unsol_mutex);

}

/*
 * Unsolicited buf cache create routine. Only called from attach.
 */
int
bnxef_unsol_buf_cache_create(bnxef_driver_state_t *dsp)
{
	char buf[KSTAT_STRLEN];

	ASSERT(dsp != NULL);

	(void) snprintf(buf, KSTAT_STRLEN, "unsolbuf%d_cache", dsp->instance);
	dsp->unsol_buf_cache = kmem_cache_create(buf,
			sizeof (bnxef_unsol_buf_t),
		8, bnxef_unsol_buf_cache_constructor,
		bnxef_unsol_buf_cache_destructor,  NULL, dsp, 0, KM_SLEEP);
	if (dsp->unsol_buf_cache == NULL) {
		cmn_err(CE_PANIC, "%s%d: Unable to allocate task cache\n",
			BNXEF_DRIVER_NAME, dsp->instance);
		return (DDI_FAILURE);
	}

	return (DDI_SUCCESS);
}

/*
 * This function will destroy the cache created at attach time. This routien
 * can only be called from detach routine.
 */
int
bnxef_unsol_buf_cache_destroy(bnxef_driver_state_t *dsp)
{

	ASSERT(dsp != NULL);

	kmem_cache_destroy(dsp->unsol_buf_cache);

	return (DDI_SUCCESS);
}

/*
 * Initialize the list to hold all the unsolicited buffers allocated
 */
int
bnxef_unsol_buf_list_init(bnxef_port_t *portp)
{

	bnxef_unsol_buf_list_t *unsollstp;
	char name[KSTAT_STRLEN];

	ASSERT(portp != NULL);

	unsollstp = &portp->unsol_buf_header;
	bzero((uint8_t *)unsollstp, sizeof (bnxef_unsol_buf_list_t));
	(void) snprintf(name, KSTAT_STRLEN, "unsolbuflst%d-mutex", portp->id);
	mutex_init(&unsollstp->unsol_buf_list_mutex, name, MUTEX_DRIVER, NULL);

	return (BNXEF_SUCCESS);
}


/*
 * This routine will destroy the unsolicited list mutex and return success.
 */
int
bnxef_unsol_buf_list_fini(bnxef_port_t *portp)
{
	bnxef_unsol_buf_list_t *unsollstp;

	ASSERT(portp != NULL);

	unsollstp = &portp->unsol_buf_header;
	ASSERT(unsollstp != NULL);

	mutex_destroy(&unsollstp->unsol_buf_list_mutex);

	return (BNXEF_SUCCESS);
}

/*
 * This routine is call at the unsolicited buffer allocation time to
 * include the buffer allocated into the list for that port.
 */
int
bnxef_unsol_buf_list_add(bnxef_port_t *portp, bnxef_unsol_buf_t *unsolbufp)
{
	bnxef_unsol_buf_list_t *unsollstp;

	ASSERT(portp != NULL);
	ASSERT(unsolbufp != NULL);

	unsollstp = &portp-> unsol_buf_header;

	ASSERT(unsollstp != NULL);

	mutex_enter(&unsollstp->unsol_buf_list_mutex);

	if ((unsollstp->head == NULL) && (unsollstp->tail == NULL)) {
		unsollstp->head = unsolbufp;
		unsollstp->tail = unsolbufp;
	} else {
		ASSERT(unsollstp->tail != NULL);
		ASSERT(unsollstp->tail->nextp == NULL);
		unsollstp->tail->nextp = unsolbufp;
		unsollstp->tail = unsolbufp;
	}

	unsollstp->num_items ++;
	mutex_exit(&unsollstp->unsol_buf_list_mutex);

	return (BNXEF_SUCCESS);
}

/*
 * The following removes the buffer from the list.
 */
int
bnxef_unsol_buf_list_remove(bnxef_port_t *portp, bnxef_unsol_buf_t *unsolbufp)
{
	bnxef_unsol_buf_list_t *unsollstp;
	bnxef_unsol_buf_t *unsolbufcp;
	bnxef_unsol_buf_t *unsolbufprevp;
	int i, found = 0;
	int ret = BNXEF_FAILURE;

	ASSERT(portp != NULL);
	ASSERT(unsolbufp != NULL);

	unsollstp = &portp->unsol_buf_header;

	ASSERT(unsollstp != NULL);
	ASSERT(unsollstp->num_items > 0);

	mutex_enter(&unsollstp->unsol_buf_list_mutex);
	unsolbufcp = unsollstp->head;
	ASSERT(unsolbufcp != NULL);
	unsolbufprevp = NULL;
	for (i = 0; i < unsollstp->num_items; i++) {
		if (unsolbufcp == unsolbufp) {
			found++;
			break;
		}
		unsolbufprevp = unsolbufcp;
		unsolbufcp = unsolbufcp->nextp;
		ASSERT(unsolbufcp != NULL);
	}

	if (found) {
		if (unsollstp->num_items == 1) {
			/* There is only one item */
			unsollstp->head = NULL;
			unsollstp->tail = NULL;
		} else {
			if (unsolbufprevp != NULL) {
				unsolbufprevp->nextp = unsolbufp->nextp;
			} else {
				unsollstp->head = unsolbufp->nextp;
			}
			if (unsollstp->tail == unsolbufp)
				unsollstp->tail = unsolbufprevp;
		}

		unsollstp->num_items --;
		ret = BNXEF_SUCCESS;
	}
	mutex_exit(&unsollstp->unsol_buf_list_mutex);

	return (ret);
}

/*
 * THis will find the released buffers available for use by FCA.
 */
bnxef_unsol_buf_t *
bnxef_find_unsol_buf(bnxef_port_t *portp, uchar_t type)
{
	bnxef_unsol_buf_list_t *unsollstp;
	bnxef_unsol_buf_t *unsolbufp;
	uchar_t	ltype;
	uint32_t lflags;
	int i, found = 0;
	bnxef_driver_state_t *dsp;

	ASSERT(portp != NULL);

	dsp = portp->statep;
	ASSERT(dsp != NULL);

	unsollstp = &portp->unsol_buf_header;
	ASSERT(unsollstp != NULL);

	mutex_enter(&unsollstp->unsol_buf_list_mutex);
	unsolbufp = unsollstp->head;
	if (unsolbufp == NULL) {
		mutex_exit(&unsollstp->unsol_buf_list_mutex);
		return (NULL);
	}

	BNXEF_DEBUG(dsp, ERROR, -1, "%s%d: Given Type 0x%x, num_items 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance, type, unsollstp->num_items);
	ASSERT(unsolbufp != NULL);
	for (i = 0; i < unsollstp->num_items; i++) {
		mutex_enter(&unsolbufp->unsol_mutex);
		ltype = unsolbufp->type;
		lflags = unsolbufp->flags;
		mutex_exit(&unsolbufp->unsol_mutex);
		if ((ltype == type) &&
			((lflags & BNXEF_UB_RELEASED)
				== BNXEF_UB_RELEASED)) {
			found++;
			break;
		}
		unsolbufp = unsolbufp->nextp;
		if (unsolbufp == NULL) break;
		ASSERT(unsolbufp != NULL);
	}
	if (found) {
		mutex_enter(&unsolbufp->unsol_mutex);
		unsolbufp->flags &= ~BNXEF_UB_RELEASED;
		mutex_exit(&unsolbufp->unsol_mutex);
	}

	mutex_exit(&unsollstp->unsol_buf_list_mutex);
	return (found? unsolbufp: NULL);
}


/*
 * THis will find the non-released buffers for this exchange Id and DID.
 */
bnxef_unsol_buf_t *
bnxef_find_non_rel_unsol_buf(bnxef_port_t *portp, uint16_t ox_id, uint32_t d_id)
{
	bnxef_unsol_buf_list_t *unsollstp;
	bnxef_unsol_buf_t *unsolbufp;
	uint32_t lflags;
	int i, found = 0;
	bnxef_driver_state_t *dsp;
	fc_frame_hdr_t  *fcp;
	fc_unsol_buf_t *ubp;

	ASSERT(portp != NULL);

	dsp = portp->statep;
	ASSERT(dsp != NULL);

	unsollstp = &portp->unsol_buf_header;
	ASSERT(unsollstp != NULL);

	mutex_enter(&unsollstp->unsol_buf_list_mutex);
	unsolbufp = unsollstp->head;
	if (unsolbufp == NULL) {
		mutex_exit(&unsollstp->unsol_buf_list_mutex);
		return (NULL);
	}

	BNXEF_DEBUG(dsp, GENERAL, -1,
		"%s%d: Given ox_id 0x%x, D_id 0x%x num_items 0x%x",
		BNXEF_DRIVER_NAME, dsp->instance,
		ox_id, d_id, unsollstp->num_items);
	ASSERT(unsolbufp != NULL);
	for (i = 0; i < unsollstp->num_items; i++) {
		mutex_enter(&unsolbufp->unsol_mutex);
		lflags = unsolbufp->flags;
		mutex_exit(&unsolbufp->unsol_mutex);
		if ((lflags & BNXEF_UB_RELEASED) == 0) {
			ubp = unsolbufp->unsol_bufp;
			fcp = &ubp->ub_frame;
			BNXEF_DEBUG(dsp, GENERAL, -1,
				"%s%d: NON REL Buf 0x%p, lflags 0x%x",
				BNXEF_DRIVER_NAME,
				dsp->instance, unsolbufp, lflags);
			BNXEF_DEBUG(dsp, GENERAL, -1,
				"%s%d: Ox_id 0x%x, D_id 0x%x",
				BNXEF_DRIVER_NAME,
				dsp->instance, fcp->ox_id, fcp->s_id);
			if ((fcp->ox_id == ox_id) && (fcp->s_id == d_id)) {
				found++;
				break;
			}
		}
		unsolbufp = unsolbufp->nextp;
		if (unsolbufp == NULL) break;
		ASSERT(unsolbufp != NULL);
	}
	mutex_exit(&unsollstp->unsol_buf_list_mutex);
	return (found? unsolbufp: NULL);
}


int
bnxef_get_active_ub_count(bnxef_port_t *portp)
{
	bnxef_unsol_buf_list_t *unsollstp;
	bnxef_unsol_buf_t *unsolbufp;
	uint32_t lflags;
	int i, found = 0;
	bnxef_driver_state_t *dsp;

	ASSERT(portp != NULL);

	dsp = portp->statep;
	ASSERT(dsp != NULL);

	unsollstp = &portp->unsol_buf_header;
	ASSERT(unsollstp != NULL);

	mutex_enter(&unsollstp->unsol_buf_list_mutex);
	unsolbufp = unsollstp->head;
	if (unsolbufp == NULL) {
		mutex_exit(&unsollstp->unsol_buf_list_mutex);
		return (-1);
	}

	ASSERT(unsolbufp != NULL);
	for (i = 0; i < unsollstp->num_items; i++) {
		mutex_enter(&unsolbufp->unsol_mutex);
		lflags = unsolbufp->flags;
		mutex_exit(&unsolbufp->unsol_mutex);
		if ((lflags & BNXEF_UB_RELEASED) == 0) {
			found++;
		}
		unsolbufp = unsolbufp->nextp;
		if (unsolbufp == NULL) break;
		ASSERT(unsolbufp != NULL);
	}
	mutex_exit(&unsollstp->unsol_buf_list_mutex);
	return (found);
}
