/*
 * The information contained in this file is confidential and proprietary to
 * Broadcom Corporation.  No part of this file may be reproduced or
 * distributed, in any form or by any means for any purpose, without the
 * express written permission of Broadcom Corporation.
 *
 * (c) COPYRIGHT 2010-2012 Broadcom Corporation, ALL RIGHTS RESERVED.
 */

#pragma ident "(C) 2010-2012 Broadcom Corporation $File$$Revision$$Change$$DateTime$"

#include <sys/mdb_modapi.h>
#include <sys/sysinfo.h>
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/stat.h>
#include <sys/pci.h>
#include <sys/sunddi.h>
#include <sys/modctl.h>
#include <sys/cmn_err.h>
#include <sys/file.h>
#include <sys/cred.h>
#include <sys/byteorder.h>
#include <sys/atomic.h>
#include <sys/scsi/scsi.h>
#include <sys/modhash.h>
#include <sys/sunldi.h>
#include <sys/kstat.h>
#include <sys/kmem.h>
#include <sys/ethernet.h>
#include <sys/stream.h>
#include <sys/sunldi.h>

#include <sys/fibre-channel/fc_types.h>
#include <sys/fibre-channel/impl/fc_fcaif.h>
#include <bnxe_binding.h>
#include <bnxef_tq_param.h>
#include <bnxef_fcf_info.h>
#include <bnxef_hsi.h>
#include <bnxef_fcoe_task.h>
#include <bnxef_fcoe_session.h>
#include <bnxef_state.h>
#include <bnxef_fip.h>
#include <bnxef_fcoe.h>
#include <bnxef_debug.h>

static int
bnxef_walk_task_free_list_init(mdb_walk_state_t *wsp)
{
	bnxef_driver_state_t	*dsp;

	if (wsp->walk_addr == NULL) {
		mdb_warn("Address should be given");
		return (WALK_ERR);
	}
	mdb_printf("Init Routine is called Walk address 0x%p\n",
		wsp->walk_addr);

	dsp = mdb_zalloc(sizeof (bnxef_driver_state_t), UM_SLEEP);
	if (dsp == NULL) {
		mdb_printf("Error in allocation for driver state\n");
		return (WALK_ERR);
	}

	if (mdb_vread(dsp, sizeof (bnxef_driver_state_t), wsp->walk_addr)
				== sizeof (bnxef_driver_state_t)) {
		if (dsp->magic == BNXEF_DRIVER_MAGIC) {
			mdb_printf("The Num Items in the list 0x%x\n",
				dsp->bnxef_task_free_list.num_items);
			mdb_printf("The head on the free list 0x%p\n",
				dsp->bnxef_task_free_list.head);
			if (dsp->bnxef_task_free_list.num_items > 0) {
				wsp->walk_addr =
				(uintptr_t)dsp->bnxef_task_free_list.head;
				mdb_printf("Walk Address 0x%p\n",
					wsp->walk_addr);
			} else {
				mdb_printf("List empty - Not walking\n");
				mdb_free(dsp, sizeof (bnxef_driver_state_t));
				return (WALK_DONE);
			}
		} else
			mdb_warn("Address is not Ptr to Driver State");
	}


	mdb_free(dsp, sizeof (bnxef_driver_state_t));
	wsp->walk_data = mdb_alloc(sizeof (bnxef_fcoe_task_t), UM_SLEEP);
	return (WALK_NEXT);
}

static int
bnxef_walk_task_free_list_step(mdb_walk_state_t *wsp)
{
	int status;

	if (wsp->walk_addr == NULL)
		return (WALK_DONE);

	if (mdb_vread(wsp->walk_data, sizeof (bnxef_fcoe_task_t),
		wsp->walk_addr) == -1) {
		mdb_warn("Failed to read the bnxef_fcoe_task_t structure at %p",
			wsp->walk_addr);
		return (WALK_DONE);
	}

	status = wsp->walk_callback(wsp->walk_addr, wsp->walk_data,
			wsp->walk_cbdata);

	wsp->walk_addr =
		(uintptr_t)(((bnxef_fcoe_task_t *)wsp->walk_data)->nextp);
	mdb_printf("Next Address 0x%p\n", wsp->walk_addr);
	return (status);
}


static void
bnxef_walk_task_free_list_fini(mdb_walk_state_t *wsp)
{

	mdb_free(wsp->walk_data, sizeof (bnxef_fcoe_task_t));
}

static int
bnxef_walk_session_list_init(mdb_walk_state_t *wsp)
{
	if (wsp->walk_addr == NULL) {
		return (WALK_NEXT);
	}

	wsp->walk_data = mdb_alloc(sizeof (bnxef_session_t), UM_SLEEP);
	return (WALK_NEXT);
}

static int
bnxef_walk_session_list_step(mdb_walk_state_t *wsp)
{
	int status;

	if (wsp->walk_addr == NULL)
		return (WALK_DONE);

	if (mdb_vread(wsp->walk_data, sizeof (bnxef_session_t),
		wsp->walk_addr) == -1) {
		mdb_warn("Failed to read the bnxef_session_t structure at %p",
			wsp->walk_addr);
		return (WALK_DONE);
	}

	status = wsp->walk_callback(wsp->walk_addr, wsp->walk_data,
			wsp->walk_cbdata);

	wsp->walk_addr =
		(uintptr_t)(((bnxef_session_t *)wsp->walk_data)->next);
	return (status);
}


static void
bnxef_walk_session_list_fini(mdb_walk_state_t *wsp)
{
	if (wsp->walk_addr != NULL)
		mdb_free(wsp->walk_data, sizeof (bnxef_session_t));
}

static int
bnxef_walk_session_task_list_init(mdb_walk_state_t *wsp)
{
	if (wsp->walk_addr == NULL)
		return (WALK_NEXT);
	wsp->walk_data = mdb_alloc(sizeof (bnxef_fcoe_task_t), UM_SLEEP);
	return (WALK_NEXT);
}

static int
bnxef_walk_session_task_list_step(mdb_walk_state_t *wsp)
{
	int status;

	if (wsp->walk_addr == NULL)
		return (WALK_DONE);
	if (mdb_vread(wsp->walk_data, sizeof (bnxef_fcoe_task_t),
		wsp->walk_addr) == -1) {
		mdb_warn("Dauked to read the task structure at %p\n",
			wsp->walk_addr);
		return (WALK_DONE);
	}

	status = wsp->walk_callback(wsp->walk_addr, wsp->walk_data,
			wsp->walk_cbdata);

	wsp->walk_addr =
		(uintptr_t)(((bnxef_fcoe_task_t *)wsp->walk_data)->snextp);
	return (status);
}

static void
bnxef_walk_session_task_list_fini(mdb_walk_state_t *wsp)
{

	if (wsp->walk_addr != NULL)
		mdb_free(wsp->walk_data, sizeof (bnxef_fcoe_task_t));
}

/*
 * The following are the walkers written to validate the functionality.
 */
mdb_walker_t bnxef_walkers[] = {
    {
        "bnxef_task_free_list_walker",
        "Walk the list of free task structures",
        bnxef_walk_task_free_list_init,
        bnxef_walk_task_free_list_step,
        bnxef_walk_task_free_list_fini
    },
    {
        "bnxef_session_list_walker",
        "Walk the session list maintained in hash buckets",
        bnxef_walk_session_list_init,
        bnxef_walk_session_list_step,
        bnxef_walk_session_list_fini
    },
    {
        "bnxef_session_task_list_walker",
        "Walk the tasks pending on a session",
        bnxef_walk_session_task_list_init,
        bnxef_walk_session_task_list_step,
        bnxef_walk_session_task_list_fini
    },
    {
        NULL
    }
};

